@@ -8,29 +8,21 @@
 
 @implementation HBAdvancedController
 
-- (id)init
+- (instancetype)init
 {
-    [super init];
-    [self loadMyNibFile];
-    
+    self = [super initWithNibName:@"AdvancedView" bundle:nil];
+    if (self)
+    {
+
+    }
+
     return self;
 }
 
-- (void) setView: (NSBox *) box
+- (void)loadView
 {
-    fOptionsBox = box;
-    [fOptionsBox setContentView:fX264optView];
-}
-
-- (BOOL) loadMyNibFile
-{
-    if(![NSBundle loadNibNamed:@"AdvancedView" owner:self])
-    {
-        NSLog(@"Warning! Could not load myNib file.\n");
-        return NO;
-    }
-    
-    return YES;
+    [super loadView];
+    [self setHidden:NO];
 }
 
 - (NSString *) optionsString
@@ -38,25 +30,15 @@
     return [fDisplayX264Options stringValue];
 }
 
-- (NSString *) optionsStringLavc
-{
-    return [fDisplayLavcOptions stringValue];
-}
-
 - (void) setOptions: (NSString *)string
 {
     [fDisplayX264Options setStringValue:string];
     [self X264AdvancedOptionsSet:nil];
 }
 
-- (void) setLavcOptions: (NSString *)string
-{
-    [fDisplayLavcOptions setStringValue:string];
-}
-
 - (void) setHidden: (BOOL) hide
 {
-    if(hide)
+    if (hide)
     {
         [fOptionsBox setContentView:fEmptyView];
     }
@@ -67,24 +49,7 @@
     return;
 }
 
-- (void) setLavcOptsEnabled: (BOOL) lavc
-{
-    if(lavc)
-    {
-        [fDisplayLavcOptions setHidden:NO];
-        [fDisplayLavcOptionsLabel setHidden:NO];
-        [fDisplayTheoraOptionsLabel setHidden:YES];
-    }
-    else
-    {
-        [fDisplayLavcOptions setHidden:YES];
-        [fDisplayLavcOptionsLabel setHidden:YES];
-        [fDisplayTheoraOptionsLabel setHidden:NO];
-    }
-    return;
-}
-
- - (void) enableUI: (bool) b
+ - (void)setUIEnabled:(BOOL)flag
 {
     unsigned i;
     NSControl * controls[] =
@@ -105,16 +70,14 @@
             NSTextField * tf = (NSTextField *) controls[i];
             if( ![tf isBezeled] )
             {
-                [tf setTextColor: b ? [NSColor controlTextColor] :
+                [tf setTextColor: flag ? [NSColor controlTextColor] :
                     [NSColor disabledControlTextColor]];
                 continue;
             }
         }
-        [controls[i] setEnabled: b];
+        [controls[i] setEnabled: flag];
 
     }
-    
-    [fX264optView setWantsLayer:YES];
 }
 
 - (void)dealloc
@@ -381,8 +344,8 @@
         currentOptsArray = [currentOptString componentsSeparatedByString:@":"];
         
         /*iterate through the array and get <opts> and <values*/
-        int loopcounter;
-        int currentOptsArrayCount = [currentOptsArray count];
+        NSUInteger loopcounter;
+        NSUInteger currentOptsArrayCount = [currentOptsArray count];
         for (loopcounter = 0; loopcounter < currentOptsArrayCount; loopcounter++)
         {
             thisOpt = [currentOptsArray objectAtIndex:loopcounter];
@@ -699,8 +662,8 @@
         currentOptsArray = [currentOptString componentsSeparatedByString:@":"];
         
         /*iterate through the array and get <opts> and <values*/
-        int loopcounter;
-        int currentOptsArrayCount = [currentOptsArray count];
+        NSUInteger loopcounter;
+        NSUInteger currentOptsArrayCount = [currentOptsArray count];
         for (loopcounter = 0; loopcounter < currentOptsArrayCount; loopcounter++)
         {
             thisOpt = [currentOptsArray objectAtIndex:loopcounter];
@@ -1033,7 +996,7 @@
         {
             /* Otherwise the format is deblock=a,b, where a and b both have an array
                offset of 7 because deblocking values start at -6 instead of at zero. */
-            thisOpt = [NSString stringWithFormat:@"%@=%d,%d",optName, ([fX264optAlphaDeblockPopUp indexOfSelectedItem] != 0) ? [fX264optAlphaDeblockPopUp indexOfSelectedItem]-7 : 0,([fX264optBetaDeblockPopUp indexOfSelectedItem] != 0) ? [fX264optBetaDeblockPopUp indexOfSelectedItem]-7 : 0];
+            thisOpt = [NSString stringWithFormat:@"%@=%ld,%ld",optName, ([fX264optAlphaDeblockPopUp indexOfSelectedItem] != 0) ? [fX264optAlphaDeblockPopUp indexOfSelectedItem]-7 : 0,([fX264optBetaDeblockPopUp indexOfSelectedItem] != 0) ? [fX264optBetaDeblockPopUp indexOfSelectedItem]-7 : 0];
         }
     }
     
@@ -1208,19 +1171,19 @@
         /* Motion estimation range uses an odd array offset because in addition
            to starting with index 0 as default, index 1 starts at 4 instead of 1,
            because merange can't go below 4. So it has to be handled separately.  */
-        thisOpt = [NSString stringWithFormat:@"%@=%d",optName,[sender indexOfSelectedItem]+3];
+        thisOpt = [NSString stringWithFormat:@"%@=%ld",optName,[sender indexOfSelectedItem]+3];
     }
     
     else if ([optName isEqualToString:@"b-adapt"])
     {
         /* B-adapt starts at index 0 with default then goes 0, 1, 2)*/
-        thisOpt = [NSString stringWithFormat:@"%@=%d", optName, [sender indexOfSelectedItem]-1];
+        thisOpt = [NSString stringWithFormat:@"%@=%ld", optName, [sender indexOfSelectedItem]-1];
     }
     
     else if ([optName isEqualToString:@"ref"])
     {
         /* Refs use actual index numbers */
-        thisOpt = [NSString stringWithFormat:@"%@=%d",optName,[sender indexOfSelectedItem]];
+        thisOpt = [NSString stringWithFormat:@"%@=%ld",optName,(long)[sender indexOfSelectedItem]];
     }
     
     else // we have a valid value to change, so change it
@@ -1230,7 +1193,7 @@
            Basically, any options that are PopUp menus with index 0 as default and
            index 1 as 0, with numerical values, are all handled right here. All of
            the above stuff is for the exceptions to the general case.              */
-            thisOpt = [NSString stringWithFormat:@"%@=%d",optName,[sender indexOfSelectedItem]-1];
+            thisOpt = [NSString stringWithFormat:@"%@=%ld",optName,[sender indexOfSelectedItem]-1];
     }
     
     return thisOpt;
@@ -1271,7 +1234,6 @@
     
     NSString * thisOpt = @"";  // The separated option such as "bframes=3"
     NSString * optName = @"";  // The option name such as "bframes"
-    NSString * optValue = @""; // The option value such as "3"
     NSArray *currentOptsArray;
     
     /* Get the current opt string being displayed. */
@@ -1297,8 +1259,8 @@
         currentOptsArray = [currentOptString componentsSeparatedByString:@":"];
         
         /* Iterate through the array and get <opts> and <values*/
-        int loopcounter;
-        int currentOptsArrayCount = [currentOptsArray count];
+        NSUInteger loopcounter;
+        NSUInteger currentOptsArrayCount = [currentOptsArray count];
         for (loopcounter = 0; loopcounter < currentOptsArrayCount; loopcounter++)
         {
             thisOpt = [currentOptsArray objectAtIndex:loopcounter];
@@ -1310,7 +1272,6 @@
                    already have at least one option=value pair in them. */
                    
                 optName = [thisOpt substringToIndex:splitOptRange.location];
-                optValue = [thisOpt substringFromIndex:splitOptRange.location + 1];
 
                 /*If the optNameToChange is found, appropriately change the value or delete it if
                     "Unspecified" is set.*/
