@@ -8,6 +8,7 @@
 ###############################################################################
 
 import fnmatch
+import glob
 import optparse
 import os
 import platform
@@ -99,12 +100,6 @@ class Configure( object ):
         cfg.infof( 'compute: makevar BUILD/  = %s\n', self.build_final )
         cfg.infof( 'compute: makevar PREFIX/ = %s\n', self.prefix_final )
 
-        ## xcode does a chdir so we need appropriate values
-        macosx = os.path.join( self.src_dir, 'macosx' )
-        self.xcode_x_src    = self._final_dir( macosx, self.src_dir )
-        self.xcode_x_build  = self._final_dir( macosx, self.build_dir )
-        self.xcode_x_prefix = self._final_dir( macosx, self.prefix_dir )
-
     ## perform chdir and enable log recording
     def chdir( self ):
         if os.path.abspath( self.build_dir ) == os.path.abspath( self.src_dir ):
@@ -252,6 +247,7 @@ class Action( object ):
                 self._dumpSession( cfg.infof )
                 cfg.errln( 'unable to continue' )
             self._dumpSession( cfg.verbosef )
+            self._failSession()
         else:
             cfg.infof( '(%s) %s\n', self.msg_pass, self.msg_end )
             self._dumpSession( cfg.verbosef )
@@ -266,6 +262,9 @@ class Action( object ):
     def _parseSession( self ):
         pass
 
+    def _failSession( self ):
+        pass
+
     def run( self ):
         if self.run_done:
             return
@@ -402,8 +401,10 @@ class LDProbe( Action ):
 ##
 ## example results from various platforms:
 ##
-##   i386-apple-darwin9.6.0     (Mac OS X 10.5.6 Intel)
 ##   powerpc-apple-darwin9.6.0  (Mac OS X 10.5.6 PPC)
+##   i386-apple-darwin9.6.0     (Mac OS X 10.5.6 Intel)
+##   x86_64-apple-darwin10.8.0  (Mac OS X 10.6.8 Intel)
+##   x86_64-apple-darwin11.2.0  (Mac OS X 10.7.2 Intel)
 ##   i686-pc-cygwin             (Cygwin, Microsoft Vista)
 ##   x86_64-unknown-linux-gnu   (Linux, Fedora 10 x86_64)
 ##
@@ -553,6 +554,9 @@ class ArchAction( Action ):
         ## some match on system should be made here; otherwise we signal a warning. 
         if host.match( '*-*-cygwin*' ):
             pass
+        elif host.match( '*-*-darwin11.*' ):
+            self.mode['i386']   = 'i386-apple-darwin%s'      % (host.release)
+            self.mode['x86_64'] = 'x86_64-apple-darwin%s'    % (host.release)
         elif host.match( '*-*-darwin*' ):
             self.mode['i386']   = 'i386-apple-darwin%s'      % (host.release)
             self.mode['x86_64'] = 'x86_64-apple-darwin%s'    % (host.release)
@@ -570,6 +574,8 @@ class ArchAction( Action ):
                 self.mode.mode = 'ppc64'
         elif host.match( '*-*-linux*' ):
             pass
+        elif host.match( '*-*-solaris*' ):
+            pass
         else:
             self.msg_pass = 'WARNING'
 
@@ -628,20 +634,24 @@ class CoreProbe( Action ):
 class SelectMode( dict ):
     def __init__( self, descr, *modes, **kwargs ):
         super( SelectMode, self ).__init__( modes )
-        self.descr    = descr
-        self.modes    = modes
-        self.default  = kwargs.get('default',modes[0][0])
-        self.mode     = self.default
+        self.descr = descr
+        self.modes = modes
+        self.what  = kwargs.get('what',' mode')
+        if modes:
+            self.default = kwargs.get('default',modes[0][0])
+        else:
+            self.default = None
+        self.mode = self.default
 
     def cli_add_option( self, parser, option ):
         parser.add_option( option, default=self.mode, metavar='MODE',
-            help='select %s mode: %s' % (self.descr,self.toString()),
+            help='select %s%s: %s' % (self.descr,self.what,self.toString()),
             action='callback', callback=self.cli_callback, type='str' )
 
     def cli_callback( self, option, opt_str, value, parser, *args, **kwargs ):
         if value not in self:
-            raise optparse.OptionValueError( 'invalid %s mode: %s (choose from %s)'
-                % (self.descr,value,self.toString( True )) )
+            raise optparse.OptionValueError( 'invalid %s%s: %s (choose from: %s)'
+                % (self.descr,self.what,value,self.toString( True )) )
         self.mode = value
 
     def toString( self, nodefault=False ):
@@ -671,13 +681,29 @@ class SelectMode( dict ):
 ##
 class RepoProbe( ShellProbe ):
     def __init__( self ):
-        super( RepoProbe, self ).__init__( 'svn info', 'svn info %s' % (cfg.src_dir) )
+        svn = 'svn'
+
+        ## Possible the repo was created using an incompatible version than what is
+        ## available in PATH when probe runs. Workaround by checking for file
+        ## .svn/HANDBRAKE_REPO_PROBE which points to a preferred svn executable.
+        try:
+            hrp = os.path.join( cfg.src_dir, '.svn', 'HANDBRAKE_REPO_PROBE' )
+            if os.path.isfile( hrp ) and os.path.getsize( hrp ) > 0:
+                file = cfg.open( hrp, 'r' )
+                line = file.readline().strip()
+                file.close()
+                if line:
+                    svn = line
+        except:
+            pass
+
+        super( RepoProbe, self ).__init__( 'svn info', '%s info %s' % (svn,cfg.src_dir) )
 
         self.url       = 'svn://nowhere.com/project/unknown'
         self.root      = 'svn://nowhere.com/project'
         self.branch    = 'unknown'
         self.uuid      = '00000000-0000-0000-0000-000000000000';
-        self.rev       = 3736
+        self.rev       = 0
         self.date      = '0000-00-00 00:00:00 -0000'
         self.official  = 0
         self.type      = 'unofficial'
@@ -721,6 +747,30 @@ class RepoProbe( ShellProbe ):
 
         self.msg_end = self.url
 
+    def _failSession( self ):
+        # Look for svn info in version file.
+        #
+        # Version file would be created manually by source packager.
+        # e.g.
+        # $ svn info HandBrake > HandBrake/version.txt
+        # $ tar -czf handbrake-source.tgz --exclude .svn HandBrake
+        cfg.infof( 'probe: version.txt...' )
+        try:
+            hvp = os.path.join( cfg.src_dir, 'version.txt' )
+            if os.path.isfile( hvp ) and os.path.getsize( hvp ) > 0:
+                file = open( hvp, 'r' )
+                self.session = file.readlines()
+                file.close()
+                if self.session:
+                    self._parseSession()
+            if self.rev != 0:
+                cfg.infof( '(pass)\n' )
+            else:
+                cfg.infof( '(fail)\n' )
+
+        except:
+            cfg.infof( '(fail)\n' )
+
 ###############################################################################
 ##
 ## project object.
@@ -735,16 +785,16 @@ class Project( Action ):
         self.name          = 'HandBrake'
         self.acro_lower    = 'hb'
         self.acro_upper    = 'HB'
-        self.url_website   = 'http://handbrake.fr'
-        self.url_community = 'http://forum.handbrake.fr'
+        self.url_website   = 'https://handbrake.fr'
+        self.url_community = 'https://forum.handbrake.fr'
         self.url_irc       = 'irc://irc.freenode.net/handbrake'
 
         self.name_lower = self.name.lower()
         self.name_upper = self.name.upper()
 
         self.vmajor = 0
-        self.vminor = 9
-        self.vpoint = 5
+        self.vminor = 10
+        self.vpoint = 0
 
     def _action( self ):
         ## add architecture to URL only for Mac
@@ -760,7 +810,7 @@ class Project( Action ):
             self.build = time.strftime('%Y%m%d') + '00'
             self.title = '%s %s (%s)' % (self.name,self.version,self.build)
         elif repo.type == 'developer':
-            self.version = 'svn%d' % (repo.rev)
+            self.version = '%dsvn' % (repo.rev)
             url_ctype = '_unstable'
             url_ntype = 'unstable'
             self.build = time.strftime('%Y%m%d') + '01'
@@ -772,8 +822,8 @@ class Project( Action ):
             self.build = time.strftime('%Y%m%d') + '99'
             self.title = '%s rev%d (%s)' % (self.name,repo.rev,self.build)
 
-        self.url_appcast = 'http://handbrake.fr/appcast%s%s.xml' % (url_ctype,url_arch)
-        self.url_appnote = 'http://handbrake.fr/appcast/%s.html' % (url_ntype)
+        self.url_appcast = 'https://handbrake.fr/appcast%s%s.xml' % (url_ctype,url_arch)
+        self.url_appnote = 'https://handbrake.fr/appcast/%s.html' % (url_ntype)
 
         self.msg_end = '%s (%s)' % (self.name,repo.type)
         self.fail = False
@@ -796,6 +846,7 @@ class ToolProbe( Action ):
         self.name = self.names[0]
         self.pretext = self.name
         self.pathname = self.names[0]
+        self.minversion = kwargs.get('minversion', None)
 
     def _action( self ):
         self.session = []
@@ -810,6 +861,8 @@ class ToolProbe( Action ):
                 break
         if self.fail:
             self.msg_end = 'not found'
+        elif self.minversion:
+            self.version = VersionProbe( [self.pathname, '--version'], minversion=self.minversion )
 
     def cli_add_option( self, parser ):
         parser.add_option( '--'+self.name, metavar='PROG',
@@ -825,6 +878,93 @@ class ToolProbe( Action ):
 
 ###############################################################################
 
+###############################################################################
+##
+## version probe: passes --version to command and only cares about first line
+## of output. If probe fails, a default version of '0.0.0' results.
+## The default rexpr is useful for some very simple version strings. A Custom
+## expression would be required for more complex version strings.
+##
+## command = full command and arguments to pipe
+## rexpr   = a regular expression which must return named subgroups:
+##              name: mandatory. The tool name.
+##              svers: mandatory. The whole version tuple to be represented as string.
+##              i0: mandatory. First element of version tuple to be parsed as int.
+##              i1: optional. Second element of version tuple to be parsed as int.
+##              i2: optional. Third element of version tuple to be parsed as int.
+##           All matching is case-insensitive.
+## abort   = if true configure will exit on probe fail
+## session = result. array of lines (stdout/stderr) from command
+## fail    = result. true if probe failed
+## svers   = result. string of version tuple
+## ivers   = result. int[3] of version tuple
+##
+class VersionProbe( Action ):
+    def __init__( self, command, minversion=None, rexpr=None, abort=False ):
+        super( VersionProbe, self ).__init__( 'version probe', os.path.basename(command[0]), abort )
+        self.command = command
+        self.minversion = minversion
+        if not rexpr:
+            rexpr = '(?P<name>[^.]+)\s+(?P<svers>(?P<i0>\d+)(\.(?P<i1>\d+))?(\.(?P<i2>\d+))?)'
+        self.rexpr = rexpr
+
+    def _action( self ):
+        ## pipe and redirect stderr to stdout; effects communicate result
+        pipe = subprocess.Popen( self.command, stdout=subprocess.PIPE, stderr=subprocess.STDOUT )
+
+        ## read data into memory buffers
+        data = pipe.communicate()
+        self.fail = pipe.returncode != 0
+
+        if data[0]:
+            self.session = data[0].splitlines()
+        else:
+            self.session = []
+
+        self.svers = '0.0.0'
+        self.ivers = [0,0,0]
+
+        try:
+            if not self.fail and self.session and len(self.session):
+                self.fail = True
+                self._parse()
+            self.fail = False
+            self.msg_end = self.svers
+        except Exception, x:
+            self.svers = '0.0.0'
+            self.ivers = [0,0,0]
+            self.msg_end = str(x)
+
+    def _dumpSession( self, printf ):
+        printf( '  + %s\n', ' '.join(self.command) )
+        super( VersionProbe, self )._dumpSession( printf )
+
+    def _parse( self ):
+        mo = re.match( self.rexpr, self.session[0], re.IGNORECASE )
+        md = mo.groupdict()
+        self.svers = md['svers']
+        if 'i0' in md and md['i0']:
+            self.ivers[0] = int(md['i0'])
+        if 'i1' in md and md['i1']:
+            self.ivers[1] = int(md['i1'])
+        if 'i2' in md and md['i2']:
+            self.ivers[2] = int(md['i2'])
+
+    def inadequate( self ):
+        if not self.minversion:
+            return False
+        return self.lesser( self.minversion )
+
+    def lesser( self, ivers ):
+        for i in range(0,3):
+            if self.ivers[i] < ivers[i]:
+                return True
+            elif self.ivers[i] > ivers[i]:
+                return False
+        return False
+
+###############################################################################
+
 class SelectTool( Action ):
     selects = []
 
@@ -865,7 +1005,7 @@ class SelectTool( Action ):
                 self.run()
                 break
         if not found:
-            raise optparse.OptionValueError( 'invalid %s mode: %s (choose from %s)'
+            raise optparse.OptionValueError( 'invalid %s mode: %s (choose from: %s)'
                 % (self.name,value,self.toString( True )) )
 
     def doc_add( self, doc ):
@@ -1003,8 +1143,8 @@ class Option( optparse.Option ):
     conf_args = []
 
     def _conf_record( self, opt, value ):
-        ## skip conf,force,launch
-        if re.match( '^--(conf|force|launch).*$', opt ):
+        ## filter out non-applicable options
+        if re.match( '^--(force|launch).*$', opt ):
             return
 
         ## remove duplicates (last duplicate wins)
@@ -1030,13 +1170,13 @@ def createCLI():
     cli.description += 'Configure %s build system.' % (project.name)
 
     ## add hidden options
-    cli.add_option( '--conf-method', default='terminal', action='store', help=optparse.SUPPRESS_HELP )
+    cli.add_option( '--xcode-driver', default='bootstrap', action='store', help=optparse.SUPPRESS_HELP )
     cli.add_option( '--force', default=False, action='store_true', help='overwrite existing build config' )
     cli.add_option( '--verbose', default=False, action='store_true', help='increase verbosity' )
 
     ## add install options
     grp = OptionGroup( cli, 'Directory Locations' )
-    h = IfHost( 'specify sysroot (e.g. for Leopard builds from Snow Leapard)', '*-*-darwin*', none=optparse.SUPPRESS_HELP ).value
+    h = IfHost( 'specify sysroot of SDK for Xcode builds', '*-*-darwin*', none=optparse.SUPPRESS_HELP ).value
     grp.add_option( '--sysroot', default=None, action='store', metavar='DIR',
         help=h )
     grp.add_option( '--src', default=cfg.src_dir, action='store', metavar='DIR',
@@ -1059,11 +1199,25 @@ def createCLI():
     grp.add_option( '--disable-gtk-update-checks', default=False, action='store_true', help=h )
     h = IfHost( 'enable GTK GUI (mingw)', '*-*-mingw*', none=optparse.SUPPRESS_HELP ).value
     grp.add_option( '--enable-gtk-mingw', default=False, action='store_true', help=h )
-    h = IfHost( 'enable use of ffmpeg mpeg2 decoding', '*-*-*', none=optparse.SUPPRESS_HELP ).value
-    grp.add_option( '--enable-ff-mpeg2', default=False, action='store_true', help=h )
+    h = IfHost( 'disable gstreamer (live preview)', '*-*-linux*', none=optparse.SUPPRESS_HELP ).value
+    grp.add_option( '--disable-gst', default=False, action='store_true', help=h )
+    h = IfHost( 'enable use of Intel Quick Sync Video hardware acceleration', '*-*-*', none=optparse.SUPPRESS_HELP ).value
+    grp.add_option( '--enable-qsv', default=False, action='store_true', help=h )
 
-    h = IfHost( 'disable Xcode', '*-*-darwin*', none=optparse.SUPPRESS_HELP ).value
-    grp.add_option( '--disable-xcode', default=False, action='store_true', help=h )
+    h = IfHost( 'enable HWD features', '*-*-*', none=optparse.SUPPRESS_HELP ).value
+    grp.add_option( '--enable-hwd', default=False, action='store_true', help=h )
+    
+    h = IfHost( 'enable use of x265 encoding', '*-*-*', none=optparse.SUPPRESS_HELP ).value
+    grp.add_option( '--enable-x265', default=True, action='store_true', help=h )
+    grp.add_option( '--disable-x265', dest="enable_x265", action='store_false' )
+
+    h = IfHost( 'enable use of fdk-aac encoder', '*-*-*', none=optparse.SUPPRESS_HELP ).value
+    grp.add_option( '--enable-fdk-aac', dest="enable_fdk_aac", default=not host.match( '*-*-darwin*' ), action='store_true', help=h )
+    grp.add_option( '--disable-fdk-aac', dest="enable_fdk_aac", action='store_false' )
+
+    h = IfHost( 'enable use of libav aac encoder', '*-*-*', none=optparse.SUPPRESS_HELP ).value
+    grp.add_option( '--enable-libav-aac', dest="enable_libav_aac", default=not host.match( '*-*-darwin*' ), action='store_true', help=h )
+    grp.add_option( '--disable-libav-aac', dest="enable_libav_aac", action='store_false' )
 
     cli.add_option_group( grp )
 
@@ -1086,11 +1240,31 @@ def createCLI():
     arch.mode.cli_add_option( grp, '--arch' )
     grp.add_option( '--cross', default=None, action='store', metavar='SPEC',
         help='specify GCC cross-compilation spec' )
-    h = IfHost( 'Min OS X Version', '*-*-darwin*', none=optparse.SUPPRESS_HELP ).value
+    h = IfHost( 'specify Mac OS X deployment target for Xcode builds', '*-*-darwin*', none=optparse.SUPPRESS_HELP ).value
     grp.add_option( '--minver', default=None, action='store', metavar='VER',
         help=h )
+
+    h = IfHost( 'Build and use local yasm', '*-*-*', none=optparse.SUPPRESS_HELP ).value
+    grp.add_option( '--enable-local-yasm', default=False, action='store_true', help=h )
+    h = IfHost( 'Build and use local autotools', '*-*-*', none=optparse.SUPPRESS_HELP ).value
+    grp.add_option( '--enable-local-autotools', default=False, action='store_true', help=h )
+    h = IfHost( 'Build and use local cmake', '*-*-*', none=optparse.SUPPRESS_HELP ).value
+    grp.add_option( '--enable-local-cmake', default=False, action='store_true', help=h )
+    h = IfHost( 'Build and use local pkg-config', '*-*-darwin*', none=optparse.SUPPRESS_HELP ).value
+    grp.add_option( '--enable-local-pkgconfig', default=False, action='store_true', help=h )
+
     cli.add_option_group( grp )
 
+    ## add Xcode options
+    if host.match( '*-*-darwin*' ):
+        grp = OptionGroup( cli, 'Xcode Options' )
+        grp.add_option( '--disable-xcode', default=False, action='store_true',
+            help='disable Xcode' )
+        grp.add_option( '--xcode-symroot', default='xroot', action='store', metavar='DIR',
+            help='specify root of the directory hierarchy that contains product files and intermediate build files' )
+        xcconfigMode.cli_add_option( grp, '--xcode-config' )
+        cli.add_option_group( grp )
+
     ## add tool locations
     grp = OptionGroup( cli, 'Tool Basenames and Locations' )
     for tool in ToolProbe.tools:
@@ -1236,15 +1410,20 @@ try:
         else:
             gmake = ToolProbe( 'GMAKE.exe', 'gmake', 'make' )
 
-        m4     = ToolProbe( 'M4.exe',     'm4' )
-        mkdir  = ToolProbe( 'MKDIR.exe',  'mkdir' )
-        patch  = ToolProbe( 'PATCH.exe',  'gpatch', 'patch' )
-        rm     = ToolProbe( 'RM.exe',     'rm' )
-        ranlib = ToolProbe( 'RANLIB.exe', 'ranlib' )
-        strip  = ToolProbe( 'STRIP.exe',  'strip' )
-        tar    = ToolProbe( 'TAR.exe',    'gtar', 'tar' )
-        wget   = ToolProbe( 'WGET.exe',   'wget', abort=False )
-        yasm   = ToolProbe( 'YASM.exe',   'yasm', abort=False )
+        m4       = ToolProbe( 'M4.exe',       'm4' )
+        mkdir    = ToolProbe( 'MKDIR.exe',    'mkdir' )
+        patch    = ToolProbe( 'PATCH.exe',    'gpatch', 'patch' )
+        rm       = ToolProbe( 'RM.exe',       'rm' )
+        ranlib   = ToolProbe( 'RANLIB.exe',   'ranlib' )
+        strip    = ToolProbe( 'STRIP.exe',    'strip' )
+        tar      = ToolProbe( 'TAR.exe',      'gtar', 'tar' )
+        wget     = ToolProbe( 'WGET.exe',     'wget', abort=False )
+        yasm     = ToolProbe( 'YASM.exe',     'yasm', abort=False, minversion=[1,2,0] )
+        autoconf = ToolProbe( 'AUTOCONF.exe', 'autoconf', abort=False )
+        automake = ToolProbe( 'AUTOMAKE.exe', 'automake', abort=False )
+        cmake    = ToolProbe( 'CMAKE.exe',    'cmake', abort=False )
+        libtool  = ToolProbe( 'LIBTOOL.exe',  'libtool', abort=False )
+        pkgconfig = ToolProbe( 'PKGCONFIG.exe', 'pkg-config', abort=False )
 
         xcodebuild = ToolProbe( 'XCODEBUILD.exe', 'xcodebuild', abort=False )
         lipo       = ToolProbe( 'LIPO.exe',       'lipo', abort=False )
@@ -1260,6 +1439,19 @@ try:
     debugMode = SelectMode( 'debug', ('none','none'), ('min','min'), ('std','std'), ('max','max') )
     optimizeMode = SelectMode( 'optimize', ('none','none'), ('speed','speed'), ('size','size'), default='speed' )
 
+    ## find xcconfig values
+    xcconfigMode = SelectMode( 'xcconfig', ('none',None), what='' )
+    if host.match( '*-*-darwin*' ):
+        for xc in glob.glob( os.path.join(cfg.dir, '../macosx/xcconfig/*.xcconfig') ):
+            bname = os.path.basename( xc )
+            xname = os.path.splitext( bname )
+            if xname and xname[0]:
+                xcconfigMode[xname[0]] = bname
+        if not 'native' in xcconfigMode:
+            raise Exception( 'native xcconfig not found' )
+        xcconfigMode.default = 'native'
+        xcconfigMode.mode = xcconfigMode.default
+
     ## create CLI and parse
     cli = createCLI()
     (options,args) = cli.parse_args()
@@ -1288,6 +1480,31 @@ try:
     for action in Action.actions:
         action.run()
 
+    ## enable local yasm when yasm probe fails or version is too old
+    ## x264 requires 1.2.0+
+    if not options.enable_local_yasm:
+        if Tools.yasm.fail:
+            stdout.write( 'note: enabling local yasm: missing system yasm\n' )
+            options.enable_local_yasm = True
+        elif Tools.yasm.version.inadequate():
+            stdout.write( 'note: enabling local yasm: minimum required version is %s and %s is %s\n' % ('.'.join([str(i) for i in Tools.yasm.version.minversion]),Tools.yasm.pathname,Tools.yasm.version.svers) )
+            options.enable_local_yasm = True
+
+    ## enable local autotools when any of { autoconf, automake, libtool } probe fails
+    if not options.enable_local_autotools and (Tools.autoconf.fail or Tools.automake.fail or Tools.libtool.fail):
+        stdout.write( 'note: enabling local autotools\n' )
+        options.enable_local_autotools = True
+
+    ## enable local cmake when cmake probe fails
+    if not options.enable_local_cmake and (Tools.cmake.fail):
+        stdout.write( 'note: enabling local cmake\n' )
+        options.enable_local_cmake = True
+
+    ## enable local pkg-config when probe fails
+    if not options.enable_local_pkgconfig and Tools.pkgconfig.fail:
+        stdout.write( 'note: enabling local pkgconfig\n' )
+        options.enable_local_pkgconfig = True
+
     if build.system == 'mingw':
         dlfcn_test = """
 #include <dlfcn.h>
@@ -1363,6 +1580,42 @@ int main ()
         iconv = LDProbe( 'static iconv', '%s -static' % Tools.gcc.pathname, '-liconv', iconv_test )
         iconv.run()
 
+        regex_test = """
+#include <stdio.h>
+#include <regex.h>
+
+int match(regex_t *x, char *s)
+{
+    regmatch_t matches[1];
+    return regexec(x, s, 1, matches, 0);
+}
+
+int main()
+{
+    int     rv;
+    regex_t exp;
+
+    rv = regcomp(&exp, "^[0-9]+$";", REG_EXTENDED);
+    if (rv != 0) {
+        return 1;
+    }
+
+    if (match(&exp, "7") != 0)
+    {
+        return 1;
+    }
+    if (match(&exp, "foo") == 0)
+    {
+        return 1;
+    }
+
+    regfree(&exp);
+    return 0;
+}
+"""
+        regex = LDProbe( 'static regex', '%s -static' % Tools.gcc.pathname, '-lregex', regex_test )
+        regex.run()
+
     ## cfg hook before doc prep
     cfg.doc_ready()
 
@@ -1431,19 +1684,15 @@ int main ()
     doc.add( 'BUILD.ncpu',    core.count )
     doc.add( 'BUILD.jobs',    core.jobs )
 
-    doc.add( 'BUILD.cross',        int(options.cross != None or arch.mode.mode != arch.mode.default) )
+    doc.add( 'BUILD.cross', int(options.cross != None or arch.mode.mode != arch.mode.default) )
     if options.cross:
         doc.add( 'BUILD.cross.prefix', '%s-' % (options.cross) )
     else:
         doc.add( 'BUILD.cross.prefix', '' )
 
-    doc.add( 'BUILD.method', 'terminal' )
     doc.add( 'BUILD.date',   time.strftime('%c') )
     doc.add( 'BUILD.arch',   arch.mode.mode )
 
-    doc.addBlank()
-    doc.add( 'CONF.method', options.conf_method )
-
     doc.addBlank()
     doc.add( 'SRC',     cfg.src_final )
     doc.add( 'SRC/',    cfg.src_final + os.sep )
@@ -1453,21 +1702,33 @@ int main ()
     doc.add( 'PREFIX/', cfg.prefix_final + os.sep )
     
     doc.addBlank()
-    doc.add( 'FEATURE.asm',   'disabled' )
-    doc.add( 'FEATURE.gtk',   int( not options.disable_gtk ))
-    doc.add( 'FEATURE.gtk.update.checks',   int( not options.disable_gtk_update_checks ))
-    doc.add( 'FEATURE.gtk.mingw',   int( options.enable_gtk_mingw ))
-    doc.add( 'FEATURE.ff.mpeg2',   int( options.enable_ff_mpeg2 ))
-    doc.add( 'FEATURE.xcode', int( not (Tools.xcodebuild.fail or options.disable_xcode or options.cross) ))
+    doc.add( 'FEATURE.local_yasm', int( options.enable_local_yasm ))
+    doc.add( 'FEATURE.local_autotools', int( options.enable_local_autotools ))
+    doc.add( 'FEATURE.local_cmake', int( options.enable_local_cmake ))
+    doc.add( 'FEATURE.local_pkgconfig', int( options.enable_local_pkgconfig ))
+    doc.add( 'FEATURE.asm',        'disabled' )
+    doc.add( 'FEATURE.gtk',        int( not options.disable_gtk ))
+    doc.add( 'FEATURE.gtk.update.checks', int( not options.disable_gtk_update_checks ))
+    doc.add( 'FEATURE.gtk.mingw',  int( options.enable_gtk_mingw ))
+    doc.add( 'FEATURE.gst',        int( not options.disable_gst ))
+    doc.add( 'FEATURE.fdk_aac',    int( options.enable_fdk_aac ))
+    doc.add( 'FEATURE.libav_aac',  int( options.enable_libav_aac ))
+    doc.add( 'FEATURE.qsv',        int( options.enable_qsv ))
+    doc.add( 'FEATURE.hwd',        int( options.enable_hwd ))
+    doc.add( 'FEATURE.xcode',      int( not (Tools.xcodebuild.fail or options.disable_xcode or options.cross) ))
+    doc.add( 'FEATURE.x265',       int( options.enable_x265 ))
 
     if not Tools.xcodebuild.fail and not options.disable_xcode:
         doc.addBlank()
-        doc.add( 'XCODE.external.src',    cfg.xcode_x_src )
-        doc.add( 'XCODE.external.build',  cfg.xcode_x_build )
-        doc.add( 'XCODE.external.prefix', cfg.xcode_x_prefix )
+        doc.add( 'XCODE.driver', options.xcode_driver )
+        if os.path.isabs(options.xcode_symroot):
+            doc.add( 'XCODE.symroot', options.xcode_symroot )
+        else:
+            doc.add( 'XCODE.symroot', os.path.abspath(os.path.join(cfg.build_dir,options.xcode_symroot)) )
+        doc.add( 'XCODE.xcconfig', xcconfigMode[xcconfigMode.mode] )
 
-    doc.addBlank()
     if build.system == 'mingw':
+        doc.addBlank()
         if not dlfcn.fail:
             doc.add( 'HAS.dlfcn', 1 )
         if not pthread.fail:
@@ -1478,11 +1739,14 @@ int main ()
             doc.add( 'HAS.libz', 1 )
         if not iconv.fail:
             doc.add( 'HAS.iconv', 1 )
+        if not regex.fail:
+            doc.add( 'HAS.regex', 1 )
 
     doc.addMake( '' )
-    doc.addMake( '## define debug mode before other includes' )
+    doc.addMake( '## define debug mode and optimize before other includes' )
     doc.addMake( '## since it is tested in some module.defs' )
     doc.add( 'GCC.g', debugMode.mode )
+    doc.add( 'GCC.O', optimizeMode.mode )
     doc.addBlank()
     doc.addMake( '## include definitions' )
     doc.addMake( 'include $(SRC/)make/include/main.defs' )
@@ -1504,11 +1768,13 @@ int main ()
         doc.add( 'GCC.archs', '' )
         doc.add( 'GCC.sysroot', '' )
         doc.add( 'GCC.minver', '' )
-    doc.add( 'GCC.ldsysroot', '$(GCC.sysroot)' )
-    doc.add( 'GCC.ldminver', '$(GCC.minver)' )
-    doc.add( 'GCC.O', optimizeMode.mode )
 
-    if options.enable_asm and not Tools.yasm.fail:
+    if build.match( 'i?86-*' ):
+        doc.add( 'LIBHB.GCC.D', 'ARCH_X86_32', append=True )
+    elif build.match( 'x86_64-*' ):
+        doc.add( 'LIBHB.GCC.D', 'ARCH_X86_64', append=True )
+
+    if options.enable_asm and ( not Tools.yasm.fail or options.enable_local_yasm ):
         asm = ''
         if build.match( 'i?86-*' ):
             asm = 'x86'
