@@ -11,21 +11,39 @@ namespace HandBrake.ApplicationServices.Factories
 {
     using System.Collections.Generic;
     using System.Collections.ObjectModel;
+    using System.ComponentModel;
     using System.Globalization;
     using System.Linq;
 
     using HandBrake.ApplicationServices.Model;
+    using HandBrake.ApplicationServices.Model.Audio;
     using HandBrake.ApplicationServices.Model.Encoding;
+    using HandBrake.ApplicationServices.Model.Subtitle;
     using HandBrake.ApplicationServices.Services;
     using HandBrake.ApplicationServices.Utilities;
     using HandBrake.Interop.Model.Encoding;
     using HandBrake.Interop.Model.Encoding.x264;
+    using HandBrake.Interop.Model.Encoding.x265;
 
     /// <summary>
     /// A Factory to translate a Plist object into a Preset.
     /// </summary>
     public class PlistPresetFactory
     {
+        /// <summary>
+        /// The lang list.
+        /// </summary>
+        private static IDictionary<string, string> langList;
+
+        /// <summary>
+        /// Initializes static members of the <see cref="PlistPresetFactory"/> class. 
+        /// </summary>
+        static PlistPresetFactory()
+        {
+            IDictionary<string, string> langMap = LanguageUtilities.MapLanguages();
+            langList = (from entry in langMap select entry).ToDictionary(pair => pair.Value, pair => pair.Key);
+        }
+
         /// <summary>
         /// The create preset.
         /// </summary>
@@ -37,7 +55,13 @@ namespace HandBrake.ApplicationServices.Factories
         /// </returns>
         public static Preset CreatePreset(PList plist)
         {
-            Preset preset = new Preset { Task = new EncodeTask(), Category = PresetService.UserPresetCatgoryName };
+            Preset preset = new Preset
+                                {
+                                    Task = new EncodeTask(),
+                                    Category = PresetService.UserPresetCatgoryName,
+                                    AudioTrackBehaviours = new AudioBehaviours(),
+                                    SubtitleTrackBehaviours = new SubtitleBehaviours()
+                                };
 
             // Parse the settings out.
             foreach (var item in plist)
@@ -79,7 +103,6 @@ namespace HandBrake.ApplicationServices.Factories
                 preset.Task.MaxHeight = preset.Task.Width;
             }
 
-
             return preset;
         }
 
@@ -103,9 +126,6 @@ namespace HandBrake.ApplicationServices.Factories
                 case "Mp4HttpOptimize":
                     preset.Task.OptimizeMP4 = kvp.Value == 1;
                     break;
-                case "Mp4LargeFile":
-                    preset.Task.LargeFile = kvp.Value == 1;
-                    break;
                 case "Mp4iPodCompatible":
                     preset.Task.IPod5GSupport = kvp.Value == 1;
                     break;
@@ -153,7 +173,7 @@ namespace HandBrake.ApplicationServices.Factories
                     preset.Task.CustomDecomb = kvp.Value;
                     break;
                 case "PictureDecombDeinterlace":
-                    preset.UseDeinterlace = kvp.Value == 1;
+                    preset.UseDeinterlace = kvp.Value == true;
                     break;
                 case "PictureDeinterlace":
                     preset.Task.Deinterlace = (Deinterlace)kvp.Value;
@@ -164,6 +184,12 @@ namespace HandBrake.ApplicationServices.Factories
                 case "PictureDenoise":
                     preset.Task.Denoise = (Denoise)kvp.Value;
                     break;
+                case "DenoisePreset":
+                    preset.Task.DenoisePreset = (DenoisePreset)kvp.Value; // TODO to be confirmed.
+                    break;
+                case "DenoiseTune":
+                    preset.Task.DenoiseTune = (DenoiseTune)kvp.Value; // TODO to be confirmed.
+                    break;
                 case "PictureDenoiseCustom":
                     preset.Task.CustomDenoise = kvp.Value;
                     break;
@@ -203,7 +229,7 @@ namespace HandBrake.ApplicationServices.Factories
                     }
                     break;
                 case "VideoGrayScale":
-                    preset.Task.Grayscale = kvp.Value == 1;
+                    preset.Task.Grayscale = kvp.Value == true;
                     break;
                 case "VideoQualitySlider":
                     preset.Task.Quality = double.Parse(kvp.Value.ToString(), CultureInfo.InvariantCulture);
@@ -238,10 +264,22 @@ namespace HandBrake.ApplicationServices.Factories
                 case "h264Level":
                     preset.Task.H264Level = kvp.Value;
                     break;
+                case "QsvPreset":
+                    preset.Task.QsvPreset = EnumHelper<QsvPreset>.GetValue(kvp.Value, true);
+                    break;
+                case "h265Profile":
+                    preset.Task.H265Profile = EnumHelper<x265Profile>.GetValue(kvp.Value, true);
+                    break;
+                case "x265Tune":
+                    preset.Task.X265Tune = EnumHelper<x265Tune>.GetValue(kvp.Value, true);
+                    break;
+                case "x265Preset":
+                    preset.Task.X265Preset = EnumHelper<x265Preset>.GetValue(kvp.Value, true);
+                    break;
 
                 // Chapter Markers Tab
                 case "ChapterMarkers":
-                    preset.Task.IncludeChapterMarkers = kvp.Value == 1;
+                    preset.Task.IncludeChapterMarkers = kvp.Value == true;
                     break;
 
                 // Advanced x264 tab
@@ -270,7 +308,7 @@ namespace HandBrake.ApplicationServices.Factories
                     preset.Name = kvp.Value;
                     break;
                 case "Type":
-                    //preset.Task.Type = kvp.Value; // TODO find out what this is
+                    // preset.Task.Type = kvp.Value; // TODO find out what this is
                     break;
                 case "UsesMaxPictureSettings":
                     // TODO Not sure if this is used now!?
@@ -284,23 +322,55 @@ namespace HandBrake.ApplicationServices.Factories
 
                 // Allowed Passthru
                 case "AudioAllowAACPass":
-                    preset.Task.AllowedPassthruOptions.AudioAllowAACPass = kvp.Value == 1;
+                    preset.Task.AllowedPassthruOptions.AudioAllowAACPass = kvp.Value == true;
                     break;
                 case "AudioAllowAC3Pass":
-                    preset.Task.AllowedPassthruOptions.AudioAllowAC3Pass = kvp.Value == 1;
+                    preset.Task.AllowedPassthruOptions.AudioAllowAC3Pass = kvp.Value == true;
                     break;
                 case "AudioAllowDTSHDPass":
-                    preset.Task.AllowedPassthruOptions.AudioAllowDTSHDPass = kvp.Value == 1;
+                    preset.Task.AllowedPassthruOptions.AudioAllowDTSHDPass = kvp.Value == true;
                     break;
                 case "AudioAllowDTSPass":
-                    preset.Task.AllowedPassthruOptions.AudioAllowDTSPass = kvp.Value == 1;
+                    preset.Task.AllowedPassthruOptions.AudioAllowDTSPass = kvp.Value == true;
                     break;
                 case "AudioAllowMP3Pass":
-                    preset.Task.AllowedPassthruOptions.AudioAllowMP3Pass = kvp.Value == 1;
+                    preset.Task.AllowedPassthruOptions.AudioAllowMP3Pass = kvp.Value == true;
                     break;
                 case "AudioEncoderFallback":
                     preset.Task.AllowedPassthruOptions.AudioEncoderFallback = EnumHelper<AudioEncoder>.GetValue(kvp.Value);
                     break;
+
+                // Audio Defaults
+                case "AudioLanguageList":
+                    preset.AudioTrackBehaviours.SelectedLangauges = new BindingList<string>(ParseLangaugeCodeList(kvp.Value));
+                    break;
+                case "AudioSecondaryEncoderMode":
+                    break;
+                case "AudioTrackSelectionBehavior":
+                    preset.AudioTrackBehaviours.SelectedBehaviour = kvp.Value == "all"
+                                                                        ? AudioBehaviourModes.AllMatching
+                                                                        : kvp.Value == "first"
+                                                                              ? AudioBehaviourModes.FirstMatch
+                                                                              : AudioBehaviourModes.None;
+                    break;
+
+                // Subtitle Defaults
+                case "SubtitleAddForeignAudioSearch":
+                    preset.SubtitleTrackBehaviours.AddForeignAudioScanTrack = kvp.Value == true;
+                    break;
+                case "SubtitleAddCC":
+                    preset.SubtitleTrackBehaviours.AddClosedCaptions = kvp.Value == true;
+                    break;
+                case "SubtitleLanguageList":
+                    preset.SubtitleTrackBehaviours.SelectedLangauges = new BindingList<string>(ParseLangaugeCodeList(kvp.Value));
+                    break;
+                case "SubtitleTrackSelectionBehavior":
+                    preset.SubtitleTrackBehaviours.SelectedBehaviour = kvp.Value == "all"
+                                                                     ? SubtitleBehaviourModes.AllMatching
+                                                                     : kvp.Value == "first"
+                                                                           ? SubtitleBehaviourModes.FirstMatch
+                                                                           : SubtitleBehaviourModes.None;
+                    break;
             }
         }
 
@@ -318,6 +388,30 @@ namespace HandBrake.ApplicationServices.Factories
             return audioList.Select(item => ParseAudioTrackParameters(item)).Cast<AudioTrack>().ToList();
         }
 
+        /// <summary>
+        /// The parse langauge code list.
+        /// </summary>
+        /// <param name="languages">
+        /// The languages.
+        /// </param>
+        /// <returns>
+        /// The <see cref="IEnumerable"/>.
+        /// </returns>
+        private static IEnumerable<string> ParseLangaugeCodeList(IEnumerable<object> languages)
+        {
+            List<string> languageCodesList = new List<string>();
+            foreach (var item in languages)
+            {
+                string language;
+                if (langList.TryGetValue(item.ToString(), out language))
+                {
+                    languageCodesList.Add(language);
+                }            
+            }
+
+            return languageCodesList;
+        }
+
         /// <summary>
         /// Parse an audio track's parameters.
         /// </summary>
@@ -344,16 +438,16 @@ namespace HandBrake.ApplicationServices.Factories
                         track.MixDown = Converters.GetAudioMixDown(item.Value.Trim());
                         break;
                     case "AudioSamplerate":
-                        track.SampleRate = item.Value == "Auto" ? 0 : double.Parse(item.Value, CultureInfo.InvariantCulture);
+                        track.SampleRate = item.Value == "Auto" ? 0 : double.Parse(item.Value);
                         break;
                     case "AudioTrack":
-                        //track.SourceTrack = value; We don't do anything with this one.
+                        // track.SourceTrack = value; We don't do anything with this one.
                         break;
                     case "AudioTrackDRCSlider":
-                        track.DRC = double.Parse(item.Value.ToString(), CultureInfo.InvariantCulture);
+                        track.DRC = item.Value;
                         break;
                     case "AudioTrackGainSlider":
-                        track.Gain = int.Parse(item.Value.ToString(), CultureInfo.InvariantCulture);
+                        track.Gain = (int)item.Value;
                         break;
                 }
             }
