@@ -18,6 +18,7 @@ namespace HandBrake.ApplicationServices.Model
     using HandBrake.Interop.Model;
     using HandBrake.Interop.Model.Encoding;
     using HandBrake.Interop.Model.Encoding.x264;
+    using HandBrake.Interop.Model.Encoding.x265;
 
     using OutputFormat = HandBrake.ApplicationServices.Model.Encoding.OutputFormat;
 
@@ -33,6 +34,11 @@ namespace HandBrake.ApplicationServices.Model
         /// </summary>
         private bool showAdvancedTab;
 
+        /// <summary>
+        /// The advanced encoder options.
+        /// </summary>
+        private string advancedEncoderOptions;
+
         #endregion
 
         /// <summary>
@@ -46,9 +52,14 @@ namespace HandBrake.ApplicationServices.Model
             this.ChapterNames = new ObservableCollection<ChapterMarker>();
             this.AllowedPassthruOptions = new AllowedPassthru();
             this.X264Preset = x264Preset.Medium;
+            this.QsvPreset = QsvPreset.Quality;
             this.H264Profile = x264Profile.None;
             this.X264Tune = x264Tune.None;
             this.Modulus = 16;
+
+            this.H265Profile = x265Profile.None;
+            this.X265Preset = x265Preset.VeryFast;
+            this.X265Tune = x265Tune.None;
         }
 
         /// <summary>
@@ -87,6 +98,8 @@ namespace HandBrake.ApplicationServices.Model
             this.Decomb = task.Decomb;
             this.Deinterlace = task.Deinterlace;
             this.Denoise = task.Denoise;
+            this.DenoisePreset = task.DenoisePreset;
+            this.DenoiseTune = task.DenoiseTune;
             this.Destination = task.Destination;
             this.Detelecine = task.Detelecine;
             this.DisplayWidth = task.DisplayWidth;
@@ -99,7 +112,6 @@ namespace HandBrake.ApplicationServices.Model
             this.IncludeChapterMarkers = task.IncludeChapterMarkers;
             this.IPod5GSupport = task.IPod5GSupport;
             this.KeepDisplayAspect = task.KeepDisplayAspect;
-            this.LargeFile = task.LargeFile;
             this.MaxHeight = task.MaxHeight;
             this.MaxWidth = task.MaxWidth;
             this.Modulus = task.Modulus;
@@ -126,6 +138,7 @@ namespace HandBrake.ApplicationServices.Model
             this.VideoEncodeRateType = task.VideoEncodeRateType;
             this.Width = task.Width;
             this.X264Preset = task.X264Preset;
+            this.QsvPreset = task.QsvPreset;
             this.H264Profile = task.H264Profile;
             this.X264Tune = task.X264Tune;
             this.H264Level = task.H264Level;
@@ -134,6 +147,12 @@ namespace HandBrake.ApplicationServices.Model
 
             this.PreviewStartAt = task.PreviewStartAt;
             this.PreviewDuration = task.PreviewDuration;
+
+            this.ShowAdvancedTab = task.ShowAdvancedTab;
+
+            this.X265Preset = task.X265Preset;
+            this.X265Tune = task.X265Tune;
+            this.H265Profile = task.H265Profile;
         }
 
         #region Source
@@ -185,11 +204,6 @@ namespace HandBrake.ApplicationServices.Model
         /// </summary>
         public OutputFormat OutputFormat { get; set; }
 
-        /// <summary>
-        /// Gets or sets a value indicating whether LargeFile.
-        /// </summary>
-        public bool LargeFile { get; set; }
-
         /// <summary>
         /// Gets or sets a value indicating whether Optimize.
         /// </summary>
@@ -301,6 +315,16 @@ namespace HandBrake.ApplicationServices.Model
         /// </summary>
         public Denoise Denoise { get; set; }
 
+        /// <summary>
+        /// Gets or sets the denoise preset.
+        /// </summary>
+        public DenoisePreset DenoisePreset { get; set; }
+
+        /// <summary>
+        /// Gets or sets the denoise tune.
+        /// </summary>
+        public DenoiseTune DenoiseTune { get; set; }
+
         /// <summary>
         /// Gets or sets CustomDenoise.
         /// </summary>
@@ -407,13 +431,28 @@ namespace HandBrake.ApplicationServices.Model
         /// <summary>
         /// Gets or sets AdvancedEncoderOptions.
         /// </summary>
-        public string AdvancedEncoderOptions { get; set; }
+        public string AdvancedEncoderOptions
+        {
+            get
+            {
+                return this.advancedEncoderOptions;
+            }
+            set
+            {
+                this.advancedEncoderOptions = value;
+            }
+        }
 
         /// <summary>
         /// Gets or sets x264Preset.
         /// </summary>
         public x264Preset X264Preset { get; set; }
 
+        /// <summary>
+        /// Gets or sets the qsv preset.
+        /// </summary>
+        public QsvPreset QsvPreset { get; set; }
+
         /// <summary>
         /// Gets or sets x264Profile.
         /// </summary>
@@ -439,6 +478,21 @@ namespace HandBrake.ApplicationServices.Model
         /// </summary>
         public string ExtraAdvancedArguments { get; set; }
 
+        /// <summary>
+        /// Gets or sets x265Preset.
+        /// </summary>
+        public x265Preset X265Preset { get; set; }
+
+        /// <summary>
+        /// Gets or sets x265Profile.
+        /// </summary>
+        public x265Profile H265Profile { get; set; }
+
+        /// <summary>
+        /// Gets or sets X265Tune.
+        /// </summary>
+        public x265Tune X265Tune { get; set; }
+
         #endregion
 
         #region Preview
@@ -464,9 +518,9 @@ namespace HandBrake.ApplicationServices.Model
         {
             get
             {
-                if (this.OutputFormat == OutputFormat.M4V || this.OutputFormat == OutputFormat.Mp4)
+                if (this.OutputFormat == OutputFormat.Mp4)
                 {
-                    bool audio = this.AudioTracks.Any(item => item.Encoder == AudioEncoder.Ac3Passthrough || 
+                    bool audio = this.AudioTracks.Any(item => item.Encoder == AudioEncoder.Ac3Passthrough ||
                         item.Encoder == AudioEncoder.Ac3 || item.Encoder == AudioEncoder.DtsPassthrough || item.Encoder == AudioEncoder.Passthrough);
 
                     bool subtitles = this.SubtitleTracks.Any(track => track.SubtitleType != SubtitleType.VobSub);
