@@ -63,7 +63,10 @@ namespace HandBrake.ApplicationServices.Model.Encoding
         [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
         private Audio scannedTrack;
 
-        private string trackName;
+        /// <summary>
+        /// The is default.
+        /// </summary>
+        private bool isDefault;
 
         #endregion
 
@@ -75,7 +78,7 @@ namespace HandBrake.ApplicationServices.Model.Encoding
         public AudioTrack()
         {
             // Default Values
-            this.Encoder = AudioEncoder.Faac;
+            this.Encoder = AudioEncoder.ffaac;
             this.MixDown = Mixdown.DolbyProLogicII;
             this.SampleRate = 48;
             this.Bitrate = 160;
@@ -183,6 +186,21 @@ namespace HandBrake.ApplicationServices.Model.Encoding
             }
         }
 
+        /// <summary>
+        /// Gets or sets a value indicating whether is default.
+        /// </summary>
+        public bool IsDefault
+        {
+            get
+            {
+                return this.isDefault;
+            }
+            set
+            {
+                this.isDefault = value;
+            }
+        }
+
         /// <summary>
         ///   Gets or sets Audio Encoder
         /// </summary>
@@ -341,7 +359,7 @@ namespace HandBrake.ApplicationServices.Model.Encoding
         {
             get
             {
-                return this.IsPassthru || this.Encoder == AudioEncoder.ffflac;
+                return this.IsPassthru || this.Encoder == AudioEncoder.ffflac || this.Encoder == AudioEncoder.ffflac24;
             }
         }
 
@@ -352,7 +370,7 @@ namespace HandBrake.ApplicationServices.Model.Encoding
         {
             get
             {
-                return this.IsPassthru || this.Encoder == AudioEncoder.ffflac;
+                return this.IsPassthru || this.Encoder == AudioEncoder.ffflac || this.Encoder == AudioEncoder.ffflac24;
             }
         }
 
@@ -367,17 +385,7 @@ namespace HandBrake.ApplicationServices.Model.Encoding
         /// <summary>
         /// Gets or sets the track name.
         /// </summary>
-        public string TrackName
-        {
-            get
-            {
-                return this.trackName;
-            }
-            set
-            {
-                this.trackName = value;
-            }
-        }
+        public string TrackName { get; set; }
 
         #endregion
     }
