@@ -11,6 +11,9 @@ namespace HandBrake.ApplicationServices.Model
 {
     using Caliburn.Micro;
 
+    using HandBrake.ApplicationServices.Model.Audio;
+    using HandBrake.ApplicationServices.Model.Subtitle;
+
     /// <summary>
     /// A Preset for encoding with.
     /// </summary>
@@ -89,10 +92,48 @@ namespace HandBrake.ApplicationServices.Model
         /// </summary>
         public string Version { get; set; }
 
+        /// <summary>
+        /// Gets or sets the audio track behaviours.
+        /// </summary>
+        public AudioBehaviours AudioTrackBehaviours { get; set; }
+
+        /// <summary>
+        /// Gets or sets the subtitle track behaviours.
+        /// </summary>
+        public SubtitleBehaviours SubtitleTrackBehaviours { get; set; }
+
         #endregion
 
         #region Public Methods
 
+        /// <summary>
+        /// Update this preset.
+        /// The given parameters should be copy-constructed.
+        /// </summary>
+        /// <param name="task">
+        /// The task.
+        /// </param>
+        /// <param name="audioBehaviours">
+        /// The audio behaviours.
+        /// </param>
+        /// <param name="subtitleBehaviours">
+        /// The subtitle behaviours.
+        /// </param>
+        public void Update(EncodeTask task, AudioBehaviours audioBehaviours, SubtitleBehaviours subtitleBehaviours)
+        {
+            // Copy over Max Width / Height for the following picture settings modes.
+            if (this.PictureSettingsMode == PresetPictureSettingsMode.Custom
+                || this.PictureSettingsMode == PresetPictureSettingsMode.SourceMaximum)
+            {
+                task.MaxWidth = this.Task.MaxWidth;
+                task.MaxHeight = this.Task.MaxHeight;
+            }
+
+            this.Task = task;
+            this.AudioTrackBehaviours = audioBehaviours;
+            this.SubtitleTrackBehaviours = subtitleBehaviours;
+        }
+
         /// <summary>
         ///  Override the ToString Method
         /// </summary>
