@@ -10,9 +10,7 @@
 namespace HandBrake.ApplicationServices.Parsing
 {
     using System;
-    using System.Collections.Generic;
-    using System.IO;
-    using System.Text.RegularExpressions;
+    using System.Xml.Serialization;
 
     using HandBrake.ApplicationServices.Model.Encoding;
     using HandBrake.ApplicationServices.Utilities;
@@ -32,6 +30,9 @@ namespace HandBrake.ApplicationServices.Parsing
         /// <summary>
         /// Initializes a new instance of the <see cref="Subtitle"/> class.
         /// </summary>
+        /// <param name="sourceId">
+        /// The source Id.
+        /// </param>
         /// <param name="trackNumber">
         /// The track number.
         /// </param>
@@ -44,14 +45,27 @@ namespace HandBrake.ApplicationServices.Parsing
         /// <param name="subtitleType">
         /// The subtitle type.
         /// </param>
-        public Subtitle(int trackNumber, string language, string languageCode, SubtitleType subtitleType)
+        /// <param name="canBurn">
+        /// The can Burn.
+        /// </param>
+        /// <param name="canForce">
+        /// The can Force.
+        /// </param>
+        public Subtitle(int sourceId, int trackNumber, string language, string languageCode, SubtitleType subtitleType, bool canBurn, bool canForce)
         {
+            this.SourceId = sourceId;
             this.TrackNumber = trackNumber;
             this.Language = language;
             this.LanguageCode = languageCode;
             this.SubtitleType = subtitleType;
+            this.CanBurnIn = canBurn;
+            this.CanForce = canForce;
         }
 
+        /// <summary>
+        /// Gets or sets the source id.
+        /// </summary>
+        public int SourceId { get; set; }
 
         /// <summary>
         /// Gets or sets the track number of this Subtitle
@@ -68,6 +82,34 @@ namespace HandBrake.ApplicationServices.Parsing
         /// </summary>
         public string LanguageCode { get; set; }
 
+        /// <summary>
+        /// Gets the language code clean.
+        /// TODO Remove this after fixing language code.
+        /// </summary>
+        public string LanguageCodeClean
+        {
+            get
+            {
+                if (this.LanguageCode != null)
+                {
+                    return this.LanguageCode.Replace("iso639-2: ", string.Empty).Trim();
+                }
+                return string.Empty;
+            }
+        }
+
+        /// <summary>
+        /// Gets a value indicating whether can burn in.
+        /// </summary>
+        [XmlIgnore]
+        public bool CanBurnIn { get; private set; }
+
+        /// <summary>
+        /// Gets a value indicating whether can force.
+        /// </summary>
+        [XmlIgnore]
+        public bool CanForce { get; private set; }
+
         /// <summary>
         /// Gets or sets the Subtitle Type
         /// </summary>
@@ -84,89 +126,6 @@ namespace HandBrake.ApplicationServices.Parsing
             }
         }
 
-        /// <summary>
-        /// Parse the input strings related to subtitles
-        /// </summary>
-        /// <param name="output">
-        /// The output.
-        /// </param>
-        /// <returns>
-        /// A Subitle object
-        /// </returns>
-        public static Subtitle Parse(StringReader output)
-        {
-            string curLine = output.ReadLine();
-
-            // + 1, English (iso639-2: eng) (Text)(SSA)
-            // + 1, English (iso639-2: eng) (Text)(UTF-8)
-            Match m = Regex.Match(curLine, @"^    \+ ([0-9]*), ([A-Za-z, ]*) \((.*)\) \(([a-zA-Z]*)\)\(([a-zA-Z0-9\-]*)\)");
-
-            if (m.Success && !curLine.Contains("HandBrake has exited."))
-            {
-                var thisSubtitle = new Subtitle
-                                       {
-                                           TrackNumber = int.Parse(m.Groups[1].Value.Trim()),
-                                           Language = m.Groups[2].Value,
-                                           LanguageCode = m.Groups[3].Value,
-                                       };
-
-                switch (m.Groups[5].Value)
-                {
-                    case "VOBSUB":
-                        thisSubtitle.SubtitleType = SubtitleType.VobSub;
-                        break;
-                    case "SRT":
-                        thisSubtitle.SubtitleType = SubtitleType.SRT;
-                        break;
-                    case "CC":
-                        thisSubtitle.SubtitleType = SubtitleType.CC;
-                        break;
-                    case "UTF-8":
-                        thisSubtitle.SubtitleType = SubtitleType.UTF8Sub;
-                        break;
-                    case "TX3G":
-                        thisSubtitle.SubtitleType = SubtitleType.TX3G;
-                        break;
-                    case "SSA":
-                        thisSubtitle.SubtitleType = SubtitleType.SSA;
-                        break;
-                    case "PGS":
-                        thisSubtitle.SubtitleType = SubtitleType.PGS;
-                        break;
-                    default:
-                        thisSubtitle.SubtitleType = SubtitleType.Unknown;
-                        break;
-                }
-
-                return thisSubtitle;
-            }
-            return null;
-        }
-
-        /// <summary>
-        /// Parse a list of Subtitle tracks from an input string.
-        /// </summary>
-        /// <param name="output">
-        /// The output.
-        /// </param>
-        /// <returns>
-        /// An array of Subtitle objects
-        /// </returns>
-        public static IEnumerable<Subtitle> ParseList(StringReader output)
-        {
-            var subtitles = new List<Subtitle>();
-            while ((char)output.Peek() != '+')
-            {
-                Subtitle thisSubtitle = Parse(output);
-
-                if (thisSubtitle != null)
-                    subtitles.Add(thisSubtitle);
-                else
-                    break;
-            }
-            return subtitles.ToArray();
-        }
-
         /// <summary>
         /// Override of the ToString method to make this object easier to use in the UI
         /// </summary>
@@ -211,14 +170,17 @@ namespace HandBrake.ApplicationServices.Parsing
             {
                 return false;
             }
+            
             if (ReferenceEquals(this, obj))
             {
                 return true;
             }
+            
             if (obj.GetType() != typeof(Subtitle))
             {
                 return false;
             }
+
             return Equals((Subtitle)obj);
         }
 
