@@ -33,11 +33,6 @@ namespace HandBrake.ApplicationServices.Services.Base
         /// </summary>
         private static readonly object FileWriterLock = new object();
 
-        /// <summary>
-        /// The User Setting Service
-        /// </summary>
-        private readonly IUserSettingService userSettingService;
-
         /// <summary>
         /// The Log File Header
         /// </summary>
@@ -58,16 +53,10 @@ namespace HandBrake.ApplicationServices.Services.Base
         /// <summary>
         /// Initializes a new instance of the <see cref="EncodeBase"/> class.
         /// </summary>
-        /// <param name="userSettingService">
-        /// The user Setting Service.
-        /// </param>
-        public EncodeBase(IUserSettingService userSettingService)
+        public EncodeBase()
         {
-            this.userSettingService = userSettingService;
             this.logBuffer = new StringBuilder();
-            header =
-                GeneralUtilities.CreateCliLogHeader();
-
+            header = GeneralUtilities.CreateCliLogHeader();
             this.LogIndex = 0;
         }
 
@@ -197,7 +186,10 @@ namespace HandBrake.ApplicationServices.Services.Base
         /// <param name="destination">
         /// The Destination File Path
         /// </param>
-        public void ProcessLogs(string destination)
+        /// <param name="configuration">
+        /// The configuration.
+        /// </param>
+        public void ProcessLogs(string destination, HBConfiguration configuration)
         {
             try
             {
@@ -220,17 +212,16 @@ namespace HandBrake.ApplicationServices.Services.Base
                 File.Copy(tempLogFile, Path.Combine(logDir, encodeLogFile));
 
                 // Save a copy of the log file in the same location as the enocde.
-                if (this.userSettingService.GetUserSetting<bool>(ASUserSettingConstants.SaveLogWithVideo))
+                if (configuration.SaveLogWithVideo)
                 {
                     File.Copy(tempLogFile, Path.Combine(encodeDestinationPath, encodeLogFile));
                 }
 
                 // Save a copy of the log file to a user specified location
-                if (Directory.Exists(this.userSettingService.GetUserSetting<string>(ASUserSettingConstants.SaveLogCopyDirectory)) &&
-                    this.userSettingService.GetUserSetting<bool>(ASUserSettingConstants.SaveLogToCopyDirectory))
+                if (Directory.Exists(configuration.SaveLogCopyDirectory) && configuration.SaveLogToCopyDirectory)
                 {
                     File.Copy(
-                        tempLogFile, Path.Combine(this.userSettingService.GetUserSetting<string>(ASUserSettingConstants.SaveLogCopyDirectory), encodeLogFile));
+                        tempLogFile, Path.Combine(configuration.SaveLogCopyDirectory, encodeLogFile));
                 }
             }
             catch (Exception)
@@ -299,7 +290,7 @@ namespace HandBrake.ApplicationServices.Services.Base
 
                 return null;
             }
-            catch (Exception exc)
+            catch (Exception)
             {
                 return null;
             }
@@ -320,13 +311,9 @@ namespace HandBrake.ApplicationServices.Services.Base
 
             try
             {
-                string query = QueryGeneratorUtility.GenerateQuery(new EncodeTask(encodeQueueTask.Task),
-                    userSettingService.GetUserSetting<int>(ASUserSettingConstants.PreviewScanCount),
-                    userSettingService.GetUserSetting<int>(ASUserSettingConstants.Verbosity),
-                    userSettingService.GetUserSetting<bool>(ASUserSettingConstants.DisableLibDvdNav));
+                string query = QueryGeneratorUtility.GenerateQuery(new EncodeTask(encodeQueueTask.Task), encodeQueueTask.Configuration);
                 this.logBuffer = new StringBuilder();
                 this.logBuffer.AppendLine(String.Format("CLI Query: {0}", query));
-                this.logBuffer.AppendLine(String.Format("User Query: {0}", encodeQueueTask.CustomQuery));
                 this.logBuffer.AppendLine();
 
                 // Clear the current Encode Logs)
@@ -343,7 +330,6 @@ namespace HandBrake.ApplicationServices.Services.Base
                 this.fileWriter = new StreamWriter(logFile) { AutoFlush = true };
                 this.fileWriter.WriteLine(header);
                 this.fileWriter.WriteLine(string.Format("CLI Query: {0}", query));
-                this.fileWriter.WriteLine(string.Format("User Query: {0}", encodeQueueTask.CustomQuery));
                 this.fileWriter.WriteLine();
             }
             catch (Exception)
@@ -385,7 +371,7 @@ namespace HandBrake.ApplicationServices.Services.Base
                         }
                     }
                 }
-                catch (Exception exc)
+                catch (Exception)
                 {
                     // Do Nothing.
                 }
