@@ -14,9 +14,8 @@ namespace HandBrake.ApplicationServices.Services
     using System.IO;
     using System.Windows.Forms;
 
-    using Caliburn.Micro;
-
     using HandBrake.ApplicationServices.EventArgs;
+    using HandBrake.ApplicationServices.Exceptions;
     using HandBrake.ApplicationServices.Model;
     using HandBrake.ApplicationServices.Services.Base;
     using HandBrake.ApplicationServices.Services.Interfaces;
@@ -29,11 +28,6 @@ namespace HandBrake.ApplicationServices.Services
     {
         #region Private Variables
 
-        /// <summary>
-        /// The User Setting Service
-        /// </summary>
-        private readonly IUserSettingService userSettingService;
-
         /// <summary>
         /// Gets the Process ID
         /// </summary>
@@ -56,18 +50,6 @@ namespace HandBrake.ApplicationServices.Services
 
         #endregion
 
-        /// <summary>
-        /// Initializes a new instance of the <see cref="Encode"/> class.
-        /// </summary>
-        /// <param name="userSettingService">
-        /// The user Setting Service.
-        /// </param>
-        public Encode(IUserSettingService userSettingService)
-            : base(userSettingService)
-        {
-            this.userSettingService = userSettingService;
-        }
-
         #region Properties
 
         /// <summary>
@@ -75,6 +57,22 @@ namespace HandBrake.ApplicationServices.Services
         /// </summary>
         protected Process HbProcess { get; set; }
 
+        /// <summary>
+        /// Gets a value indicating whether can pause.
+        /// </summary>
+        public bool CanPause
+        {
+            get
+            {
+                return false;
+            }
+        }
+
+        /// <summary>
+        /// Gets a value indicating whether is pasued.
+        /// </summary>
+        public bool IsPasued { get; private set; }
+
         #endregion
 
         #region Public Methods
@@ -86,23 +84,19 @@ namespace HandBrake.ApplicationServices.Services
         /// <param name="encodeQueueTask">
         /// The encodeQueueTask.
         /// </param>
-        /// <param name="enableLogging">
-        /// Enable Logging. When Disabled we onlt parse Standard Ouput for progress info. Standard Error log data is ignored.
-        /// </param>
-        public void Start(QueueTask encodeQueueTask, bool enableLogging)
+        public void Start(QueueTask encodeQueueTask)
         {
             try
             {
                 if (this.IsEncoding)
                 {
-                    throw new Exception("HandBrake is already encodeing.");
+                    throw new GeneralApplicationException("HandBrake is already encodeing.", "Please try again in a minute", null);
                 }
 
                 this.IsEncoding = true;
-
                 this.currentTask = encodeQueueTask;
 
-                if (enableLogging)
+                if (encodeQueueTask.Configuration.IsLoggingEnabled)
                 {
                     try
                     {
@@ -115,11 +109,6 @@ namespace HandBrake.ApplicationServices.Services
                     }
                 }
 
-                if (this.userSettingService.GetUserSetting<bool>(ASUserSettingConstants.PreventSleep))
-                {
-                    Win32.PreventSleep();
-                }
-
                 // Make sure the path exists, attempt to create it if it doesn't
                 this.VerifyEncodeDestinationPath(currentTask);
 
@@ -129,20 +118,15 @@ namespace HandBrake.ApplicationServices.Services
                 string query = this.currentTask.Task.IsPreviewEncode
                                    ? QueryGeneratorUtility.GeneratePreviewQuery(
                                        new EncodeTask(this.currentTask.Task),
+                                       encodeQueueTask.Configuration,
                                        this.currentTask.Task.PreviewEncodeDuration,
-                                       this.currentTask.Task.PreviewEncodeStartAt,
-                                       userSettingService.GetUserSetting<int>(ASUserSettingConstants.PreviewScanCount),
-                                       userSettingService.GetUserSetting<int>(ASUserSettingConstants.Verbosity),
-                                       userSettingService.GetUserSetting<bool>(ASUserSettingConstants.DisableLibDvdNav))
-                                   : QueryGeneratorUtility.GenerateQuery(new EncodeTask(this.currentTask.Task),
-                                   userSettingService.GetUserSetting<int>(ASUserSettingConstants.PreviewScanCount),
-                                   userSettingService.GetUserSetting<int>(ASUserSettingConstants.Verbosity),
-                                   userSettingService.GetUserSetting<bool>(ASUserSettingConstants.DisableLibDvdNav));
+                                       this.currentTask.Task.PreviewEncodeStartAt)
+                                   : QueryGeneratorUtility.GenerateQuery(new EncodeTask(this.currentTask.Task), encodeQueueTask.Configuration);
 
                 ProcessStartInfo cliStart = new ProcessStartInfo(handbrakeCLIPath, query)
                 {
                     RedirectStandardOutput = true,
-                    RedirectStandardError = enableLogging,
+                    RedirectStandardError = encodeQueueTask.Configuration.IsLoggingEnabled,
                     UseShellExecute = false,
                     CreateNoWindow = true
                 };
@@ -153,7 +137,7 @@ namespace HandBrake.ApplicationServices.Services
 
                 this.startTime = DateTime.Now;
 
-                if (enableLogging)
+                if (encodeQueueTask.Configuration.IsLoggingEnabled)
                 {
                     this.HbProcess.ErrorDataReceived += this.HbProcErrorDataReceived;
                     this.HbProcess.BeginErrorReadLine();
@@ -172,7 +156,7 @@ namespace HandBrake.ApplicationServices.Services
                 }
 
                 // Set the Process Priority
-                switch (this.userSettingService.GetUserSetting<string>(ASUserSettingConstants.ProcessPriority))
+                switch (encodeQueueTask.Configuration.ProcessPriority)
                 {
                     case "Realtime":
                         this.HbProcess.PriorityClass = ProcessPriorityClass.RealTime;
@@ -200,13 +184,36 @@ namespace HandBrake.ApplicationServices.Services
             catch (Exception exc)
             {
                 encodeQueueTask.Status = QueueItemStatus.Error;
+                this.IsEncoding = false;
                 this.InvokeEncodeCompleted(
                     new EncodeCompletedEventArgs(
-                        false, null, "An Error occured when trying to encode this source. "));
+                        false, exc, "An Error occured when trying to encode this source. ", this.currentTask.Task.Destination));
                 throw;
             }
         }
 
+        /// <summary>
+        /// The pause.
+        /// </summary>
+        /// <exception cref="NotImplementedException">
+        /// This feature is not available for CLI based encoding.
+        /// </exception>
+        public void Pause()
+        {
+            throw new NotImplementedException("This feature is not available for CLI based encoding.");
+        }
+
+        /// <summary>
+        /// The resume.
+        /// </summary>
+        /// <exception cref="NotImplementedException">
+        /// This feature is not available for CLI based encoding.
+        /// </exception>
+        public void Resume()
+        {
+            throw new NotImplementedException("This feature is not available for CLI based encoding.");
+        }
+
         /// <summary>
         /// Kill the CLI process
         /// </summary>
@@ -256,22 +263,14 @@ namespace HandBrake.ApplicationServices.Services
                 this.HbProcess.CancelOutputRead();
                 this.ShutdownFileWriter();
             }
-            catch (Exception exc)
+            catch (Exception)
             {
                 // This exception doesn't warrent user interaction, but it should be logged (TODO)
             }
 
-            Execute.OnUIThread(() =>
-            {
-                if (this.userSettingService.GetUserSetting<bool>(ASUserSettingConstants.PreventSleep))
-                {
-                    Win32.AllowSleep();
-                }
-            });
-
             this.currentTask.Status = QueueItemStatus.Completed;
             this.IsEncoding = false;
-            this.InvokeEncodeCompleted(new EncodeCompletedEventArgs(true, null, string.Empty));
+            this.InvokeEncodeCompleted(new EncodeCompletedEventArgs(true, null, string.Empty, this.currentTask.Task.Destination));
         }
 
         /// <summary>
