@@ -71,16 +71,33 @@ namespace HandBrake.ApplicationServices.Services.Interfaces
         /// </summary>
         int LogIndex { get; }
 
+        /// <summary>
+        /// Gets a value indicating whether can pause.
+        /// </summary>
+        bool CanPause { get; }
+
+        /// <summary>
+        /// Gets a value indicating whether is pasued.
+        /// </summary>
+        bool IsPasued { get; }
+
         /// <summary>
         /// Start with a LibHb EncodeJob Object
         /// </summary>
         /// <param name="job">
         /// The job.
         /// </param>
-        /// <param name="enableLogging">
-        /// The enable Logging.
-        /// </param>
-        void Start(QueueTask job, bool enableLogging);
+        void Start(QueueTask job);
+
+        /// <summary>
+        /// The pause.
+        /// </summary>
+        void Pause();
+
+        /// <summary>
+        /// The resume.
+        /// </summary>
+        void Resume();
 
         /// <summary>
         /// Kill the CLI process
@@ -93,7 +110,10 @@ namespace HandBrake.ApplicationServices.Services.Interfaces
         /// <param name="destination">
         /// The destination.
         /// </param>
-        void ProcessLogs(string destination);
+        /// <param name="configuration">
+        /// The configuration.
+        /// </param>
+        void ProcessLogs(string destination, HBConfiguration configuration);
 
         /// <summary>
         /// Shutdown the service.
