@@ -11,7 +11,6 @@ namespace HandBrake.ApplicationServices.Services
 {
     using System;
     using System.Diagnostics;
-    using System.Globalization;
 
     using HandBrake.ApplicationServices.Model;
     using HandBrake.ApplicationServices.Services.Base;
@@ -35,67 +34,75 @@ namespace HandBrake.ApplicationServices.Services
         /// <summary>
         /// Lock for the log file
         /// </summary>
-        private static readonly object logLock = new object();
+        private static readonly object LogLock = new object();
 
         /// <summary>
-        /// The User Setting Service
+        /// The instance.
         /// </summary>
-        private readonly IUserSettingService userSettingService;
+        private IHandBrakeInstance instance;
 
         /// <summary>
         /// The Start time of the current Encode;
         /// </summary>
         private DateTime startTime;
 
-        /// <summary>
-        /// An Instance of the HandBrake Interop Library
-        /// </summary>
-        private IHandBrakeInstance instance;
-
         /// <summary>
         /// A flag to indicate if logging is enabled or not.
         /// </summary>
         private bool loggingEnabled;
 
+        /// <summary>
+        /// The Current Task
+        /// </summary>
+        private QueueTask currentTask;
+
         #endregion
 
         /// <summary>
         /// Initializes a new instance of the <see cref="LibEncode"/> class.
         /// </summary>
-        /// <param name="userSettingService">
-        /// The user Setting Service.
-        /// </param>
-        /// <param name="handBrakeInstance">
-        /// The hand Brake Instance.
-        /// </param>
-        public LibEncode(IUserSettingService userSettingService, IHandBrakeInstance handBrakeInstance)
-            : base(userSettingService)
+        public LibEncode()
         {
-            this.userSettingService = userSettingService;
-
-            // Setup the HandBrake Instance
-            this.instance = handBrakeInstance;
-            this.instance.EncodeCompleted += this.InstanceEncodeCompleted;
-            this.instance.EncodeProgress += this.InstanceEncodeProgress;
-
             HandBrakeUtils.MessageLogged += this.HandBrakeInstanceMessageLogged;
             HandBrakeUtils.ErrorLogged += this.HandBrakeInstanceErrorLogged;
         }
 
+        /// <summary>
+        /// Gets a value indicating whether can pause.
+        /// </summary>
+        public bool CanPause
+        {
+            get
+            {
+                return true;
+            }
+        }
+
+        /// <summary>
+        /// Gets a value indicating whether is pasued.
+        /// </summary>
+        public bool IsPasued { get; private set; }
+
         /// <summary>
         /// Start with a LibHb EncodeJob Object
         /// </summary>
         /// <param name="job">
         /// The job.
         /// </param>
-        /// <param name="enableLogging">
-        /// The enable Logging.
-        /// </param>
-        public void Start(QueueTask job, bool enableLogging)
+        public void Start(QueueTask job)
         {
+            // Setup
             this.startTime = DateTime.Now;
-            this.loggingEnabled = enableLogging;
+            this.loggingEnabled = job.Configuration.IsLoggingEnabled;
+            this.currentTask = job;
 
+            // Create a new HandBrake instance
+            // Setup the HandBrake Instance
+            instance = new HandBrakeInstance();
+            instance.Initialize(1);
+            instance.EncodeCompleted += this.InstanceEncodeCompleted;
+            instance.EncodeProgress += this.InstanceEncodeProgress;
+            
             try
             {
                 // Sanity Checking and Setup
@@ -106,11 +113,8 @@ namespace HandBrake.ApplicationServices.Services
 
                 this.IsEncoding = true;
 
-                // Get an EncodeJob object for the Interop Library
-                EncodeJob encodeJob = InteropModelCreator.GetEncodeJob(job);
-
                 // Enable logging if required.
-                if (enableLogging)
+                if (job.Configuration.IsLoggingEnabled)
                 {
                     try
                     {
@@ -123,47 +127,45 @@ namespace HandBrake.ApplicationServices.Services
                     }
                 }
 
-                // Prvent the system from sleeping if the user asks
-                if (this.userSettingService.GetUserSetting<bool>(ASUserSettingConstants.PreventSleep) )
-                {
-                    Win32.PreventSleep();
-                }
-
                 // Verify the Destination Path Exists, and if not, create it.
                 this.VerifyEncodeDestinationPath(job);
 
-                // Start the Encode
-                this.instance.StartEncode(encodeJob);
+                // We have to scan the source again but only the title so the HandBrake instance is initialised correctly. 
+                // Since the UI sends the crop params down, we don't have to do all the previews.
+                instance.StartScan(job.Task.Source, job.Configuration.PreviewScanCount, job.Task.Title);
 
-                // Set the Process Priority
-                switch (this.userSettingService.GetUserSetting<string>(ASUserSettingConstants.ProcessPriority))
-                {
-                    case "Realtime":
-                        Process.GetCurrentProcess().PriorityClass = ProcessPriorityClass.RealTime;
-                        break;
-                    case "High":
-                        Process.GetCurrentProcess().PriorityClass = ProcessPriorityClass.High;
-                        break;
-                    case "Above Normal":
-                        Process.GetCurrentProcess().PriorityClass = ProcessPriorityClass.AboveNormal;
-                        break;
-                    case "Normal":
-                        Process.GetCurrentProcess().PriorityClass = ProcessPriorityClass.Normal;
-                        break;
-                    case "Low":
-                        Process.GetCurrentProcess().PriorityClass = ProcessPriorityClass.Idle;
-                        break;
-                    default:
-                        Process.GetCurrentProcess().PriorityClass = ProcessPriorityClass.BelowNormal;
-                        break;
-                }
-
-                // Fire the Encode Started Event
-                this.InvokeEncodeStarted(EventArgs.Empty);
+                instance.ScanCompleted += delegate
+                    {
+                        ScanCompleted(job, instance);
+                    };
             }
             catch (Exception exc)
             {
-                this.InvokeEncodeCompleted(new EncodeCompletedEventArgs(false, exc, "An Error has occured."));
+                this.InvokeEncodeCompleted(new EncodeCompletedEventArgs(false, exc, "An Error has occured.", this.currentTask.Task.Destination));
+            }
+        }
+
+        /// <summary>
+        /// Pause the currently running encode.
+        /// </summary>
+        public void Pause()
+        {
+            if (this.instance != null)
+            {
+                this.instance.PauseEncode();
+                this.IsPasued = true;
+            }
+        }
+
+        /// <summary>
+        /// Resume the currently running encode.
+        /// </summary>
+        public void Resume()
+        {
+            if (this.instance != null)
+            {
+                this.instance.ResumeEncode();
+                this.IsPasued = false;
             }
         }
 
@@ -191,6 +193,50 @@ namespace HandBrake.ApplicationServices.Services
             // Nothing to do for this implementation.
         }
 
+        /// <summary>
+        /// The scan completed.
+        /// </summary>
+        /// <param name="job">
+        /// The job.
+        /// </param>
+        /// <param name="instance">
+        /// The instance.
+        /// </param>
+        private void ScanCompleted(QueueTask job, IHandBrakeInstance instance)
+        {
+            // Get an EncodeJob object for the Interop Library
+            EncodeJob encodeJob = InteropModelCreator.GetEncodeJob(job);
+
+            // Start the Encode
+            instance.StartEncode(encodeJob, job.Configuration.PreviewScanCount);
+
+            // Fire the Encode Started Event
+            this.InvokeEncodeStarted(EventArgs.Empty);
+
+            // Set the Process Priority
+            switch (job.Configuration.ProcessPriority)
+            {
+                case "Realtime":
+                    Process.GetCurrentProcess().PriorityClass = ProcessPriorityClass.RealTime;
+                    break;
+                case "High":
+                    Process.GetCurrentProcess().PriorityClass = ProcessPriorityClass.High;
+                    break;
+                case "Above Normal":
+                    Process.GetCurrentProcess().PriorityClass = ProcessPriorityClass.AboveNormal;
+                    break;
+                case "Normal":
+                    Process.GetCurrentProcess().PriorityClass = ProcessPriorityClass.Normal;
+                    break;
+                case "Low":
+                    Process.GetCurrentProcess().PriorityClass = ProcessPriorityClass.Idle;
+                    break;
+                default:
+                    Process.GetCurrentProcess().PriorityClass = ProcessPriorityClass.BelowNormal;
+                    break;
+            }
+        }
+
         #region HandBrakeInstance Event Handlers.
         /// <summary>
         /// Log a message
@@ -205,7 +251,7 @@ namespace HandBrake.ApplicationServices.Services
         {
             if (this.loggingEnabled)
             {
-                lock (logLock)
+                lock (LogLock)
                 {
                     this.ProcessLogMessage(e.Message);
                 }
@@ -225,7 +271,7 @@ namespace HandBrake.ApplicationServices.Services
         {
             if (this.loggingEnabled)
             {
-                lock (logLock)
+                lock (LogLock)
                 {
                     this.ProcessLogMessage(e.Message);
                 }
@@ -271,13 +317,8 @@ namespace HandBrake.ApplicationServices.Services
 
             this.InvokeEncodeCompleted(
                 e.Error
-                    ? new EncodeCompletedEventArgs(false, null, string.Empty)
-                    : new EncodeCompletedEventArgs(true, null, string.Empty));
-
-            if (this.userSettingService.GetUserSetting<bool>(ASUserSettingConstants.PreventSleep))
-            {
-                Win32.AllowSleep();
-            }
+                    ? new EncodeCompletedEventArgs(false, null, string.Empty, this.currentTask.Task.Destination)
+                    : new EncodeCompletedEventArgs(true, null, string.Empty, this.currentTask.Task.Destination));
 
             this.ShutdownFileWriter();
         }
