@@ -11,21 +11,29 @@ namespace HandBrake.ApplicationServices.Services
 {
     using System;
     using System.Collections.Generic;
+    using System.IO;
     using System.Text;
-    using System.Threading;
+    using System.Text.RegularExpressions;
+    using System.Windows.Media.Imaging;
 
     using HandBrake.ApplicationServices.EventArgs;
-    using HandBrake.ApplicationServices.Model.Encoding;
+    using HandBrake.ApplicationServices.Model;
     using HandBrake.ApplicationServices.Parsing;
     using HandBrake.ApplicationServices.Services.Interfaces;
     using HandBrake.ApplicationServices.Utilities;
     using HandBrake.Interop;
     using HandBrake.Interop.EventArgs;
     using HandBrake.Interop.Interfaces;
+    using HandBrake.Interop.Model;
+    using HandBrake.Interop.SourceData;
 
     using AudioTrack = HandBrake.ApplicationServices.Parsing.Audio;
+    using Chapter = HandBrake.ApplicationServices.Parsing.Chapter;
     using ScanProgressEventArgs = HandBrake.Interop.EventArgs.ScanProgressEventArgs;
     using Size = System.Drawing.Size;
+    using Subtitle = HandBrake.ApplicationServices.Parsing.Subtitle;
+    using SubtitleType = HandBrake.ApplicationServices.Model.Encoding.SubtitleType;
+    using Title = HandBrake.ApplicationServices.Parsing.Title;
 
     /// <summary>
     /// Scan a Source
@@ -35,7 +43,7 @@ namespace HandBrake.ApplicationServices.Services
         /*
          * TODO
          * 1. Expose the Previews code.
-         * 2. Expose the Logging.
+         * 2. Cleanup old instances.
          * 
          */
 
@@ -46,11 +54,6 @@ namespace HandBrake.ApplicationServices.Services
         /// </summary>
         static readonly object LogLock = new object();
 
-        /// <summary>
-        /// LibHB Instance
-        /// </summary>
-        private readonly IHandBrakeInstance instance;
-
         /// <summary>
         /// Log data from HandBrakeInstance
         /// </summary>
@@ -66,27 +69,51 @@ namespace HandBrake.ApplicationServices.Services
         /// </summary>
         private string currentSourceScanPath;
 
+        /// <summary>
+        /// The log dir.
+        /// </summary>
+        private static string logDir = Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData) + "\\HandBrake\\logs";
+
+        /// <summary>
+        /// The dvd info path.
+        /// </summary>
+        private string dvdInfoPath = Path.Combine(logDir, string.Format("last_scan_log{0}.txt", GeneralUtilities.ProcessId));
+
+        /// <summary>
+        /// The scan log.
+        /// </summary>
+        private StreamWriter scanLog;
+
+        /// <summary>
+        /// LibHB Instance
+        /// </summary>
+        private IHandBrakeInstance instance;
+
+        /// <summary>
+        /// The post scan operation.
+        /// </summary>
+        private Action<bool> postScanOperation;
+
         #endregion
 
         /// <summary>
         /// Initializes a new instance of the <see cref="LibScan"/> class. 
         /// </summary>
-        /// <param name="handBrakeInstance">
-        /// The hand Brake Instance.
-        /// </param>
-        public LibScan(IHandBrakeInstance handBrakeInstance)
+        public LibScan()
         {
             logging = new StringBuilder();
 
             header = GeneralUtilities.CreateCliLogHeader();
 
-            instance = handBrakeInstance;
-            instance.Initialize(1);
-            instance.ScanProgress += this.InstanceScanProgress;
-            instance.ScanCompleted += this.InstanceScanCompleted;
-
-            HandBrakeUtils.MessageLogged += this.HandBrakeInstanceMessageLogged;
-            HandBrakeUtils.ErrorLogged += this.HandBrakeInstanceErrorLogged;
+            try
+            {
+                HandBrakeUtils.MessageLogged += this.HandBrakeInstanceMessageLogged;
+                HandBrakeUtils.ErrorLogged += this.HandBrakeInstanceErrorLogged;
+            }
+            catch (Exception)
+            {
+                // Do nothing. 
+            }
         }
 
         #region Events
@@ -127,7 +154,8 @@ namespace HandBrake.ApplicationServices.Services
         {
             get
             {
-                return string.IsNullOrEmpty(this.logging.ToString()) ? this.header + "No log data available..." : this.header + this.logging.ToString();
+                string noLog = "No log data available... Log data will show here after you scan a source. \n\nOpen the log file directory to get previous log files.";
+                return string.IsNullOrEmpty(this.logging.ToString()) ? this.header + noLog : this.header + this.logging.ToString();
             }
         }
 
@@ -145,16 +173,64 @@ namespace HandBrake.ApplicationServices.Services
         /// <param name="title">
         /// int title number. 0 for scan all
         /// </param>
-        /// <param name="previewCount">
-        /// The preview Count.
-        /// </param>
         /// <param name="postAction">
         /// The post Action.
         /// </param>
-        public void Scan(string sourcePath, int title, int previewCount, Action<bool> postAction)
+        /// <param name="configuraiton">
+        /// The configuraiton.
+        /// </param>
+        public void Scan(string sourcePath, int title, Action<bool> postAction, HBConfiguration configuraiton)
         {
-            Thread t = new Thread(unused => this.ScanSource(sourcePath, title, previewCount));
-            t.Start();
+            // Try to cleanup any previous scan instances.
+            if (instance != null)
+            {
+                try
+                {
+                    this.scanLog.Close();
+                    this.scanLog.Dispose();
+                    instance.Dispose();
+                }
+                catch (Exception)
+                {
+                    // Do Nothing
+                }
+            }
+
+            // Handle the post scan operation.
+            postScanOperation = postAction;
+
+            // Clear down the logging
+            this.logging.Clear();
+
+            try
+            {
+                // Make we don't pick up a stale last_scan_log_xyz.txt (and that we have rights to the file)
+                if (File.Exists(dvdInfoPath))
+                {
+                    File.Delete(dvdInfoPath);
+                }
+            }
+            catch (Exception)
+            {
+                // Do nothing.
+            }
+
+            if (!Directory.Exists(Path.GetDirectoryName(dvdInfoPath)))
+            {
+                Directory.CreateDirectory(Path.GetDirectoryName(dvdInfoPath));
+            }
+
+            // Create a new scan log.
+            scanLog = new StreamWriter(dvdInfoPath);
+
+            // Create a new HandBrake Instance.
+            instance = new HandBrakeInstance();
+            instance.Initialize(1);
+            instance.ScanProgress += this.InstanceScanProgress;
+            instance.ScanCompleted += this.InstanceScanCompleted;
+
+            // Start the scan on a back
+            this.ScanSource(sourcePath, title, configuraiton.PreviewScanCount, configuraiton);
         }
 
         /// <summary>
@@ -163,28 +239,53 @@ namespace HandBrake.ApplicationServices.Services
         public void Stop()
         {
             instance.StopScan();
+
+            try
+            {
+                if (this.scanLog != null)
+                {
+                    this.scanLog.Close();
+                    this.scanLog.Dispose();
+                }
+            }
+            catch (Exception)
+            {
+                // Do Nothing.
+            }
         }
 
         /// <summary>
-        /// Debug a Scan Log (Only Available for CLI Mode, not LIBHB)
+        /// Get a Preview image for the current job and preview number.
         /// </summary>
-        /// <param name="path">
-        /// The path.
+        /// <param name="job">
+        /// The job.
         /// </param>
-        /// <exception cref="NotImplementedException">
-        /// (Only Available for CLI Mode, not LIBHB)
-        /// </exception>
-        public void DebugScanLog(string path)
+        /// <param name="preview">
+        /// The preview.
+        /// </param>
+        /// <returns>
+        /// The <see cref="BitmapImage"/>.
+        /// </returns>
+        public BitmapImage GetPreview(EncodeTask job, int preview)
         {
-            throw new NotImplementedException("Only Available when using the CLI mode. Not LibHB");
-        }
+            if (this.instance == null)
+            {
+                return null;
+            }
 
-        /// <summary>
-        /// Shutdown the service.
-        /// </summary>
-        public void Shutdown()
-        {
-            // Nothing to do for this implementation.
+            EncodeJob encodeJob = InteropModelCreator.GetEncodeJob(job);
+
+            BitmapImage bitmapImage = null;
+            try
+            {
+                bitmapImage = this.instance.GetPreview(encodeJob, preview);
+            }
+            catch (AccessViolationException e)
+            {
+                Console.WriteLine(e);
+            }
+           
+            return bitmapImage;
         }
 
         #endregion
@@ -203,7 +304,10 @@ namespace HandBrake.ApplicationServices.Services
         /// <param name="previewCount">
         /// The preview Count.
         /// </param>
-        private void ScanSource(object sourcePath, int title, int previewCount)
+        /// <param name="configuraiton">
+        /// The configuraiton.
+        /// </param>
+        private void ScanSource(object sourcePath, int title, int previewCount, HBConfiguration configuraiton)
         {
             try
             {
@@ -217,10 +321,13 @@ namespace HandBrake.ApplicationServices.Services
                 if (this.ScanStared != null)
                     this.ScanStared(this, new EventArgs());
 
-                if (title != 0)
-                    instance.StartScan(sourcePath.ToString(), previewCount, title);
-                else
-                    instance.StartScan(sourcePath.ToString(), previewCount);
+                TimeSpan minDuration =
+                    TimeSpan.FromSeconds(
+                       configuraiton.MinScanDuration);
+
+                HandBrakeUtils.SetDvdNav(!configuraiton.IsDvdNavDisabled);
+
+                this.instance.StartScan(sourcePath.ToString(), previewCount, minDuration, title != 0 ? title : 0);
             }
             catch (Exception exc)
             {
@@ -245,6 +352,19 @@ namespace HandBrake.ApplicationServices.Services
         /// </param>
         private void InstanceScanCompleted(object sender, EventArgs e)
         {
+            // Write the log file out before we start processing incase we crash.
+            try
+            {
+                if (this.scanLog != null)
+                {
+                    this.scanLog.Flush();
+                }
+            }
+            catch (Exception)
+            {
+                // Do Nothing.
+            }
+
             // TODO -> Might be a better place to fix this.
             string path = currentSourceScanPath;
             if (currentSourceScanPath.Contains("\""))
@@ -252,12 +372,19 @@ namespace HandBrake.ApplicationServices.Services
                 path = currentSourceScanPath.Trim('\"');
             }
 
-            this.SouceData = new Source { Titles = ConvertTitles(this.instance.Titles), ScanPath = path };
+            // Process into internal structures.
+            this.SouceData = new Source { Titles = ConvertTitles(this.instance.Titles, this.instance.FeatureTitle), ScanPath = path };
 
             IsScanning = false;
 
-            if (this.ScanCompleted != null)
-                this.ScanCompleted(this, new ScanCompletedEventArgs(false, null, string.Empty));
+            if (postScanOperation != null)
+            {
+                postScanOperation(true);
+            }
+            else
+            {
+                if (this.ScanCompleted != null) this.ScanCompleted(this, new ScanCompletedEventArgs(false, null, string.Empty));
+            }
         }
 
         /// <summary>
@@ -277,7 +404,8 @@ namespace HandBrake.ApplicationServices.Services
                     new ApplicationServices.EventArgs.ScanProgressEventArgs
                         {
                             CurrentTitle = e.CurrentTitle,
-                            Titles = e.Titles
+                            Titles = e.Titles,
+                            Percentage = Math.Round((decimal)e.Progress * 100, 0)
                         };
 
                 this.ScanStatusChanged(this, eventArgs);
@@ -297,6 +425,11 @@ namespace HandBrake.ApplicationServices.Services
         {
             lock (LogLock)
             {
+                if (this.scanLog != null)
+                {
+                    this.scanLog.WriteLine(e.Message);
+                }
+
                 this.logging.AppendLine(e.Message);
             }
         }
@@ -314,6 +447,11 @@ namespace HandBrake.ApplicationServices.Services
         {
             lock (LogLock)
             {
+                if (this.scanLog != null)
+                {
+                    this.scanLog.WriteLine(e.Message);
+                }
+
                 this.logging.AppendLine(e.Message);
             }
         }
@@ -324,29 +462,36 @@ namespace HandBrake.ApplicationServices.Services
         /// <param name="titles">
         /// The titles.
         /// </param>
+        /// <param name="featureTitle">
+        /// The feature Title.
+        /// </param>
         /// <returns>
         /// The convert titles.
         /// </returns>
-        private static List<Title> ConvertTitles(IEnumerable<Interop.SourceData.Title> titles)
+        private static List<Title> ConvertTitles(IEnumerable<Interop.SourceData.Title> titles, int featureTitle)
         {
             List<Title> titleList = new List<Title>();
             foreach (Interop.SourceData.Title title in titles)
             {
                 Title converted = new Title
                     {
-                        TitleNumber = title.TitleNumber, 
-                        Duration = title.Duration, 
-                        Resolution = new Size(title.Resolution.Width, title.Resolution.Height), 
-                        AspectRatio = title.AspectRatio, 
-                        AngleCount = title.AngleCount, 
-                        ParVal = new Size(title.ParVal.Width, title.ParVal.Height), 
+                        TitleNumber = title.TitleNumber,
+                        Duration = title.Duration,
+                        Resolution = new Size(title.Resolution.Width, title.Resolution.Height),
+                        AspectRatio = title.AspectRatio,
+                        AngleCount = title.AngleCount,
+                        ParVal = new Size(title.ParVal.Width, title.ParVal.Height),
                         AutoCropDimensions = title.AutoCropDimensions,
-                        Fps = title.Framerate
+                        Fps = title.Framerate,
+                        SourceName = title.Path,
+                        MainTitle = title.TitleNumber == featureTitle,
+                        Playlist = title.InputType == InputType.Bluray ? string.Format(" {0:d5}.MPLS", title.Playlist).Trim() : null
                     };
 
                 foreach (Interop.SourceData.Chapter chapter in title.Chapters)
                 {
-                    converted.Chapters.Add(new Chapter(chapter.ChapterNumber, string.Empty, chapter.Duration));
+                    string chapterName = !string.IsNullOrEmpty(chapter.Name) ? chapter.Name : string.Empty;
+                    converted.Chapters.Add(new Chapter(chapter.ChapterNumber, chapterName, chapter.Duration));
                 }
 
                 foreach (Interop.SourceData.AudioTrack track in title.AudioTracks)
@@ -381,9 +526,12 @@ namespace HandBrake.ApplicationServices.Services
                         case Interop.SourceData.SubtitleSource.CC708:
                             convertedType = SubtitleType.CC;
                             break;
+                        case Interop.SourceData.SubtitleSource.PGS:
+                            convertedType = SubtitleType.PGS;
+                            break;
                     }
 
-                    converted.Subtitles.Add(new Subtitle(track.TrackNumber, track.Language, track.LanguageCode, convertedType));
+                    converted.Subtitles.Add(new Subtitle(track.SubtitleSourceInt, track.TrackNumber, track.Language, track.LanguageCode, convertedType, track.CanBurn, track.CanSetForcedOnly));
                 }
 
                 titleList.Add(converted);
