@@ -12,10 +12,8 @@ namespace HandBrake.ApplicationServices.Services
     using System;
     using System.Collections.Generic;
     using System.ComponentModel;
-    using System.Diagnostics;
     using System.IO;
     using System.Linq;
-    using System.Windows.Forms;
     using System.Xml.Serialization;
 
     using Caliburn.Micro;
@@ -43,15 +41,15 @@ namespace HandBrake.ApplicationServices.Services
         /// </summary>
         private readonly BindingList<QueueTask> queue = new BindingList<QueueTask>();
 
-        /// <summary>
-        /// The User Setting Service
-        /// </summary>
-        private readonly IUserSettingService userSettingService;
-
         /// <summary>
         /// HandBrakes Queue file with a place holder for an extra string.
         /// </summary>
-        private string queueFile;
+        private readonly string queueFile;
+
+        /// <summary>
+        /// The clear completed.
+        /// </summary>
+        private bool clearCompleted;
 
         #endregion
 
@@ -63,15 +61,11 @@ namespace HandBrake.ApplicationServices.Services
         /// <param name="encodeService">
         /// The encode Service.
         /// </param>
-        /// <param name="userSettingService">
-        /// The user Setting Service.
-        /// </param>
         /// <exception cref="ArgumentNullException">
         /// Services are not setup
         /// </exception>
-        public QueueProcessor(IEncodeServiceWrapper encodeService, IUserSettingService userSettingService)
+        public QueueProcessor(IEncodeServiceWrapper encodeService)
         {
-            this.userSettingService = userSettingService;
             this.EncodeService = encodeService;
 
             // If this is the first instance, just use the main queue file, otherwise add the instance id to the filename.
@@ -93,6 +87,17 @@ namespace HandBrake.ApplicationServices.Services
         /// </param>
         public delegate void QueueProgressStatus(object sender, QueueProgressEventArgs e);
 
+        /// <summary>
+        /// The queue completed.
+        /// </summary>
+        /// <param name="sender">
+        /// The sender.
+        /// </param>
+        /// <param name="e">
+        /// The e.
+        /// </param>
+        public delegate void QueueCompletedEventDelegate(object sender, QueueCompletedEventArgs e);
+
         #endregion
 
         #region Events
@@ -111,7 +116,7 @@ namespace HandBrake.ApplicationServices.Services
         /// <summary>
         /// Fires when the entire encode queue has completed.
         /// </summary>
-        public event EventHandler QueueCompleted;
+        public event QueueCompletedEventDelegate QueueCompleted;
 
         /// <summary>
         /// Fires when a pause to the encode queue has been requested.
@@ -215,7 +220,7 @@ namespace HandBrake.ApplicationServices.Services
         /// </returns>
         public bool CheckForDestinationPathDuplicates(string destination)
         {
-            return this.queue.Any(job => job.Task != null && job.Task.Destination.Contains(destination.Replace("\\\\", "\\")));
+            return this.queue.Any(job => job.Task != null && job.Status == QueueItemStatus.Waiting && job.Task.Destination != null && job.Task.Destination.Contains(destination.Replace("\\\\", "\\")));
         }
 
         /// <summary>
@@ -238,15 +243,15 @@ namespace HandBrake.ApplicationServices.Services
         {
             Execute.OnUIThread(
                 () =>
+                {
+                    List<QueueTask> deleteList =
+                        this.queue.Where(task => task.Status == QueueItemStatus.Completed).ToList();
+                    foreach (QueueTask item in deleteList)
                     {
-                        List<QueueTask> deleteList =
-                            this.queue.Where(task => task.Status == QueueItemStatus.Completed).ToList();
-                        foreach (QueueTask item in deleteList)
-                        {
-                            this.queue.Remove(item);
-                        }
-                        this.InvokeQueueChanged(EventArgs.Empty);
-                    });
+                        this.queue.Remove(item);
+                    }
+                    this.InvokeQueueChanged(EventArgs.Empty);
+                });
         }
 
         /// <summary>
@@ -381,8 +386,8 @@ namespace HandBrake.ApplicationServices.Services
                         catch (Exception exc)
                         {
                             throw new GeneralApplicationException(
-                                "Unable to restore queue file.", 
-                                "The file may be corrupted or from an older incompatible version of HandBrake", 
+                                "Unable to restore queue file.",
+                                "The file may be corrupted or from an older incompatible version of HandBrake",
                                 exc);
                         }
 
@@ -427,16 +432,32 @@ namespace HandBrake.ApplicationServices.Services
         /// Starts encoding the first job in the queue and continues encoding until all jobs
         /// have been encoded.
         /// </summary>
-        public void Start()
+        /// <param name="isClearCompleted">
+        /// The is Clear Completed.
+        /// </param>
+        public void Start(bool isClearCompleted)
         {
             if (this.IsProcessing)
             {
                 throw new Exception("Already Processing the Queue");
             }
 
-            this.IsProcessing = true;
+            clearCompleted = isClearCompleted;
+
+            this.EncodeService.EncodeCompleted -= this.EncodeServiceEncodeCompleted;
             this.EncodeService.EncodeCompleted += this.EncodeServiceEncodeCompleted;
-            this.ProcessNextJob();
+
+            if (this.EncodeService.CanPause && this.EncodeService.IsEncoding)
+            {
+                this.EncodeService.Resume();
+            }
+
+            if (!this.EncodeService.IsEncoding)
+            {
+                this.ProcessNextJob();
+            }
+
+            this.IsProcessing = true;
         }
 
         #endregion
@@ -457,7 +478,7 @@ namespace HandBrake.ApplicationServices.Services
             this.LastProcessedJob.Status = QueueItemStatus.Completed;
 
             // Clear the completed item of the queue if the setting is set.
-            if (this.userSettingService.GetUserSetting<bool>(ASUserSettingConstants.ClearCompletedFromQueue))
+            if (clearCompleted)
             {
                 this.ClearCompleted();
             }
@@ -469,13 +490,7 @@ namespace HandBrake.ApplicationServices.Services
             }
 
             // Handling Log Data 
-            this.EncodeService.ProcessLogs(this.LastProcessedJob.Task.Destination);
-
-            // Post-Processing
-            if (e.Successful)
-            {
-                this.SendToApplication(this.LastProcessedJob.Task.Destination);
-            }
+            this.EncodeService.ProcessLogs(this.LastProcessedJob.Task.Destination, this.LastProcessedJob.Configuration);
 
             // Move onto the next job.
             if (this.IsProcessing)
@@ -485,40 +500,11 @@ namespace HandBrake.ApplicationServices.Services
             else
             {
                 this.EncodeService.EncodeCompleted -= this.EncodeServiceEncodeCompleted;
-                this.InvokeQueueCompleted(EventArgs.Empty);
+                this.OnQueueCompleted(new QueueCompletedEventArgs(true));
                 this.BackupQueue(string.Empty);
             }
         }
 
-        /// <summary>
-        /// Perform an action after an encode. e.g a shutdown, standby, restart etc.
-        /// </summary>
-        private void Finish()
-        {
-            // Do something whent he encode ends.
-            switch (this.userSettingService.GetUserSetting<string>(ASUserSettingConstants.WhenCompleteAction))
-            {
-                case "Shutdown":
-                    Process.Start("Shutdown", "-s -t 60");
-                    break;
-                case "Log off":
-                    Win32.ExitWindowsEx(0, 0);
-                    break;
-                case "Suspend":
-                    Application.SetSuspendState(PowerState.Suspend, true, true);
-                    break;
-                case "Hibernate":
-                    Application.SetSuspendState(PowerState.Hibernate, true, true);
-                    break;
-                case "Lock System":
-                    Win32.LockWorkStation();
-                    break;
-                case "Quit HandBrake":
-                    Execute.OnUIThread(Application.Exit);
-                    break;
-            }
-        }
-
         /// <summary>
         /// Invoke the JobProcessingStarted event
         /// </summary>
@@ -558,23 +544,6 @@ namespace HandBrake.ApplicationServices.Services
             }
         }
 
-        /// <summary>
-        /// Invoke the QueueCompleted event.
-        /// </summary>
-        /// <param name="e">
-        /// The EventArgs.
-        /// </param>
-        private void InvokeQueueCompleted(EventArgs e)
-        {
-            this.IsProcessing = false;
-
-            EventHandler handler = this.QueueCompleted;
-            if (handler != null)
-            {
-                handler(this, e);
-            }
-        }
-
         /// <summary>
         /// Invoke the QueuePaused event
         /// </summary>
@@ -583,6 +552,8 @@ namespace HandBrake.ApplicationServices.Services
         /// </param>
         private void InvokeQueuePaused(EventArgs e)
         {
+            this.IsProcessing = false;
+
             EventHandler handler = this.QueuePaused;
             if (handler != null)
             {
@@ -590,6 +561,23 @@ namespace HandBrake.ApplicationServices.Services
             }
         }
 
+        /// <summary>
+        /// The on queue completed.
+        /// </summary>
+        /// <param name="e">
+        /// The e.
+        /// </param>
+        protected virtual void OnQueueCompleted(QueueCompletedEventArgs e)
+        {
+            QueueCompletedEventDelegate handler = this.QueueCompleted;
+            if (handler != null)
+            {
+                handler(this, e);
+            }
+
+            this.IsProcessing = false;
+        }
+
         /// <summary>
         /// Run through all the jobs on the queue.
         /// </summary>
@@ -599,7 +587,7 @@ namespace HandBrake.ApplicationServices.Services
             if (job != null)
             {
                 this.InvokeJobProcessingStarted(new QueueProgressEventArgs(job));
-                this.EncodeService.Start(job, true);
+                this.EncodeService.Start(job);
             }
             else
             {
@@ -607,32 +595,7 @@ namespace HandBrake.ApplicationServices.Services
                 this.EncodeService.EncodeCompleted -= this.EncodeServiceEncodeCompleted;
 
                 // Fire the event to tell connected services.
-                this.InvokeQueueCompleted(EventArgs.Empty);
-
-                // Run the After encode completeion work
-                this.Finish();
-            }
-        }
-
-        /// <summary>
-        /// Send a file to a 3rd party application after encoding has completed.
-        /// </summary>
-        /// <param name="file">
-        /// The file path
-        /// </param>
-        private void SendToApplication(string file)
-        {
-            if (this.userSettingService.GetUserSetting<bool>(ASUserSettingConstants.SendFile) &&
-                !string.IsNullOrEmpty(this.userSettingService.GetUserSetting<string>(ASUserSettingConstants.SendFileTo)))
-            {
-                string args = string.Format(
-                    "{0} \"{1}\"", 
-                    this.userSettingService.GetUserSetting<string>(ASUserSettingConstants.SendFileToArgs), 
-                    file);
-                var vlc =
-                    new ProcessStartInfo(
-                        this.userSettingService.GetUserSetting<string>(ASUserSettingConstants.SendFileTo), args);
-                Process.Start(vlc);
+                this.OnQueueCompleted(new QueueCompletedEventArgs(false));
             }
         }
 
