@@ -15,6 +15,7 @@ namespace HandBrake.ApplicationServices.Utilities
     using HandBrake.ApplicationServices.Model.Encoding;
     using HandBrake.Interop.Model.Encoding;
     using HandBrake.Interop.Model.Encoding.x264;
+    using HandBrake.Interop.Model.Encoding.x265;
 
     /// <summary>
     /// A class to convert various things to native C# objects
@@ -25,7 +26,7 @@ namespace HandBrake.ApplicationServices.Utilities
          * TODO:
          * - Many of these converters can be ditched at a later point. Should be able to model all this within the enums themsevles.
          * 
-         **/ 
+         **/
 
         /// <summary>
         /// Convert HandBrakes time remaining into a TimeSpan
@@ -129,7 +130,7 @@ namespace HandBrake.ApplicationServices.Utilities
             {
                 case Mixdown.Auto:
                 case Mixdown.None:
-                    return "auto";
+                    return "none";
                 case Mixdown.Mono:
                     return "mono";
                 case Mixdown.LeftOnly:
@@ -155,44 +156,6 @@ namespace HandBrake.ApplicationServices.Utilities
             }
         }
 
-        /// <summary>
-        /// Get the GUI equiv to a CLI audio encoder
-        /// </summary>
-        /// <param name="audioEnc">The Audio Encoder</param>
-        /// <returns>The GUI representation of that audio encoder</returns>
-        public static string GetGUIAudioEncoder(string audioEnc)
-        {
-            switch (audioEnc)
-            {
-                case "faac":
-                    return "AAC (faac)";
-                case "lame":
-                    return "MP3 (lame)";
-                case "vorbis":
-                    return "Vorbis (vorbis)";
-                case "ffac3":
-                    return "AC3 (ffmpeg)";
-                case "copy:ac3":
-                    return "AC3 Passthru";
-                case "copy:dts":
-                    return "DTS Passthru";
-                case "copy:dtshd":
-                    return "MP3 Passthru";
-                case "copy:mp3":
-                    return "AAC Passthru";
-                case "copy:aac":
-                    return "DTS-HD Passthru";
-                case "ffaac":
-                    return "AAC (ffmpeg)";
-                case "ffflac":
-                    return "Flac (ffmpeg)";
-                case "copy":
-                    return "Auto Passthru";
-                default:
-                    return "AAC (faac)";
-            }
-        }
-
         /// <summary>
         /// Get the GUI equiv to a CLI audio encoder
         /// </summary>
@@ -203,17 +166,26 @@ namespace HandBrake.ApplicationServices.Utilities
             switch (audioEnc)
             {
                 case "faac":
-                    return AudioEncoder.Faac;
                 case "ffaac":
                     return AudioEncoder.ffaac;
+                case "fdk_aac":
+                    return AudioEncoder.fdkaac;
+                case "fdk_haac":
+                    return AudioEncoder.fdkheaac;
+                case "mp3":
                 case "lame":
                     return AudioEncoder.Lame;
                 case "vorbis":
                     return AudioEncoder.Vorbis;
+                case "ac3":
                 case "ffac3":
                     return AudioEncoder.Ac3;
+                case "flac16":
                 case "ffflac":
                     return AudioEncoder.ffflac;
+                case "flac24":
+                case "ffflac24":
+                    return AudioEncoder.ffflac24;
                 case "copy:ac3":
                     return AudioEncoder.Ac3Passthrough;
                 case "copy:dts":
@@ -227,7 +199,7 @@ namespace HandBrake.ApplicationServices.Utilities
                 case "copy":
                     return AudioEncoder.Passthrough;
                 default:
-                    return AudioEncoder.Faac;
+                    return AudioEncoder.ffaac;
             }
         }
 
@@ -240,17 +212,25 @@ namespace HandBrake.ApplicationServices.Utilities
         {
             switch (audioEnc)
             {
-                case "AAC (faac)":
-                    return AudioEncoder.Faac;
+                case "AAC (faac)":         
                 case "AAC (ffmpeg)":
+                case "AAC (avcodec)":
                     return AudioEncoder.ffaac;
+                case "AAC (FDK)":
                 case "AAC (CoreAudio)":
-                    return AudioEncoder.Faac;
+                    return AudioEncoder.fdkaac;
+                case "HE-AAC (FDK)":
+                case "HE-AAC (CoreAudio)":
+                case "HE-AAC":
+                    return AudioEncoder.fdkheaac;
                 case "MP3 (lame)":
+                case "MP3":
                     return AudioEncoder.Lame;
                 case "Vorbis (vorbis)":
+                case "Vorbis":
                     return AudioEncoder.Vorbis;
                 case "AC3 (ffmpeg)":
+                case "AC3":
                     return AudioEncoder.Ac3;
                 case "AC3 Passthru":
                     return AudioEncoder.Ac3Passthrough;
@@ -262,12 +242,16 @@ namespace HandBrake.ApplicationServices.Utilities
                     return AudioEncoder.AacPassthru;
                 case "MP3 Passthru":
                     return AudioEncoder.Mp3Passthru;
-                case "Flac (ffmpeg)":
+                case "FLAC (ffmpeg)":
+                case "FLAC 16-bit":
                     return AudioEncoder.ffflac;
+                case "FLAC (24-bit)":
+                case "FLAC 24-bit":
+                    return AudioEncoder.ffflac24;
                 case "Auto Passthru":
                     return AudioEncoder.Passthrough;
                 default:
-                    return AudioEncoder.Faac;
+                    return AudioEncoder.ffaac;
             }
         }
 
@@ -282,40 +266,12 @@ namespace HandBrake.ApplicationServices.Utilities
         /// </returns>
         public static string GetCliAudioEncoder(AudioEncoder selectedEncoder)
         {
-            switch (selectedEncoder)
-            {
-                case AudioEncoder.Faac:
-                    return "faac";
-                case AudioEncoder.ffaac:
-                    return "ffaac";
-                case AudioEncoder.Lame:
-                    return "lame";
-                case AudioEncoder.Vorbis:
-                    return "vorbis";
-                case AudioEncoder.Ac3Passthrough:
-                    return "copy:ac3";
-                case AudioEncoder.DtsPassthrough:
-                    return "copy:dts";
-                case AudioEncoder.DtsHDPassthrough:
-                    return "copy:dtshd";
-                case AudioEncoder.Ac3:
-                    return "ffac3";
-                case AudioEncoder.AacPassthru:
-                    return "copy:aac";
-                case AudioEncoder.Mp3Passthru:
-                    return "copy:mp3";
-                case AudioEncoder.Passthrough:
-                    return "copy";
-                case AudioEncoder.ffflac:
-                    return "ffflac";
-                default:
-                    return "faac";
-            }
+            return EnumHelper<AudioEncoder>.GetShortName(selectedEncoder);
         }
 
         #endregion
 
-        #region Video 
+        #region Video
 
         /// <summary>
         /// Get the Video Encoder for a given string
@@ -330,16 +286,23 @@ namespace HandBrake.ApplicationServices.Utilities
         {
             switch (encoder)
             {
-                case "":
                 case "ffmpeg":
                 case "ffmpeg4":
+                case "mpeg4":
                     return VideoEncoder.FFMpeg;
                 case "ffmpeg2":
+                case "mpeg2":
                     return VideoEncoder.FFMpeg2;
                 case "x264":
                     return VideoEncoder.X264;
+                case "qsv_h264":
+                    return VideoEncoder.QuickSync;
                 case "theora":
                     return VideoEncoder.Theora;
+                case "x265":
+                    return VideoEncoder.X265;
+                case "VP8":
+                    return VideoEncoder.VP8;
                 default:
                     return VideoEncoder.X264;
             }
@@ -359,13 +322,19 @@ namespace HandBrake.ApplicationServices.Utilities
             switch (encoder)
             {
                 case VideoEncoder.FFMpeg:
-                    return "ffmpeg4";
+                    return "mpeg4";
                 case VideoEncoder.FFMpeg2:
-                    return "ffmpeg2";
+                    return "mpeg2";
                 case VideoEncoder.X264:
                     return "x264";
+                case VideoEncoder.QuickSync:
+                    return "qsv_h264";
                 case VideoEncoder.Theora:
                     return "theora";
+                case VideoEncoder.X265:
+                    return "x265";
+                case VideoEncoder.VP8:
+                    return "VP8";
                 default:
                     return "x264";
             }
@@ -432,12 +401,6 @@ namespace HandBrake.ApplicationServices.Utilities
                     return x264Profile.Main;
                 case "high":
                     return x264Profile.High;
-                //case "high10":
-                //    return x264Profile.High10;
-                //case "high422":
-                //    return x264Profile.High422;
-                //case "high444":
-                //    return x264Profile.High444;
                 default:
                     return x264Profile.Main;
             }
@@ -477,6 +440,94 @@ namespace HandBrake.ApplicationServices.Utilities
 
         #endregion
 
+        #region x265
+
+        /// <summary>
+        /// Get the x265Preset from a cli parameter
+        /// </summary>
+        /// <param name="preset">
+        /// The preset.
+        /// </param>
+        /// <returns>
+        /// The x265Preset enum value
+        /// </returns>
+        public static x265Preset Getx265PresetFromCli(string preset)
+        {
+            switch (preset)
+            {
+                case "ultrafast":
+                    return x265Preset.Ultrafast;
+                case "superfast":
+                    return x265Preset.Superfast;
+                case "veryfast":
+                    return x265Preset.VeryFast;
+                case "faster":
+                    return x265Preset.Faster;
+                case "fast":
+                    return x265Preset.Fast;
+                case "medium":
+                    return x265Preset.Medium;
+                case "slow":
+                    return x265Preset.Slow;
+                case "slower":
+                    return x265Preset.Slower;
+                case "veryslow":
+                    return x265Preset.VerySlow;
+                case "placebo":
+                    return x265Preset.Placebo;
+                default:
+                    return x265Preset.Faster;
+            }
+        }
+
+        /// <summary>
+        /// Get the x265 Profile from the cli
+        /// </summary>
+        /// <param name="profile">
+        /// The preset.
+        /// </param>
+        /// <returns>
+        /// The x265Profile enum value
+        /// </returns>
+        public static x265Profile Getx265ProfileFromCli(string profile)
+        {
+            switch (profile)
+            {
+                case "mainstillpicture":
+                    return x265Profile.Mainstillpicture;
+                case "main":
+                    return x265Profile.Main;
+                case "main10":
+                    return x265Profile.Main10;
+                default:
+                    return x265Profile.Main;
+            }
+        }
+
+        /// <summary>
+        /// Get x265Tune enum from a cli string
+        /// </summary>
+        /// <param name="tune">
+        /// The tune.
+        /// </param>
+        /// <returns>
+        /// The x265Tune enum value
+        /// </returns>
+        public static x265Tune Getx265TuneFromCli(string tune)
+        {
+            switch (tune)
+            {
+                case "psnr":
+                    return x265Tune.Psnr;
+                case "ssim":
+                    return x265Tune.Ssim;
+                default:
+                    return x265Tune.Psnr;
+            }
+        }
+
+        #endregion
+
         #region File Format
 
         /// <summary>
@@ -495,7 +546,7 @@ namespace HandBrake.ApplicationServices.Utilities
                 default:
                     return OutputFormat.Mp4;
                 case "m4v":
-                    return OutputFormat.M4V;
+                    return OutputFormat.Mp4;
                 case "mkv":
                     return OutputFormat.Mkv;
             }
@@ -516,7 +567,7 @@ namespace HandBrake.ApplicationServices.Utilities
             {
                 default:
                     return "mp4";
-                case OutputFormat.M4V:
+                case OutputFormat.Mp4:
                     return "m4v";
                 case OutputFormat.Mkv:
                     return "mkv";
