@@ -12,11 +12,9 @@ namespace HandBrake.ApplicationServices.Utilities
     using System;
     using System.Collections.Generic;
     using System.Diagnostics;
-    using System.Globalization;
     using System.IO;
     using System.Linq;
     using System.Text;
-    using System.Text.RegularExpressions;
     using System.Windows.Forms;
 
     using HandBrake.ApplicationServices.Model;
@@ -33,6 +31,11 @@ namespace HandBrake.ApplicationServices.Utilities
         /// </summary>
         private static readonly string LogDir = Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData) + "\\HandBrake\\logs";
 
+        /// <summary>
+        /// The is lib hb present.
+        /// </summary>
+        private static bool? isLibHbPresent;
+
         #endregion
 
         #region Properties
@@ -48,6 +51,22 @@ namespace HandBrake.ApplicationServices.Utilities
             }
         }
 
+        /// <summary>
+        /// Gets a value indicating whether is lib hb present.
+        /// </summary>
+        public static bool IsLibHbPresent
+        {
+            get
+            {
+                if (isLibHbPresent == null)
+                {
+                    isLibHbPresent = File.Exists(Path.Combine(Application.StartupPath, "hb.dll"));
+                }
+
+                return isLibHbPresent.Value;
+            }
+        }
+
         #endregion
 
         #region Public Methods
@@ -69,13 +88,20 @@ namespace HandBrake.ApplicationServices.Utilities
                 // Delete old and excessivly large files (> ~50MB).
                 foreach (FileInfo file in logFiles)
                 {
-                    if (file.LastWriteTime < DateTime.Now.AddDays(-daysToKeep))
+                    try
                     {
-                        File.Delete(file.FullName);
+                        if (file.LastWriteTime < DateTime.Now.AddDays(-daysToKeep))
+                        {
+                            File.Delete(file.FullName);
+                        }
+                        else if (file.Length > 50000000)
+                        {
+                            File.Delete(file.FullName);
+                        }
                     }
-                    else if (file.Length > 50000000)
+                    catch (Exception)
                     {
-                        File.Delete(file.FullName);
+                        // Silently ignore files we can't delete. They are probably being used by the app right now.
                     }
                 }
             }
@@ -91,10 +117,17 @@ namespace HandBrake.ApplicationServices.Utilities
         {
             var logHeader = new StringBuilder();
 
+            StringBuilder gpuBuilder = new StringBuilder();
+            foreach (var item in SystemInfo.GetGPUInfo)
+            {
+                gpuBuilder.AppendLine(string.Format("  {0}", item));
+            }
+
             logHeader.AppendLine(String.Format("HandBrake {0} - {1}", VersionHelper.GetVersion(), VersionHelper.GetPlatformBitnessVersion()));
-            logHeader.AppendLine(String.Format("OS: {0}", Environment.OSVersion));
+            logHeader.AppendLine(String.Format("OS: {0} - {1}", Environment.OSVersion, Environment.Is64BitOperatingSystem ? "64bit" : "32bit"));
             logHeader.AppendLine(String.Format("CPU: {0}", SystemInfo.GetCpuCount));
-            logHeader.Append(String.Format("Ram: {0} MB, ", SystemInfo.TotalPhysicalMemory));
+            logHeader.AppendLine(String.Format("Ram: {0} MB, ", SystemInfo.TotalPhysicalMemory));
+            logHeader.AppendLine(String.Format("GPU Information:{0}{1}", Environment.NewLine, gpuBuilder.ToString().TrimEnd()));
             logHeader.AppendLine(String.Format("Screen: {0}x{1}", SystemInfo.ScreenBounds.Bounds.Width, SystemInfo.ScreenBounds.Bounds.Height));
             logHeader.AppendLine(String.Format("Temp Dir: {0}", Path.GetTempPath()));
             logHeader.AppendLine(String.Format("Install Dir: {0}", Application.StartupPath));
