@@ -19,6 +19,7 @@ namespace HandBrake.ApplicationServices.Utilities
     using HandBrake.ApplicationServices.Model.Encoding;
     using HandBrake.Interop.Model.Encoding;
     using HandBrake.Interop.Model.Encoding.x264;
+    using HandBrake.Interop.Model.Encoding.x265;
 
     /// <summary>
     /// Generate a CLI Query for HandBrakeCLI
@@ -31,19 +32,13 @@ namespace HandBrake.ApplicationServices.Utilities
         /// <param name="task">
         /// The task.
         /// </param>
-        /// <param name="previewScanCount">
-        /// The preview Scan Count.
-        /// </param>
-        /// <param name="verbosity">
-        /// The verbosity.
-        /// </param>
-        /// <param name="disableLibDvdNav">
-        /// The disable Lib Dvd Nav.
+        /// <param name="configuration">
+        /// The configuration.
         /// </param>
         /// <returns>
         /// A Cli Query
         /// </returns>
-        public static string GenerateQuery(EncodeTask task, int previewScanCount, int verbosity, bool disableLibDvdNav)
+        public static string GenerateQuery(EncodeTask task, HBConfiguration configuration)
         {
             if (string.IsNullOrEmpty(task.Source))
             {
@@ -51,9 +46,9 @@ namespace HandBrake.ApplicationServices.Utilities
             }
 
             string query = string.Empty;
-            query += SourceQuery(task, null, null, previewScanCount);
+            query += SourceQuery(task, null, null, configuration.PreviewScanCount);
             query += DestinationQuery(task);
-            query += GenerateTabbedComponentsQuery(task, true, verbosity, disableLibDvdNav);
+            query += GenerateTabbedComponentsQuery(task, true, configuration.Verbosity, configuration.IsDvdNavDisabled, configuration.DisableQuickSyncDecoding, configuration.EnableDxva, configuration.ScalingMode == VideoScaler.BicubicCl);
 
             return query;
         }
@@ -64,30 +59,24 @@ namespace HandBrake.ApplicationServices.Utilities
         /// <param name="task">
         /// The task.
         /// </param>
+        /// <param name="configuration">
+        /// The configuration.
+        /// </param>
         /// <param name="duration">
         /// The duration.
         /// </param>
         /// <param name="startAtPreview">
         /// The start At Preview.
         /// </param>
-        /// <param name="previewScanCount">
-        /// The preview Scan Count.
-        /// </param>
-        /// <param name="verbosity">
-        /// The verbosity.
-        /// </param>
-        /// <param name="disableLibDvdNav">
-        /// The disable Lib Dvd Nav.
-        /// </param>
         /// <returns>
         /// A Cli query suitable for generating a preview video.
         /// </returns>
-        public static string GeneratePreviewQuery(EncodeTask task, int duration, string startAtPreview, int previewScanCount, int verbosity, bool disableLibDvdNav)
+        public static string GeneratePreviewQuery(EncodeTask task, HBConfiguration configuration, int duration, string startAtPreview)
         {
             string query = string.Empty;
-            query += SourceQuery(task, duration, startAtPreview, previewScanCount);
+            query += SourceQuery(task, duration, startAtPreview, configuration.PreviewScanCount);
             query += DestinationQuery(task);
-            query += GenerateTabbedComponentsQuery(task, true, verbosity, disableLibDvdNav);
+            query += GenerateTabbedComponentsQuery(task, true, configuration.Verbosity, configuration.IsDvdNavDisabled, configuration.DisableQuickSyncDecoding, false, false);
 
             return query;
         }
@@ -109,10 +98,19 @@ namespace HandBrake.ApplicationServices.Utilities
         /// <param name="disableLibDvdNav">
         /// The disable Lib Dvd Nav.
         /// </param>
+        /// <param name="disableQsvDecode">
+        /// The disable Qsv Decode.
+        /// </param>
+        /// <param name="enableHwd">
+        /// The enable Hwd.
+        /// </param>
+        /// <param name="enableOpenCL">
+        /// The enable Open CL.
+        /// </param>
         /// <returns>
         /// The CLI query for the Tabbed section of the main window UI
         /// </returns>
-        private static string GenerateTabbedComponentsQuery(EncodeTask task, bool enableFilters, int verbosity, bool disableLibDvdNav)
+        private static string GenerateTabbedComponentsQuery(EncodeTask task, bool enableFilters, int verbosity, bool disableLibDvdNav, bool disableQsvDecode, bool enableHwd, bool enableOpenCL)
         {
             string query = string.Empty;
 
@@ -142,7 +140,7 @@ namespace HandBrake.ApplicationServices.Utilities
             query += AdvancedQuery(task);
 
             // Extra Settings
-            query += ExtraSettings(verbosity, disableLibDvdNav);
+            query += ExtraSettings(verbosity, disableLibDvdNav, disableQsvDecode, enableHwd, enableOpenCL, task.VideoEncoder == VideoEncoder.QuickSync);
 
             return query;
         }
@@ -234,9 +232,6 @@ namespace HandBrake.ApplicationServices.Utilities
             query += string.Format(" -f {0} ", EnumHelper<Enum>.GetDescription(task.OutputFormat).ToLower());
 
             // These are output settings features
-            if (task.LargeFile)
-                query += " -4 ";
-
             if (task.IPod5GSupport)
                 query += " -I ";
 
@@ -261,17 +256,17 @@ namespace HandBrake.ApplicationServices.Utilities
 
             if (task.Anamorphic != Anamorphic.Strict)
             {
-                //if (task.MaxWidth.HasValue) query += string.Format(" -X {0}", task.MaxWidth);
                 if (task.Width.HasValue && task.Width != 0) query += string.Format(" -w {0}", task.Width);
 
-                //if (task.MaxHeight.HasValue) query += string.Format(" -Y {0}", task.MaxHeight);
                 if (task.Height.HasValue && task.Height != 0) query += string.Format(" -l {0}", task.Height);
             }
 
-            if (task.HasCropping)
-            {
-                query += string.Format(" --crop {0}:{1}:{2}:{3}", task.Cropping.Top, task.Cropping.Bottom, task.Cropping.Left, task.Cropping.Right);
-            }
+            query += string.Format(
+                " --crop {0}:{1}:{2}:{3}",
+                task.Cropping.Top,
+                task.Cropping.Bottom,
+                task.Cropping.Left,
+                task.Cropping.Right);
 
             switch (task.Anamorphic)
             {
@@ -377,23 +372,65 @@ namespace HandBrake.ApplicationServices.Utilities
                     break;
             }
 
-            switch (task.Denoise) // Denoise
+            if (task.Denoise == Denoise.hqdn3d)
             {
-                case Denoise.Weak:
-                    query += " --denoise=\"weak\"";
-                    break;
-                case Denoise.Medium:
-                    query += " --denoise=\"medium\"";
-                    break;
-                case Denoise.Strong:
-                    query += " --denoise=\"strong\"";
-                    break;
-                case Denoise.Custom:
-                    query += string.Format(" --denoise=\"{0}\"", task.CustomDenoise);
-                    break;
-                default:
-                    query += string.Empty;
-                    break;
+                switch (task.DenoisePreset) // Denoise
+                {
+                    case DenoisePreset.Weak:
+                        query += " --denoise=\"weak\"";
+                        break;
+                    case DenoisePreset.Medium:
+                        query += " --denoise=\"medium\"";
+                        break;
+                    case DenoisePreset.Strong:
+                        query += " --denoise=\"strong\"";
+                        break;
+                    case DenoisePreset.Custom:
+                        query += string.Format(" --denoise=\"{0}\"", task.CustomDenoise);
+                        break;
+                }
+            }
+
+            // NL Means
+            if (task.Denoise == Denoise.NLMeans)
+            {
+                switch (task.DenoisePreset) // Denoise
+                {
+                    case DenoisePreset.Light:
+                        query += " --nlmeans=\"light\"";
+                        break;
+                    case DenoisePreset.Ultralight:
+                        query += " --nlmeans=\"ultralight\"";
+                        break;
+                    case DenoisePreset.Medium:
+                        query += " --nlmeans=\"medium\"";
+                        break;
+                    case DenoisePreset.Strong:
+                        query += " --nlmeans=\"strong\"";
+                        break;
+                    default:
+                        query += string.Empty;
+                        break;
+                }
+
+                switch (task.DenoiseTune)
+                {
+                    case DenoiseTune.Animation:
+                        query += " --nlmeans-tune=\"animation\"";
+                        break;
+                    case DenoiseTune.Film:
+                        query += " --nlmeans-tune=\"film\"";
+                        break;
+                    case DenoiseTune.Grain:
+                        query += " --nlmeans-tune=\"grain\"";
+                        break;
+                    case DenoiseTune.HighMotion:
+                        query += " --nlmeans-tune=\"highmotion\"";
+                        break;
+                    default:
+                        query += string.Empty;
+                        break;
+                }
             }
 
             if (task.Deblock > 4)
@@ -429,12 +466,21 @@ namespace HandBrake.ApplicationServices.Utilities
                 case VideoEncoder.X264:
                     query += " -e x264";
                     break;
+                case VideoEncoder.QuickSync:
+                    query += " -e qsv_h264";
+                    break;
                 case VideoEncoder.Theora:
                     query += " -e theora";
                     break;
+                case VideoEncoder.VP8:
+                    query += " -e VP8";
+                    break;
                 default:
                     query += " -e x264";
                     break;
+                case VideoEncoder.X265:
+                    query += " -e x265";
+                    break;
             }
 
             switch (task.VideoEncodeRateType)
@@ -444,27 +490,14 @@ namespace HandBrake.ApplicationServices.Utilities
                         query += string.Format(" -b {0}", task.VideoBitrate.Value);
                     break;
                 case VideoEncodeRateType.ConstantQuality:
-                    double value;
-                    switch (task.VideoEncoder)
-                    {
-                        case VideoEncoder.FFMpeg:
-                        case VideoEncoder.FFMpeg2:
-                            query += string.Format(" -q {0}", task.Quality.Value.ToString(CultureInfo.InvariantCulture));
-                            break;
-                        case VideoEncoder.X264:
-                            query += string.Format(" -q {0}", task.Quality.Value.ToString(CultureInfo.InvariantCulture));
-                            break;
-                        case VideoEncoder.Theora:
-                            query += string.Format(" -q {0}", task.Quality.Value.ToString(CultureInfo.InvariantCulture));
-                            break;
-                    }
+                    query += string.Format(" -q {0}", task.Quality.Value.ToString(CultureInfo.InvariantCulture));
                     break;
             }
 
             if (task.TwoPass)
                 query += " -2 ";
 
-            if (task.TurboFirstPass)
+            if (task.TurboFirstPass && task.VideoEncoder == VideoEncoder.X264)
                 query += " -T ";
 
             if (task.Framerate.HasValue)
@@ -577,7 +610,7 @@ namespace HandBrake.ApplicationServices.Utilities
                     firstLoop = false;
                 }
                 else
-                    audioItems += "," +  Converters.GetCliAudioEncoder(item);
+                    audioItems += "," + Converters.GetCliAudioEncoder(item);
             }
             if (audioItems.Trim() != String.Empty)
                 query += " -E " + audioItems;
@@ -603,7 +636,7 @@ namespace HandBrake.ApplicationServices.Utilities
             // Sample Rate (-R)
             foreach (double item in samplerates)
             {
-                string add = (item == 0.0) ? "Auto" : item.ToString();
+                string add = (item == 0.0) ? "Auto" : item.ToString(new CultureInfo("en-US"));
                 if (firstLoop)
                 {
                     audioItems = add;
@@ -678,16 +711,18 @@ namespace HandBrake.ApplicationServices.Utilities
                     foundTrackName = true;
                 }
 
+                string fixedTrackName = trackName != null ? trackName.Replace(",", "\\,") : string.Empty;
+
                 if (firstLoop)
                 {
-                    audioItems = string.IsNullOrEmpty(trackName) ? "\"\"" : string.Format("\"{0}\"", trackName.Trim());
+                    audioItems = string.IsNullOrEmpty(fixedTrackName) ? string.Empty : string.Format("{0}", fixedTrackName.Trim());
                     firstLoop = false;
                 }
                 else
-                    audioItems += "," + (string.IsNullOrEmpty(trackName) ? "\"\"" : string.Format("\"{0}\"", trackName.Trim()));
+                    audioItems += "," + (string.IsNullOrEmpty(fixedTrackName) ? string.Empty : string.Format("{0}", fixedTrackName.Trim()));
             }
             if (foundTrackName)
-                query += string.Format(" --aname={0}", audioItems);
+                query += string.Format(" --aname=\"{0}\"", audioItems);
 
             // Passthru Settings
             if (task.AllowedPassthruOptions != null)
@@ -726,10 +761,10 @@ namespace HandBrake.ApplicationServices.Utilities
                     {
                         query += string.Format(" --audio-copy-mask {0}", fallbackEncoders);
                     }
-                } 
+                }
                 else
                 {
-                    query += string.Format(" --audio-copy-mask none");  
+                    query += string.Format(" --audio-copy-mask none");
                 }
 
                 query += string.Format(" --audio-fallback {0}", Converters.GetCliAudioEncoder(task.AllowedPassthruOptions.AudioEncoderFallback));
@@ -764,6 +799,7 @@ namespace HandBrake.ApplicationServices.Utilities
                 string srtOffset = String.Empty;
                 string srtLang = String.Empty;
                 string srtDefault = String.Empty;
+                int? srtBurnTrack = null;
                 int srtCount = 0;
                 int subCount = 0;
 
@@ -784,11 +820,13 @@ namespace HandBrake.ApplicationServices.Utilities
                         if (item.Default)
                             srtDefault = srtCount.ToString();
 
-                        itemToAdd = item.SrtPath;
+                        itemToAdd = item.SrtPath.Replace("\\", "\\\\").Replace(",", "\\,");
                         srtFile += srtFile == string.Empty ? itemToAdd : "," + itemToAdd;
 
                         itemToAdd = item.SrtOffset.ToString();
                         srtOffset += srtOffset == string.Empty ? itemToAdd : "," + itemToAdd;
+                        if (item.Burned)
+                            srtBurnTrack = srtCount;
                     }
                     else // We have Bitmap or CC
                     {
@@ -844,6 +882,8 @@ namespace HandBrake.ApplicationServices.Utilities
                         query += " --srt-lang " + srtLang;
                     if (srtDefault != string.Empty)
                         query += " --srt-default=" + srtDefault;
+                    if (srtBurnTrack.HasValue)
+                        query += " --srt-burn=" + srtBurnTrack.Value;
                 }
             }
 
@@ -899,57 +939,105 @@ namespace HandBrake.ApplicationServices.Utilities
         /// </returns>
         private static string AdvancedQuery(EncodeTask task)
         {
+            string query = string.Empty;
+
+            // X264 Only
             if (task.VideoEncoder == VideoEncoder.X264)
             {
-                string query = string.Empty; 
-
-                if (task.X264Preset != x264Preset.Medium)
+                if (!task.ShowAdvancedTab)
                 {
-                    query += string.Format(" --x264-preset={0} ", task.X264Preset.ToString().ToLower().Replace(" ", string.Empty));
-                }
-
-                if (task.H264Profile != x264Profile.None)
-                {
-                    query += string.Format(" --x264-profile={0} ", task.H264Profile.ToString().ToLower().Replace(" ", string.Empty));
-                }
-
-                if (task.X264Tune != x264Tune.None)
-                {
-                    string tune = string.Empty;
-
-                    if (task.FastDecode)
+                    if (task.X264Preset != x264Preset.Medium)
                     {
-                        tune = "fastdecode";
+                        query += string.Format(
+                            " --encoder-preset={0} ", task.X264Preset.ToString().ToLower().Replace(" ", string.Empty));
                     }
 
-                    string tuneDropdown = task.X264Tune.ToString().ToLower().Replace(" ", string.Empty);
-                    if (task.X264Tune != x264Tune.None && !string.IsNullOrEmpty(tuneDropdown))
+                    if (task.X264Tune != x264Tune.None)
                     {
-                        tune = string.IsNullOrEmpty(tune) ? tuneDropdown : string.Format(",{0}", tuneDropdown);  
+                        string tune = string.Empty;
+
+                        if (task.FastDecode)
+                        {
+                            tune = "fastdecode";
+                        }
+
+                        string tuneDropdown = task.X264Tune.ToString().ToLower().Replace(" ", string.Empty);
+                        if (task.X264Tune != x264Tune.None && !string.IsNullOrEmpty(tuneDropdown))
+                        {
+                            tune = string.IsNullOrEmpty(tune) ? tuneDropdown : string.Format(",{0}", tuneDropdown);
+                        }
+
+                        query += string.Format(" --encoder-tune=\"{0}\" ", tune);
                     }
 
-                    query += string.Format(" --x264-tune=\"{0}\" ", tune);
+                    if (!string.IsNullOrEmpty(task.ExtraAdvancedArguments))
+                    {
+                        query += string.Format(" -x {0}", task.ExtraAdvancedArguments);
+                    }
                 }
-
-                if (task.H264Level != "Auto")
-                {
-                    query += string.Format(" --h264-level=\"{0}\" ", task.H264Level);
-                }
-
-                if (!string.IsNullOrEmpty(task.AdvancedEncoderOptions))
-                {
-                    query += string.Format(" -x {0}", task.AdvancedEncoderOptions);
-                }
-
-                if (!string.IsNullOrEmpty(task.ExtraAdvancedArguments))
-                {
-                    query += string.Format(" -x {0}", task.ExtraAdvancedArguments);
-                }
-
-                return query;
             }
 
-            return string.IsNullOrEmpty(task.AdvancedEncoderOptions) ? string.Empty : string.Format(" -x {0}", task.AdvancedEncoderOptions);
+            // X265 Only
+            if (task.VideoEncoder == VideoEncoder.X265)
+            {
+                if (!task.ShowAdvancedTab)
+                {
+                    if (task.X265Preset != x265Preset.Medium)
+                    {
+                        query += string.Format(
+                            " --encoder-preset={0} ", task.X265Preset.ToString().ToLower().Replace(" ", string.Empty));
+                    }
+
+                    if (task.X265Tune != x265Tune.None)
+                    {
+                        query += string.Format(
+                            " --encoder-tune=\"{0}\" ", task.X265Tune.ToString().ToLower().Replace(" ", string.Empty));
+                    }
+
+                    if (task.H265Profile != x265Profile.None)
+                    {
+                        query += string.Format(
+                            " --encoder-profile={0} ", task.H265Profile.ToString().ToLower().Replace(" ", string.Empty));
+                    }
+
+                    if (!string.IsNullOrEmpty(task.ExtraAdvancedArguments))
+                    {
+                        query += string.Format(" -x {0}", task.ExtraAdvancedArguments);
+                    }
+                }
+            }
+
+            // QSV Only
+            if (task.VideoEncoder == VideoEncoder.QuickSync)
+            {
+                query += string.Format(" --encoder-preset={0}", task.QsvPreset.ToString().ToLower());
+            }
+
+            // options that apply to all encoders
+            if (!string.IsNullOrEmpty(task.AdvancedEncoderOptions))
+            {
+                query += string.Format(" -x {0}", task.AdvancedEncoderOptions);
+            }
+
+            // Options that apply to both x264 and QuickSync
+            if (task.VideoEncoder == VideoEncoder.QuickSync || task.VideoEncoder == VideoEncoder.X264)
+            {
+                // when using x264 with the advanced panel, the H.264 profile/level widgets are disabled
+                if (!(task.VideoEncoder == VideoEncoder.X264 && task.ShowAdvancedTab))
+                {
+                    if (task.H264Level != "Auto")
+                    {
+                        query += string.Format(" --encoder-level=\"{0}\" ", task.H264Level);
+                    }
+                    if (task.H264Profile != x264Profile.None)
+                    {
+                        query += string.Format(
+                            " --encoder-profile={0} ", task.H264Profile.ToString().ToLower().Replace(" ", string.Empty));
+                    }
+                }
+            }
+
+            return query;
         }
 
         /// <summary>
@@ -961,10 +1049,22 @@ namespace HandBrake.ApplicationServices.Utilities
         /// <param name="disableLibdvdNav">
         /// The disable Libdvd Nav.
         /// </param>
+        /// <param name="disableQsvDecode">
+        /// The disable Qsv Decode.
+        /// </param>
+        /// <param name="enableHwd">
+        /// The enable Hwd.
+        /// </param>
+        /// <param name="enableOpenCL">
+        /// The enable Open CL.
+        /// </param>
+        /// <param name="isQsv">
+        /// The is Qsv.
+        /// </param>
         /// <returns>
         /// A Cli Query as a string
         /// </returns>
-        private static string ExtraSettings(int verbosity, bool disableLibdvdNav)
+        private static string ExtraSettings(int verbosity, bool disableLibdvdNav, bool disableQsvDecode, bool enableHwd, bool enableOpenCL, bool isQsv)
         {
             string query = string.Empty;
 
@@ -975,6 +1075,17 @@ namespace HandBrake.ApplicationServices.Utilities
             if (disableLibdvdNav)
                 query += " --no-dvdnav";
 
+            if (disableQsvDecode)
+                query += " --disable-qsv-decoding";
+
+            if (enableOpenCL && !isQsv)
+                query += " -P ";
+
+            if (enableHwd && !isQsv)
+            {
+                query += " -U ";
+            }
+
             return query;
         }
 
