@@ -54,7 +54,6 @@ namespace HandBrake.ApplicationServices.Utilities
             // Output Settings
             Match format = Regex.Match(input, @"-f ([a-zA-Z0-9]+)");
             Match grayscale = Regex.Match(input, @" -g");
-            Match largerMp4 = Regex.Match(input, @" -4");
             Match ipodAtom = Regex.Match(input, @" -I");
 
             // Picture Settings Tab
@@ -75,9 +74,11 @@ namespace HandBrake.ApplicationServices.Utilities
 
             // Picture Settings - Filters
             Match decomb = Regex.Match(input, @" --decomb");
-            Match decombValue = Regex.Match(input, @" --decomb=\""([a-zA-Z0-9.:]*)\""");
+            Match decombValue = Regex.Match(input, @" --decomb=([a-zA-Z0-9.:""\\]*)");
             Match deinterlace = Regex.Match(input, @"--deinterlace=\""([a-zA-Z0-9.:]*)\""");
             Match denoise = Regex.Match(input, @"--denoise=\""([a-zA-Z0-9.:]*)\""");
+            Match nlmeans = Regex.Match(input, @"--nlmeans=\""([a-zA-Z0-9.:]*)\""");
+            Match nlmeansTune = Regex.Match(input, @"--nlmeans-tune=\""([a-zA-Z0-9.:]*)\""");
             Match deblock = Regex.Match(input, @"--deblock=([0-9:]*)");
             Match detelecine = Regex.Match(input, @"--detelecine");
             Match detelecineValue = Regex.Match(input, @" --detelecine=\""([a-zA-Z0-9.:]*)\""");
@@ -115,6 +116,9 @@ namespace HandBrake.ApplicationServices.Utilities
             Match x264Profile = Regex.Match(input, @"--x264-profile([=a-zA-Z0-9\s ]*)");
             Match h264Level = Regex.Match(input, @"--h264-level([=a-zA-Z0-9.\s ]*)");
 
+            Match x265Profile = Regex.Match(input, @"--x265-profile([=a-zA-Z0-9\s ]*)");
+            Match x265Tune = Regex.Match(input, @"--x265-tune([=,a-zA-Z0-9\s ]*)");
+            Match x265Preset = Regex.Match(input, @"--x265-preset([=a-zA-Z0-9\s ]*)");
             #endregion
 
             #region Set Varibles
@@ -152,7 +156,6 @@ namespace HandBrake.ApplicationServices.Utilities
                 {
                     parsed.OutputFormat = Converters.GetFileFormat(format.Groups[1].ToString());
                 }
-                parsed.LargeFile = largerMp4.Success;
                 parsed.IPod5GSupport = ipodAtom.Success;
                 parsed.OptimizeMP4 = optimizeMP4.Success;
 
@@ -272,24 +275,64 @@ namespace HandBrake.ApplicationServices.Utilities
                 parsed.Denoise = Denoise.Off;
                 if (denoise.Success)
                 {
+                    parsed.Denoise = Denoise.hqdn3d;
                     switch (denoise.ToString().Replace("--denoise=", string.Empty).Replace("\"", string.Empty))
                     {
                         case "weak":
-                            parsed.Denoise = Denoise.Weak;
+                            parsed.DenoisePreset = DenoisePreset.Weak;
                             break;
                         case "medium":
-                            parsed.Denoise = Denoise.Medium;
+                            parsed.DenoisePreset = DenoisePreset.Medium;
                             break;
                         case "strong":
-                            parsed.Denoise = Denoise.Strong;
+                            parsed.DenoisePreset = DenoisePreset.Strong;
                             break;
                         default:
-                            parsed.Denoise = Denoise.Custom;
+                            parsed.DenoisePreset = DenoisePreset.Custom;
                             parsed.CustomDenoise = denoise.ToString().Replace("--denoise=", string.Empty).Replace("\"", string.Empty);
                             break;
                     }
                 }
 
+                if (nlmeans.Success)
+                {
+                    parsed.Denoise = Denoise.NLMeans;
+                    switch (nlmeans.ToString().Replace("--nlmeans=", string.Empty).Replace("\"", string.Empty))
+                    {
+                        case "ultralight":
+                            parsed.DenoisePreset = DenoisePreset.Ultralight;
+                            break;
+                        case "light":
+                            parsed.DenoisePreset = DenoisePreset.Light;
+                            break;
+                        case "medium":
+                            parsed.DenoisePreset = DenoisePreset.Medium;
+                            break;
+                        case "strong":
+                            parsed.DenoisePreset = DenoisePreset.Strong;
+                            break;
+                    }
+
+                    if (nlmeansTune.Success)
+                    {
+                        switch (nlmeansTune.ToString().Replace("--nlmeans-tune=", string.Empty).Replace("\"", string.Empty))
+                        {
+                            case "animation":
+                                parsed.DenoiseTune = DenoiseTune.Animation;
+                                break;
+                            case "film":
+                                parsed.DenoiseTune = DenoiseTune.Film;
+                                break;
+                            case "grain":
+                                parsed.DenoiseTune = DenoiseTune.Grain;
+                                break;
+                            case "highmotion":
+                                parsed.DenoiseTune = DenoiseTune.HighMotion;
+                                break;
+                        }                      
+                    }
+                }
+
                 parsed.Deblock = 0;
                 if (deblock.Success)
                 {
@@ -477,7 +520,33 @@ namespace HandBrake.ApplicationServices.Utilities
 
                     parsed.X264Tune = Converters.Getx264TuneFromCli(tuneOptions);
                 }
-                  
+
+                if (x265Preset.Success)
+                    parsed.X265Preset =
+                        Converters.Getx265PresetFromCli(x265Preset.ToString().Replace("--x265-preset", string.Empty).Replace("=", string.Empty).Trim());
+
+                if (h264Profile.Success)
+                    parsed.H265Profile =
+                        Converters.Getx265ProfileFromCli(h264Profile.ToString().Replace("--h265-profile", string.Empty).Replace("=", string.Empty).Trim());
+
+                if (x265Profile.Success)
+                    parsed.H265Profile =
+                       Converters.Getx265ProfileFromCli(x265Profile.ToString().Replace("--x265-profile", string.Empty).Replace("=", string.Empty).Trim());
+
+                if (x265Tune.Success)
+                {
+                    string tuneOptions =
+                        x265Tune.ToString().Replace("--x265-tune", string.Empty).Replace("=", string.Empty).Trim();
+
+                    parsed.FastDecode = tuneOptions.Contains("fastdecode");
+
+                    // Remove these options. They are not in the dropdown.
+                    tuneOptions = tuneOptions.Replace("fastdecode", string.Empty).Replace(
+                        ",", string.Empty);
+
+                    parsed.X265Tune = Converters.Getx265TuneFromCli(tuneOptions);
+                }
+
                 #endregion
             }
             catch (Exception exc)
