@@ -13,34 +13,59 @@ namespace HandBrake.Interop.Model
 	using System.Collections.Generic;
 	using System.Linq;
 
+	using HandBrake.Interop.Converters;
 	using HandBrake.Interop.HbLib;
+	using HandBrake.Interop.Helpers;
 	using HandBrake.Interop.Model.Encoding;
 	using HandBrake.Interop.SourceData;
 
-    /// <summary>
-    /// The encoders.
-    /// </summary>
-    public static class Encoders
+	/// <summary>
+	/// The encoders.
+	/// </summary>
+	public static class Encoders
 	{
-	    /// <summary>
-	    /// The audio encoders.
-	    /// </summary>
-	    private static List<HBAudioEncoder> audioEncoders;
+		/// <summary>
+		/// The audio encoders.
+		/// </summary>
+		private static List<HBAudioEncoder> audioEncoders;
 
-	    /// <summary>
-	    /// The video encoders.
-	    /// </summary>
-	    private static List<HBVideoEncoder> videoEncoders;
+		/// <summary>
+		/// The video encoders.
+		/// </summary>
+		private static List<HBVideoEncoder> videoEncoders;
 
-	    /// <summary>
-	    /// The mixdowns.
-	    /// </summary>
-	    private static List<HBMixdown> mixdowns;
+		/// <summary>
+		/// Video framerates in pts.
+		/// </summary>
+		private static List<HBRate> videoFramerates; 
 
-	    /// <summary>
-	    /// The audio bitrates.
-	    /// </summary>
-	    private static List<int> audioBitrates; 
+		/// <summary>
+		/// List of HandBrake mixdowns.
+		/// </summary>
+		private static List<HBMixdown> mixdowns;
+
+		/// <summary>
+		/// List of HandBrake containers.
+		/// </summary>
+		private static List<HBContainer> containers; 
+
+		/// <summary>
+		/// The audio bitrates.
+		/// </summary>
+		private static List<int> audioBitrates;
+
+		/// <summary>
+		/// Audio sample rates in Hz.
+		/// </summary>
+		private static List<HBRate> audioSampleRates; 
+
+		/// <summary>
+		/// Initializes static members of the Encoders class.
+		/// </summary>
+		static Encoders()
+		{
+			HandBrakeUtils.EnsureGlobalInit();
+		}
 
 		/// <summary>
 		/// Gets a list of supported audio encoders.
@@ -51,12 +76,7 @@ namespace HandBrake.Interop.Model
 			{
 				if (audioEncoders == null)
 				{
-					IntPtr encodersPtr = HBFunctions.hb_get_audio_encoders();
-					int encoderCount = HBFunctions.hb_get_audio_encoders_count();
-
-					audioEncoders = InteropUtilities.ConvertArray<hb_encoder_s>(encodersPtr, encoderCount)
-						.Select(Converters.NativeToAudioEncoder)
-						.ToList();
+					audioEncoders = InteropUtilities.ToListFromIterator<hb_encoder_s, HBAudioEncoder>(HBFunctions.hb_audio_encoder_get_next, Converters.NativeToAudioEncoder);
 				}
 
 				return audioEncoders;
@@ -72,18 +92,29 @@ namespace HandBrake.Interop.Model
 			{
 				if (videoEncoders == null)
 				{
-					IntPtr encodersPtr = HBFunctions.hb_get_video_encoders();
-					int encoderCount = HBFunctions.hb_get_video_encoders_count();
-
-					videoEncoders = InteropUtilities.ConvertArray<hb_encoder_s>(encodersPtr, encoderCount)
-						.Select(Converters.NativeToVideoEncoder)
-						.ToList();
+					videoEncoders = InteropUtilities.ToListFromIterator<hb_encoder_s, HBVideoEncoder>(HBFunctions.hb_video_encoder_get_next, Converters.NativeToVideoEncoder);
 				}
 
 				return videoEncoders;
 			}
 		}
 
+		/// <summary>
+		/// Gets a list of supported video framerates (in pts).
+		/// </summary>
+		public static List<HBRate> VideoFramerates
+		{
+			get
+			{
+				if (videoFramerates == null)
+				{
+					videoFramerates = InteropUtilities.ToListFromIterator<hb_rate_s, HBRate>(HBFunctions.hb_video_framerate_get_next, Converters.NativeToRate);
+				}
+
+				return videoFramerates;
+			}
+		} 
+
 		/// <summary>
 		/// Gets a list of supported mixdowns.
 		/// </summary>
@@ -93,12 +124,7 @@ namespace HandBrake.Interop.Model
 			{
 				if (mixdowns == null)
 				{
-					IntPtr mixdownsPtr = HBFunctions.hb_get_audio_mixdowns();
-					int mixdownsCount = HBFunctions.hb_get_audio_mixdowns_count();
-
-					mixdowns = InteropUtilities.ConvertArray<hb_mixdown_s>(mixdownsPtr, mixdownsCount)
-						.Select(Converters.NativeToMixdown)
-						.ToList();
+					mixdowns = InteropUtilities.ToListFromIterator<hb_mixdown_s, HBMixdown>(HBFunctions.hb_mixdown_get_next, Converters.NativeToMixdown);
 				}
 
 				return mixdowns;
@@ -114,18 +140,56 @@ namespace HandBrake.Interop.Model
 			{
 				if (audioBitrates == null)
 				{
-					IntPtr audioBitratesPtr = HBFunctions.hb_get_audio_bitrates();
-					int audioBitratesCount = HBFunctions.hb_get_audio_bitrates_count();
-
-					audioBitrates = InteropUtilities.ConvertArray<hb_rate_s>(audioBitratesPtr, audioBitratesCount)
-						.Select(b => b.rate)
-						.ToList();
+					audioBitrates = InteropUtilities.ToListFromIterator<hb_rate_s, int>(HBFunctions.hb_audio_bitrate_get_next, b => b.rate);
 				}
 
 				return audioBitrates;
 			}
 		}
 
+		/// <summary>
+		/// Gets a list of supported audio sample rates (in Hz).
+		/// </summary>
+		public static List<HBRate> AudioSampleRates
+		{
+			get
+			{
+				if (audioSampleRates == null)
+				{
+					audioSampleRates = InteropUtilities.ToListFromIterator<hb_rate_s, HBRate>(HBFunctions.hb_audio_samplerate_get_next, Converters.NativeToRate);
+				}
+
+				return audioSampleRates;
+			}
+		}
+
+		/// <summary>
+		/// Gets a list of supported containers.
+		/// </summary>
+		public static List<HBContainer> Containers
+		{
+			get
+			{
+				if (containers == null)
+				{
+					containers = InteropUtilities.ToListFromIterator<hb_container_s, HBContainer>(HBFunctions.hb_container_get_next, Converters.NativeToContainer);
+				}
+
+				return containers;
+			}
+		}
+
+		/// <summary>
+		/// Gets a value indicating whether SRT subtitles can be burnt in.
+		/// </summary>
+		public static bool CanBurnSrt
+		{
+			get
+			{
+				return HBFunctions.hb_subtitle_can_burn((int)hb_subtitle_s_subsource.SRTSUB) > 0;
+			}
+		}
+
 		/// <summary>
 		/// Gets the audio encoder with the specified short name.
 		/// </summary>
@@ -136,6 +200,16 @@ namespace HandBrake.Interop.Model
 			return AudioEncoders.SingleOrDefault(e => e.ShortName == shortName);
 		}
 
+		/// <summary>
+		/// Gets the audio encoder with the specified codec ID.
+		/// </summary>
+		/// <param name="codecId">The ID of the audio encoder.</param>
+		/// <returns>The requested audio encoder.</returns>
+		public static HBAudioEncoder GetAudioEncoder(int codecId)
+		{
+			return AudioEncoders.SingleOrDefault(e => e.Id == codecId);
+		}
+
 		/// <summary>
 		/// Gets the video encoder with the specified short name.
 		/// </summary>
@@ -156,6 +230,16 @@ namespace HandBrake.Interop.Model
 			return Mixdowns.SingleOrDefault(m => m.ShortName == shortName);
 		}
 
+		/// <summary>
+		/// Gets the container with the specified short name.
+		/// </summary>
+		/// <param name="shortName">The name of the container.</param>
+		/// <returns>The requested container.</returns>
+		public static HBContainer GetContainer(string shortName)
+		{
+			return Containers.SingleOrDefault(c => c.ShortName == shortName);
+		}
+
 		/// <summary>
 		/// Determines if the given encoder is compatible with the given track.
 		/// </summary>
@@ -190,6 +274,27 @@ namespace HandBrake.Interop.Model
 			return HBFunctions.hb_mixdown_has_codec_support(mixdown.Id, (uint) encoder.Id) > 0;
 		}
 
+		/// <summary>
+		/// Determines if DRC can be applied to the given track with the given encoder.
+		/// </summary>
+		/// <param name="track">The track to apply DRC to.</param>
+		/// <param name="encoder">The encoder to use for DRC.</param>
+		/// <returns>True if DRC can be applied to the track with the given encoder.</returns>
+		public static bool CanApplyDrc(AudioTrack track, HBAudioEncoder encoder)
+		{
+			return HBFunctions.hb_audio_can_apply_drc(track.CodecId, track.CodecParam, encoder.Id) > 0;
+		}
+
+		/// <summary>
+		/// Determines if the given input audio codec can be passed through.
+		/// </summary>
+		/// <param name="codecId">The input codec to consider.</param>
+		/// <returns>True if the codec can be passed through.</returns>
+		public static bool CanPassthroughAudio(int codecId)
+		{
+			return (codecId & NativeConstants.HB_ACODEC_PASS_MASK) > 0;
+		}
+
 		/// <summary>
 		/// Sanitizes a mixdown given the output codec and input channel layout.
 		/// </summary>
@@ -199,7 +304,7 @@ namespace HandBrake.Interop.Model
 		/// <returns>A sanitized mixdown value.</returns>
 		public static HBMixdown SanitizeMixdown(HBMixdown mixdown, HBAudioEncoder encoder, ulong layout)
 		{
-			int sanitizedMixdown = HBFunctions.hb_get_best_mixdown((uint)encoder.Id, layout, mixdown.Id);
+			int sanitizedMixdown = HBFunctions.hb_mixdown_get_best((uint)encoder.Id, layout, mixdown.Id);
 			return Mixdowns.Single(m => m.Id == sanitizedMixdown);
 		}
 
@@ -211,7 +316,7 @@ namespace HandBrake.Interop.Model
 		/// <returns>The default mixdown for the given codec and channel layout.</returns>
 		public static HBMixdown GetDefaultMixdown(HBAudioEncoder encoder, ulong layout)
 		{
-			int defaultMixdown = HBFunctions.hb_get_default_mixdown((uint)encoder.Id, layout);
+			int defaultMixdown = HBFunctions.hb_mixdown_get_default((uint)encoder.Id, layout);
 			return Mixdowns.Single(m => m.Id == defaultMixdown);
 		}
 
@@ -227,11 +332,34 @@ namespace HandBrake.Interop.Model
 			int low = 0;
 			int high = 0;
 
-			HBFunctions.hb_get_audio_bitrate_limits((uint)encoder.Id, sampleRate, mixdown.Id, ref low, ref high);
+			HBFunctions.hb_audio_bitrate_get_limits((uint)encoder.Id, sampleRate, mixdown.Id, ref low, ref high);
 
 			return new BitrateLimits { Low = low, High = high };
 		}
 
+		/// <summary>
+		/// Gets the video quality limits for the given video codec.
+		/// </summary>
+		/// <param name="encoder">The video encoder to check.</param>
+		/// <returns>Limits on the video quality for the encoder.</returns>
+		public static VideoQualityLimits GetVideoQualityLimits(HBVideoEncoder encoder)
+		{
+			float low = 0;
+			float high = 0;
+			float granularity = 0;
+			int direction = 0;
+
+			HBFunctions.hb_video_quality_get_limits((uint)encoder.Id, ref low, ref high, ref granularity, ref direction);
+
+			return new VideoQualityLimits
+				{
+					Low = low,
+					High = high,
+					Granularity = granularity,
+					Ascending = direction == 0
+				};
+		}
+
 		/// <summary>
 		/// Sanitizes an audio bitrate given the output codec, sample rate and mixdown.
 		/// </summary>
@@ -242,7 +370,7 @@ namespace HandBrake.Interop.Model
 		/// <returns>A sanitized audio bitrate.</returns>
 		public static int SanitizeAudioBitrate(int audioBitrate, HBAudioEncoder encoder, int sampleRate, HBMixdown mixdown)
 		{
-			return HBFunctions.hb_get_best_audio_bitrate((uint)encoder.Id, audioBitrate, sampleRate, mixdown.Id);
+			return HBFunctions.hb_audio_bitrate_get_best((uint)encoder.Id, audioBitrate, sampleRate, mixdown.Id);
 		}
 
 		/// <summary>
@@ -254,7 +382,7 @@ namespace HandBrake.Interop.Model
 		/// <returns>The default bitrate for these parameters.</returns>
 		public static int GetDefaultBitrate(HBAudioEncoder encoder, int sampleRate, HBMixdown mixdown)
 		{
-			return HBFunctions.hb_get_default_audio_bitrate((uint) encoder.Id, sampleRate, mixdown.Id);
+			return HBFunctions.hb_audio_bitrate_get_default((uint) encoder.Id, sampleRate, mixdown.Id);
 		}
 
 		/// <summary>
@@ -266,7 +394,7 @@ namespace HandBrake.Interop.Model
 		{
 			float low = 0, high = 0, granularity = 0;
 			int direction = 0;
-			HBFunctions.hb_get_audio_quality_limits((uint)encoderId, ref low, ref high, ref granularity, ref direction);
+			HBFunctions.hb_audio_quality_get_limits((uint)encoderId, ref low, ref high, ref granularity, ref direction);
 
 			return new RangeLimits
 			{
@@ -286,7 +414,7 @@ namespace HandBrake.Interop.Model
 		{
 			float low = 0, high = 0, granularity = 0;
 			int direction = 0;
-			HBFunctions.hb_get_audio_compression_limits((uint)encoderId, ref low, ref high, ref granularity, ref direction);
+			HBFunctions.hb_audio_compression_get_limits((uint)encoderId, ref low, ref high, ref granularity, ref direction);
 
 			return new RangeLimits
 			{
