@@ -2,16 +2,17 @@
 // <copyright file="AudioEncodeRateType.cs" company="HandBrake Project (http://handbrake.fr)">
 //   This file is part of the HandBrake source code - It may be used under the terms of the GNU General Public License.
 // </copyright>
+// <summary>
+//   The audio encode rate type.
+// </summary>
 // --------------------------------------------------------------------------------------------------------------------
 
 namespace HandBrake.Interop.Model.Encoding
 {
-	using System;
-	using System.Collections.Generic;
-	using System.Linq;
-	using System.Text;
-
-	public enum AudioEncodeRateType
+    /// <summary>
+    /// The audio encode rate type.
+    /// </summary>
+    public enum AudioEncodeRateType
 	{
 		Bitrate,
 		Quality
