@@ -9,11 +9,15 @@
 
 namespace HandBrake.Interop.Model.Encoding
 {
-	public enum Decomb
+    /// <summary>
+    /// The decomb.
+    /// </summary>
+    public enum Decomb
 	{
 		Off = 0,
-		Default,
-		Custom,
-        Fast
+		Default = 2,
+		Fast = 3,
+		Bob = 4,
+		Custom = 1
 	}
 }
