@@ -9,12 +9,13 @@
 
 namespace HandBrake.Interop.Model.Encoding
 {
-	public enum Denoise
+    /// <summary>
+    /// The denoise.
+    /// </summary>
+    public enum Denoise
 	{
 		Off = 0,
-		Weak,
-		Medium,
-		Strong,
-		Custom
+        hqdn3d = 1,
+		NLMeans = 2,
 	}
 }
