@@ -16,9 +16,6 @@ namespace HandBrake.Interop.Model.Encoding.x264
     /// </summary>
     public enum x264Preset
     {
-        [Display(Name = "None")]
-        None = 0,
-
         [Display(Name = "Ultrafast")]
         Ultrafast,
 
