@@ -29,6 +29,11 @@ namespace HandBrake.Interop.SourceData
 		/// </summary>
 		public uint CodecId { get; set; }
 
+		/// <summary>
+		/// Gets or sets the codec parameters for this track.
+		/// </summary>
+		public uint CodecParam { get; set; }
+
 		/// <summary>
 		/// Gets or sets the language (if detected) of this Audio Track
 		/// </summary>
