@@ -18,6 +18,7 @@ namespace HandBrakeWPF
 
     using HandBrake.ApplicationServices.Exceptions;
 
+    using HandBrakeWPF.Utilities;
     using HandBrakeWPF.ViewModels;
     using HandBrakeWPF.ViewModels.Interfaces;
 
@@ -44,14 +45,40 @@ namespace HandBrakeWPF
         /// </param>
         protected override void OnStartup(StartupEventArgs e)
         {
+            OperatingSystem OS = Environment.OSVersion;
+            if ((OS.Platform == PlatformID.Win32NT) && (OS.Version.Major == 5 && OS.Version.Minor <= 1))
+            {
+                MessageBox.Show("Windows XP and earlier are no longer supported. Version 0.9.9 was the last version to support these versions. ", "Notice", MessageBoxButton.OK, MessageBoxImage.Warning);
+                Application.Current.Shutdown();
+                return;
+            }
+
+            if (e.Args.Any(f => f.Equals("--instant")))
+            {
+                AppArguments.IsInstantHandBrake = true;
+                MessageBox.Show("Instant HandBrake is just a prototype for toying with ideas. It may or may not work, or even be included in future builds.", "Warning", MessageBoxButton.OK, MessageBoxImage.Warning);
+            }
+
+            if (e.Args.Any(f => f.Equals("--reset")))
+            {
+                HandBrakeApp.ResetToDefaults();
+                Application.Current.Shutdown();
+                return;
+            }
+
+            if (e.Args.Any(f => f.Equals("--enable-libhb")))
+            {
+                AppArguments.EnableLibHB = true;
+            }
+
             base.OnStartup(e);
 
             // If we have a file dropped on the icon, try scanning it.
-            string[] fileNames = e.Args;
-            if (fileNames.Any() && (File.Exists(fileNames[0]) || Directory.Exists(fileNames[0])))
+            string[] args = e.Args;
+            if (args.Any() && (File.Exists(args[0]) || Directory.Exists(args[0])))
             {
                 IMainViewModel mvm = IoC.Get<IMainViewModel>();
-                mvm.StartScan(fileNames[0], 0);
+                mvm.StartScan(args[0], 0);
             }
         }
 
@@ -102,6 +129,20 @@ namespace HandBrakeWPF
             {
                 this.ShowError(e.Exception.InnerException);
             }
+            else if (e.Exception.InnerException != null && e.Exception.InnerException.GetType() == typeof(Castle.MicroKernel.ComponentActivator.ComponentActivatorException))
+            {
+                // Handle Component Activation Exceptions. Can happen when one of the services throws an execption when being constructed.
+                Exception innerException = e.Exception.InnerException.InnerException;
+                if (innerException != null && innerException.InnerException != null &&
+                    innerException.InnerException.GetType() == typeof(GeneralApplicationException))
+                {
+                    this.ShowError(innerException.InnerException);
+                }
+                else
+                {
+                    this.ShowError(innerException);
+                }
+            }
             else
             {
                 this.ShowError(e.Exception);
@@ -124,10 +165,10 @@ namespace HandBrakeWPF
                 if (windowManager != null)
                 {
                     ErrorViewModel errorView = new ErrorViewModel();
-
+                    GeneralApplicationException applicationException = null;
                     if (exception.GetType() == typeof(GeneralApplicationException))
                     {
-                        GeneralApplicationException applicationException = exception as GeneralApplicationException;
+                        applicationException = exception as GeneralApplicationException;
                         if (applicationException != null)
                         {
                             string details = string.Format(
@@ -136,9 +177,7 @@ namespace HandBrakeWPF
                                 Environment.NewLine,
                                 applicationException.Solution,
                                 Environment.NewLine,
-                                applicationException.ActualException != null
-                                    ? applicationException.ActualException.ToString()
-                                    : "No additional exception information available.");
+                                applicationException.ActualException != null ? applicationException.ActualException.ToString() : "No additional exception information available.");
 
                             errorView.ErrorMessage = applicationException.Error;
                             errorView.Solution = applicationException.Solution;
@@ -150,7 +189,17 @@ namespace HandBrakeWPF
                         errorView.Details = exception.ToString();
                     }
 
-                    windowManager.ShowDialog(errorView);
+                    try
+                    {
+                        windowManager.ShowDialog(errorView);
+                    }
+                    catch (Exception)
+                    {
+                        if (applicationException != null)
+                        {
+                            MessageBox.Show(applicationException.Error + Environment.NewLine + Environment.NewLine + applicationException.Solution, "Error", MessageBoxButton.OK, MessageBoxImage.Error);
+                        }
+                    }
                 }
             }
             catch (Exception)
