@@ -15,6 +15,7 @@ namespace HandBrakeWPF.Converters
     using System;
 
     using HandBrake.ApplicationServices.Model;
+    using HandBrake.ApplicationServices.Model.Encoding;
     using HandBrake.ApplicationServices.Utilities;
     using HandBrake.Interop.Model.Encoding;
     using HandBrake.Interop.Model.Encoding.x264;
@@ -65,7 +66,10 @@ namespace HandBrakeWPF.Converters
             {
                 return EnumHelper<Mixdown>.GetEnumDisplayValues(typeof(Mixdown));
             }
- 
+            if (targetType == typeof(QsvPreset) || value.GetType() == typeof(QsvPreset))
+            {
+                return EnumHelper<QsvPreset>.GetDisplay((QsvPreset)value);
+            }
             if (value is IEnumerable<PresetPictureSettingsMode>)
             {
                 return EnumHelper<PresetPictureSettingsMode>.GetEnumDisplayValues(typeof(PresetPictureSettingsMode));
@@ -86,8 +90,14 @@ namespace HandBrakeWPF.Converters
             {
                 return EnumHelper<Denoise>.GetEnumDisplayValues(typeof(Denoise));
             }
-
-
+            if (value is IEnumerable<VideoScaler>)
+            {
+                return EnumHelper<VideoScaler>.GetEnumDisplayValues(typeof(VideoScaler));
+            }
+            if (value is IEnumerable<OutputFormat>)
+            {
+                return EnumHelper<OutputFormat>.GetEnumDisplayValues(typeof(OutputFormat));
+            }
 
             // Single Items
             if (targetType == typeof(x264Preset) || value.GetType() == typeof(x264Preset))
@@ -136,6 +146,16 @@ namespace HandBrakeWPF.Converters
                 return EnumHelper<QueueItemStatus>.GetDisplay((QueueItemStatus)value);
             }
 
+            if (targetType == typeof(VideoScaler) || value.GetType() == typeof(VideoScaler))
+            {
+                return EnumHelper<VideoScaler>.GetDisplay((VideoScaler)value);
+            }
+
+            if (targetType == typeof(OutputFormat) || value.GetType() == typeof(OutputFormat))
+            {
+                return EnumHelper<OutputFormat>.GetDisplay((OutputFormat)value);
+            }
+
             return null;
         }
 
@@ -182,7 +202,10 @@ namespace HandBrakeWPF.Converters
             {
                 return EnumHelper<Mixdown>.GetValue(value.ToString());
             }
-
+            if (targetType == typeof(QsvPreset) || value.GetType() == typeof(QsvPreset))
+            {
+                return EnumHelper<QsvPreset>.GetValue(value.ToString());
+            }
             if (targetType == typeof(PresetPictureSettingsMode) || value.GetType() == typeof(PresetPictureSettingsMode))
             {
                 return EnumHelper<PresetPictureSettingsMode>.GetValue(value.ToString());
@@ -203,6 +226,15 @@ namespace HandBrakeWPF.Converters
             {
                 return EnumHelper<Detelecine>.GetValue(value.ToString());
             }
+            if (targetType == typeof(VideoScaler) || value.GetType() == typeof(VideoScaler))
+            {
+                return EnumHelper<VideoScaler>.GetValue(value.ToString());
+            }
+
+            if (targetType == typeof(OutputFormat) || value.GetType() == typeof(OutputFormat))
+            {
+                return EnumHelper<OutputFormat>.GetValue(value.ToString());
+            }
 
             return null;
         }
