@@ -18,7 +18,9 @@ namespace HandBrakeWPF.Helpers
     using HandBrake.ApplicationServices.Extensions;
     using HandBrake.ApplicationServices.Model;
     using HandBrake.ApplicationServices.Model.Encoding;
-    using HandBrake.ApplicationServices.Services.Interfaces;
+    using HandBrake.Interop.Model.Encoding;
+
+    using HandBrakeWPF.Services.Interfaces;
 
     /// <summary>
     /// The Destination AutoName Helper
@@ -47,7 +49,7 @@ namespace HandBrakeWPF.Helpers
             }
 
             string autoNamePath = string.Empty;
-            if (task.Title != 0) // TODO check.
+            if (task.Title != 0)
             {
                 // Get the Source Name and remove any invalid characters
                 string sourceName = Path.GetInvalidFileNameChars().Aggregate(sourceOrLabelName, (current, character) => current.Replace(character.ToString(), string.Empty));
@@ -56,6 +58,13 @@ namespace HandBrakeWPF.Helpers
                 if (userSettingService.GetUserSetting<bool>(UserSettingConstants.AutoNameRemoveUnderscore))
                     sourceName = sourceName.Replace("_", " ");
 
+                if (userSettingService.GetUserSetting<bool>(UserSettingConstants.RemovePunctuation))
+                {
+                    sourceName = sourceName.Replace("-", string.Empty);
+                    sourceName = sourceName.Replace(",", string.Empty);
+                    sourceName = sourceName.Replace(".", string.Empty); 
+                }
+                  
                 // Switch to "Title Case"
                 if (userSettingService.GetUserSetting<bool>(UserSettingConstants.AutoNameTitleCase))
                     sourceName = sourceName.ToTitleCase();
@@ -79,9 +88,21 @@ namespace HandBrakeWPF.Helpers
                 {
                     destinationFilename = userSettingService.GetUserSetting<string>(UserSettingConstants.AutoNameFormat);
                     destinationFilename = destinationFilename.Replace("{source}", sourceName)
-                                                             .Replace("{title}", dvdTitle)
-                                                             .Replace("{chapters}", combinedChapterTag)
-                                                             .Replace("{date}", DateTime.Now.Date.ToShortDateString().Replace('/', '-'));
+                                                             .Replace(Constants.Title, dvdTitle)
+                                                             .Replace(Constants.Chapters, combinedChapterTag)
+                                                             .Replace(Constants.Date, DateTime.Now.Date.ToShortDateString().Replace('/', '-'))
+                                                             .Replace(Constants.Time, DateTime.Now.ToString("HH:mm"));
+
+                    if (task.VideoEncodeRateType == VideoEncodeRateType.ConstantQuality)
+                    {
+                        destinationFilename = destinationFilename.Replace(Constants.Quality, task.Quality.ToString());
+                        destinationFilename = destinationFilename.Replace(Constants.Bitrate, string.Empty);
+                    }
+                    else
+                    {
+                        destinationFilename = destinationFilename.Replace(Constants.Bitrate, task.VideoBitrate.ToString());
+                        destinationFilename = destinationFilename.Replace(Constants.Quality, string.Empty);
+                    }
                 }
                 else
                     destinationFilename = sourceName + "_T" + dvdTitle + "_C" + combinedChapterTag;
@@ -89,7 +110,7 @@ namespace HandBrakeWPF.Helpers
                 /*
                  * File Extension
                  */
-                if (task.OutputFormat == OutputFormat.Mp4 || task.OutputFormat == OutputFormat.M4V)
+                if (task.OutputFormat == OutputFormat.Mp4)
                 {
                     switch (userSettingService.GetUserSetting<int>(UserSettingConstants.UseM4v))
                     {
@@ -170,21 +191,16 @@ namespace HandBrakeWPF.Helpers
         public static bool IsAutonamingEnabled()
         {
             IUserSettingService userSettingService = IoC.Get<IUserSettingService>();
+
+            if (!userSettingService.GetUserSetting<bool>(UserSettingConstants.AutoNaming))
+            {
+                return false;
+            }
+
             // If there is an auto name path, use it...
-            if (userSettingService.GetUserSetting<string>(UserSettingConstants.AutoNamePath).Trim().StartsWith("{source_path}"))
-            {
-                return true;
-            }
-            else if (userSettingService.GetUserSetting<string>(UserSettingConstants.AutoNamePath).Contains("{source_folder_name}"))
-            {
-                return true;
-            }
-            else
-            {
-                return
-                    Directory.Exists(
-                        userSettingService.GetUserSetting<string>(UserSettingConstants.AutoNamePath).Trim());
-            }
+            return userSettingService.GetUserSetting<string>(UserSettingConstants.AutoNamePath).Trim().StartsWith("{source_path}") ||
+                (userSettingService.GetUserSetting<string>(UserSettingConstants.AutoNamePath).Contains("{source_folder_name}") ||
+                 Directory.Exists(userSettingService.GetUserSetting<string>(UserSettingConstants.AutoNamePath).Trim()));
         }
     }
 }
