@@ -17,9 +17,9 @@ namespace HandBrakeWPF
         #region Constants and Fields
 
         /// <summary>
-        /// Add Only One Per Langage
+        /// HandBrakes build
         /// </summary>
-        public const string AddOnlyOneAudioPerLanguage = "addOnlyOneAudioPerLanguage";
+        public const string HandBrakeBuild = "HandBrakeBuild";
 
         /// <summary>
         /// Auto name format
@@ -61,21 +61,6 @@ namespace HandBrakeWPF
         /// </summary>
         public const string DefaultPlayer = "defaultPlayer";
 
-        /// <summary>
-        /// Dub mode
-        /// </summary>
-        public const string DubMode = "DubMode";
-
-        /// <summary>
-        /// Dub Mode Audio
-        /// </summary>
-        public const string DubModeAudio = "DubModeAudio";
-
-        /// <summary>
-        /// Dub Mode Subs
-        /// </summary>
-        public const string DubModeSubtitle = "DubModeSubtitle";
-
         /// <summary>
         /// Last Update Check
         /// </summary>
@@ -91,31 +76,11 @@ namespace HandBrakeWPF
         /// </summary>
         public const string MinTitleLength = "MinTitleLength";
 
-        /// <summary>
-        /// Native Language Audio
-        /// </summary>
-        public const string NativeLanguage = "NativeLanguage";
-
-        /// <summary>
-        /// Native Language Subs
-        /// </summary>
-        public const string NativeLanguageForSubtitles = "NativeLanguageSubtitles";
-
         /// <summary>
         /// Preset Notification
         /// </summary>
         public const string PresetNotification = "presetNotification";
 
-        /// <summary>
-        /// Selected Languages
-        /// </summary>
-        public const string SelectedLanguages = "SelectedLanguages";
-
-        /// <summary>
-        /// AUudio Passthru
-        /// </summary>
-        public const string ShowAdvancedAudioPassthruOpts = "ShowAdvancedAudioPassthruOpts";
-
         /// <summary>
         /// Skip Version
         /// </summary>
@@ -126,11 +91,6 @@ namespace HandBrakeWPF
         /// </summary>
         public const string UpdateStatus = "updateStatus";
 
-        /// <summary>
-        /// Closed Captions
-        /// </summary>
-        public const string UseClosedCaption = "useClosedCaption";
-
         /// <summary>
         /// Use m4v
         /// </summary>
@@ -139,7 +99,7 @@ namespace HandBrakeWPF
         /// <summary>
         /// Vlc Path
         /// </summary>
-        public const string VLC_Path = "VLC_Path";
+        public const string VLCPath = "VLC_Path";
 
         /// <summary>
         /// The enable process isolation.
@@ -151,11 +111,6 @@ namespace HandBrakeWPF
         /// </summary>
         public const string ServerPort = "ServerPort";
 
-        /// <summary>
-        /// Enable Debugging features within the GUI.
-        /// </summary>
-        public const string EnableDebugFeatures = "EnableDebugFeatures";
-
         /// <summary>
         /// Enable the use of LibHb instead of HandBrakeCLI
         /// </summary>
@@ -195,11 +150,116 @@ namespace HandBrakeWPF
         /// The last preview duration
         /// </summary>
         public const string LastPreviewDuration = "LastPreviewDuration";
+    
+        /// <summary>
+        /// When Complete Action
+        /// </summary>
+        public const string WhenCompleteAction = "WhenCompleteAction";
 
         /// <summary>
-        /// Disable LibHb Features
+        /// Send file enabled.
         /// </summary>
-        public const string DisableLibHbFeatures = "DisableLibHbFeatures";
+        public const string SendFile = "SendFile";
+
+        /// <summary>
+        /// Send file to application path
+        /// </summary>
+        public const string SendFileTo = "SendFileTo";
+
+        /// <summary>
+        /// Send file to arguments
+        /// </summary>
+        public const string SendFileToArgs = "SendFileToArgs";
+
+        /// <summary>
+        /// Prevent Sleep
+        /// </summary>
+        public const string PreventSleep = "PreventSleep";
+
+        /// <summary>
+        /// The remove punctuation.
+        /// </summary>
+        public const string RemovePunctuation = "RemovePunctuation";
+
+        /// <summary>
+        /// The Show Preset Panel
+        /// </summary>
+        public const string ShowPresetPanel = "ShowPresetPanel";
+
+        /// <summary>
+        /// The use system colours.
+        /// </summary>
+        public const string UseSystemColours = "UseSystemColours";
+
+        /// <summary>
+        /// The reset when done action.
+        /// </summary>
+        public const string ResetWhenDoneAction = "ResetWhenDoneAction";
+
+        /// <summary>
+        /// The disable lib dvd nav.
+        /// </summary>
+        public const string DisableLibDvdNav = "DisableLibDvdNav";
+
+        /// <summary>
+        /// The disable quick sync decoding.
+        /// </summary>
+        public const string DisableQuickSyncDecoding = "DisableQuickSyncDecoding";
+
+        /// <summary>
+        /// The enable dxva.
+        /// </summary>
+        public const string EnableDxva = "EnableDxva";
+
+        /// <summary>
+        /// The scaling mode.
+        /// </summary>
+        public const string ScalingMode = "ScalingMode";
+
+        /// <summary>
+        /// Preview Scan Count
+        /// </summary>
+        public const string PreviewScanCount = "previewScanCount";
+
+        /// <summary>
+        /// The Verbosity
+        /// </summary>
+        public const string Verbosity = "Verbosity";
+
+        /// <summary>
+        /// Min Title Scan Duration
+        /// </summary>
+        public const string MinScanDuration = "MinTitleScanDuration";
+
+        /// <summary>
+        /// Process Priority
+        /// </summary>
+        public const string ProcessPriority = "ProcessPriority";
+
+        /// <summary>
+        /// Save Log Directory
+        /// </summary>
+        public const string SaveLogToCopyDirectory = "SaveLogToCopyDirectory";
+
+        /// <summary>
+        /// Save log with video
+        /// </summary>
+        public const string SaveLogWithVideo = "SaveLogWithVideo";
+
+        /// <summary>
+        /// Save copy of the log to a directory
+        /// </summary>
+        public const string SaveLogCopyDirectory = "SaveLogCopyDirectory";
+
+        /// <summary>
+        /// The clear completed from queue.
+        /// </summary>
+        public const string ClearCompletedFromQueue = "ClearCompletedFromQueue";
+
+        /// <summary>
+        /// The enable static preview.
+        /// </summary>
+        public const string EnableStaticPreview = "EnableStaticPreview";
 
         #endregion
     }
