@@ -13,10 +13,15 @@ namespace HandBrakeWPF.ViewModels
     using System.Windows;
 
     using HandBrake.ApplicationServices.Model;
+    using HandBrake.ApplicationServices.Model.Audio;
+    using HandBrake.ApplicationServices.Model.Subtitle;
+    using HandBrake.ApplicationServices.Parsing;
     using HandBrake.ApplicationServices.Services;
     using HandBrake.ApplicationServices.Services.Interfaces;
     using HandBrake.ApplicationServices.Utilities;
+    using HandBrake.Interop.Model.Encoding;
 
+    using HandBrakeWPF.Properties;
     using HandBrakeWPF.Services.Interfaces;
     using HandBrakeWPF.ViewModels.Interfaces;
 
@@ -25,8 +30,6 @@ namespace HandBrakeWPF.ViewModels
     /// </summary>
     public class AddPresetViewModel : ViewModelBase, IAddPresetViewModel
     {
-        /* TODO this window is up for redesign. Quite a few nippy edge cases that can cause odd behaviour with importing presets. */
-
         /// <summary>
         /// Backing field for the Preset Service
         /// </summary>
@@ -47,6 +50,11 @@ namespace HandBrakeWPF.ViewModels
         /// </summary>
         private bool showCustomInputs;
 
+        /// <summary>
+        /// The source.
+        /// </summary>
+        private Title selectedTitle;
+
         /// <summary>
         /// Initializes a new instance of the <see cref="AddPresetViewModel"/> class.
         /// </summary>
@@ -61,7 +69,7 @@ namespace HandBrakeWPF.ViewModels
             this.presetService = presetService;
             this.errorService = errorService;
             this.Title = "Add Preset";
-            this.Preset = new Preset { IsBuildIn = false, IsDefault = false, Category = PresetService.UserPresetCatgoryName, UsePictureFilters = true};
+            this.Preset = new Preset { IsBuildIn = false, IsDefault = false, Category = PresetService.UserPresetCatgoryName, UsePictureFilters = true };
             this.PictureSettingsModes = EnumHelper<PresetPictureSettingsMode>.GetEnumList();
         }
 
@@ -78,12 +86,12 @@ namespace HandBrakeWPF.ViewModels
         /// <summary>
         /// Gets or sets CustomWidth.
         /// </summary>
-        public int CustomWidth { get; set; }
+        public int? CustomWidth { get; set; }
 
         /// <summary>
         /// Gets or sets CustomHeight.
         /// </summary>
-        public int CustomHeight { get; set; }
+        public int? CustomHeight { get; set; }
 
         /// <summary>
         /// Gets or sets a value indicating whether ShowCustomInputs.
@@ -123,9 +131,36 @@ namespace HandBrakeWPF.ViewModels
         /// <param name="task">
         /// The Encode Task.
         /// </param>
-        public void Setup(EncodeTask task)
+        /// <param name="title">
+        /// The title.
+        /// </param>
+        /// <param name="audioBehaviours">
+        /// The audio Behaviours.
+        /// </param>
+        /// <param name="subtitleBehaviours">
+        /// The subtitle Behaviours.
+        /// </param>
+        public void Setup(EncodeTask task, Title title, AudioBehaviours audioBehaviours, SubtitleBehaviours subtitleBehaviours)
         {
             this.Preset.Task = new EncodeTask(task);
+            this.Preset.AudioTrackBehaviours = audioBehaviours.Clone();
+            this.Preset.SubtitleTrackBehaviours = subtitleBehaviours.Clone();
+            this.selectedTitle = title;
+
+            switch (task.Anamorphic)
+            {
+                default:
+                    this.SelectedPictureSettingMode = PresetPictureSettingsMode.Custom;
+                    if (title != null && title.Resolution != null)
+                    {
+                        this.CustomWidth = title.Resolution.Width;
+                        this.CustomHeight = title.Resolution.Height;
+                    }
+                    break;
+                case Anamorphic.Strict:
+                    this.SelectedPictureSettingMode = PresetPictureSettingsMode.SourceMaximum;
+                    break;
+            }
         }
 
         /// <summary>
@@ -135,22 +170,28 @@ namespace HandBrakeWPF.ViewModels
         {
             if (string.IsNullOrEmpty(this.Preset.Name))
             {
-                this.errorService.ShowMessageBox("A Preset must have a Name. Please fill out the Preset Name field.", "Error", MessageBoxButton.OK, MessageBoxImage.Error);
+                this.errorService.ShowMessageBox("A Preset must have a Name. Please fill out the Preset Name field.", Resources.Error, MessageBoxButton.OK, MessageBoxImage.Error);
                 return;
             }
 
             if (this.presetService.CheckIfPresetExists(this.Preset.Name))
             {
-               MessageBoxResult result = this.errorService.ShowMessageBox("A Preset with this name already exists. Would you like to overwrite it?", "Error", MessageBoxButton.YesNo, MessageBoxImage.Error);
-               if (result == MessageBoxResult.No)
-               {
-                   return;
-               }
+                MessageBoxResult result = this.errorService.ShowMessageBox("A Preset with this name already exists. Would you like to overwrite it?", Resources.Error, MessageBoxButton.YesNo, MessageBoxImage.Error);
+                if (result == MessageBoxResult.No)
+                {
+                    return;
+                }
             }
 
-            if (this.SelectedPictureSettingMode == PresetPictureSettingsMode.SourceMaximum && (this.Preset.Task.Width == null || this.Preset.Task.Width == 0))
+            if (this.SelectedPictureSettingMode == PresetPictureSettingsMode.SourceMaximum && this.selectedTitle == null)
             {
-                this.errorService.ShowMessageBox("You must first scan a source to use the 'Source Maximum' Option.", "Error", MessageBoxButton.OK, MessageBoxImage.Error);
+                this.errorService.ShowMessageBox("You must first scan a source to use the 'Source Maximum' Option.", Resources.Error, MessageBoxButton.OK, MessageBoxImage.Error);
+                return;
+            }
+
+            if (this.CustomWidth == null && this.CustomHeight == null && this.SelectedPictureSettingMode == PresetPictureSettingsMode.Custom)
+            {
+                this.errorService.ShowMessageBox("The Custom Width or Height fields must be filled in for the 'Custom' option.", Resources.Error, MessageBoxButton.OK, MessageBoxImage.Error);
                 return;
             }
 
@@ -174,8 +215,8 @@ namespace HandBrakeWPF.ViewModels
 
             if (this.SelectedPictureSettingMode == PresetPictureSettingsMode.SourceMaximum)
             {
-                this.Preset.Task.MaxWidth = this.Preset.Task.Width;
-                this.Preset.Task.MaxHeight = this.Preset.Task.Height;
+                this.Preset.Task.MaxWidth = selectedTitle.Resolution.Width;
+                this.Preset.Task.MaxHeight = selectedTitle.Resolution.Height;
             }
 
             // Add the Preset
