@@ -9,23 +9,21 @@
 
 namespace HandBrakeWPF.ViewModels
 {
-    using System;
     using System.Collections.Generic;
     using System.Collections.ObjectModel;
-    using System.Collections.Specialized;
+    using System.ComponentModel;
     using System.Linq;
 
     using Caliburn.Micro;
 
     using HandBrake.ApplicationServices.Model;
+    using HandBrake.ApplicationServices.Model.Audio;
     using HandBrake.ApplicationServices.Model.Encoding;
     using HandBrake.ApplicationServices.Parsing;
-    using HandBrake.ApplicationServices.Services.Interfaces;
     using HandBrake.ApplicationServices.Utilities;
     using HandBrake.Interop.Model.Encoding;
 
-    using HandBrakeWPF.Commands;
-    using HandBrakeWPF.Model;
+    using HandBrakeWPF.Services.Interfaces;
     using HandBrakeWPF.ViewModels.Interfaces;
 
     /// <summary>
@@ -38,8 +36,26 @@ namespace HandBrakeWPF.ViewModels
         /// </summary>
         private IEnumerable<Audio> sourceTracks;
 
+        /// <summary>
+        /// The current preset.
+        /// </summary>
         private Preset currentPreset;
 
+        /// <summary>
+        /// The show audio defaults panel.
+        /// </summary>
+        private bool showAudioDefaultsPanel;
+
+        /// <summary>
+        /// The available languages.
+        /// </summary>
+        private BindingList<string> availableLanguages;
+
+        /// <summary>
+        /// The audio behaviours.
+        /// </summary>
+        private AudioBehaviours audioBehaviours;
+
         #region Constructors and Destructors
 
         /// <summary>
@@ -58,12 +74,49 @@ namespace HandBrakeWPF.ViewModels
             this.AudioEncoders = EnumHelper<AudioEncoder>.GetEnumList();
             this.AudioMixdowns = EnumHelper<Mixdown>.GetEnumList();
             this.SourceTracks = new List<Audio>();
+
+            this.AudioBehaviours = new AudioBehaviours();
+            this.SelectedAvailableToMove = new BindingList<string>();
+            this.SelectedLangaugesToMove = new BindingList<string>();
+            this.AvailableLanguages = new BindingList<string>();
+            this.SetupLanguages(null);
         }
 
         #endregion
 
         #region Properties
 
+        /// <summary>
+        /// Gets or sets the audio behaviours.
+        /// </summary>
+        public AudioBehaviours AudioBehaviours
+        {
+            get
+            {
+                return this.audioBehaviours;
+            }
+            set
+            {
+                if (Equals(value, this.audioBehaviours))
+                {
+                    return;
+                }
+                this.audioBehaviours = value;
+                this.NotifyOfPropertyChange(() => this.AudioBehaviours);
+            }
+        }
+
+        /// <summary>
+        /// Gets the audio behaviour modes.
+        /// </summary>
+        public BindingList<AudioBehaviourModes> AudioBehaviourModeList
+        {
+            get
+            {
+                return new BindingList<AudioBehaviourModes>(EnumHelper<AudioBehaviourModes>.GetEnumList().ToList());
+            }
+        }
+
         /// <summary>
         /// Gets or sets AudioBitrates.
         /// </summary>
@@ -106,16 +159,76 @@ namespace HandBrakeWPF.ViewModels
         public EncodeTask Task { get; set; }
 
         /// <summary>
-        /// Gets a value indicating whether ShowPassthruOptions.
+        /// Gets or sets a value indicating whether show audio defaults panel.
         /// </summary>
-        public bool ShowPassthruOptions
+        public bool ShowAudioDefaultsPanel
         {
             get
             {
-                return this.UserSettingService.GetUserSetting<bool>(UserSettingConstants.ShowAdvancedAudioPassthruOpts);
+                return this.showAudioDefaultsPanel;
+            }
+            set
+            {
+                if (value.Equals(this.showAudioDefaultsPanel))
+                {
+                    return;
+                }
+                this.showAudioDefaultsPanel = value;
+                this.NotifyOfPropertyChange(() => this.ShowAudioDefaultsPanel);
+                this.NotifyOfPropertyChange(() => this.PanelTitle);
+                this.NotifyOfPropertyChange(() => this.SwitchDisplayTitle);
             }
         }
 
+        /// <summary>
+        /// Gets the panel title.
+        /// </summary>
+        public string PanelTitle
+        {
+            get
+            {
+                return this.ShowAudioDefaultsPanel ? "Audio Defaults" : "Audio Tracks";
+            }
+        }
+
+        /// <summary>
+        /// Gets the switch display title.
+        /// </summary>
+        public string SwitchDisplayTitle
+        {
+            get
+            {
+                return this.ShowAudioDefaultsPanel ? "Switch to Tracks" : "Switch to Defaults";
+            }
+        }
+
+        /// <summary>
+        /// Gets or sets AvailableLanguages.
+        /// </summary>
+        public BindingList<string> AvailableLanguages
+        {
+            get
+            {
+                return this.availableLanguages;
+            }
+
+            set
+            {
+                this.availableLanguages = value;
+                this.NotifyOfPropertyChange("AvailableLanguages");
+            }
+        }
+
+        /// <summary>
+        /// Gets or sets SelectedLangauges.
+        /// </summary>
+        public BindingList<string> SelectedAvailableToMove { get; set; }
+
+        /// <summary>
+        /// Gets or sets SelectedLangauges.
+        /// </summary>
+        public BindingList<string> SelectedLangaugesToMove { get; set; }
+
         #endregion
 
         #region Public Methods
@@ -177,16 +290,71 @@ namespace HandBrakeWPF.ViewModels
         /// </summary>
         public void SetDefaultBehaviour()
         {
-            OpenOptionsScreenCommand command = new OpenOptionsScreenCommand();
-            command.Execute(OptionsTab.AudioAndSubtitles);
+            this.ShowAudioDefaultsPanel = true;
+        }
+
+        /// <summary>
+        /// The show audio defaults.
+        /// </summary>
+        public void ShowAudioDefaults()
+        {
+            this.ShowAudioDefaultsPanel = !this.ShowAudioDefaultsPanel;
+        }
+
+        /// <summary>
+        /// Audio List Move Left
+        /// </summary>
+        public void LanguageMoveRight()
+        {
+            if (this.SelectedAvailableToMove.Count > 0)
+            {
+                List<string> copiedList = SelectedAvailableToMove.ToList();
+                foreach (string item in copiedList)
+                {
+                    this.AvailableLanguages.Remove(item);
+                    this.AudioBehaviours.SelectedLangauges.Add(item);
+                }
+
+                this.AvailableLanguages = new BindingList<string>(this.AvailableLanguages.OrderBy(o => o).ToList());
+            }
+        }
+
+        /// <summary>
+        /// Audio List Move Right
+        /// </summary>
+        public void LanguageMoveLeft()
+        {
+            if (this.SelectedLangaugesToMove.Count > 0)
+            {
+                List<string> copiedList = SelectedLangaugesToMove.ToList();
+                foreach (string item in copiedList)
+                {
+                    this.AudioBehaviours.SelectedLangauges.Remove(item);
+                    this.AvailableLanguages.Add(item);
+                }
+            }
+
+            this.AvailableLanguages = new BindingList<string>(this.AvailableLanguages.OrderBy(o => o).ToList());
+        }
+
+        /// <summary>
+        /// Audio List Clear all selected languages
+        /// </summary>
+        public void LanguageClearAll()
+        {
+            foreach (string item in this.AudioBehaviours.SelectedLangauges)
+            {
+                this.AvailableLanguages.Add(item);
+            }
+            this.AvailableLanguages = new BindingList<string>(this.AvailableLanguages.OrderBy(o => o).ToList());
+
+            this.AudioBehaviours.SelectedLangauges.Clear();
         }
 
         #endregion
 
         #region Implemented Interfaces
 
-        #region ITabInterface
-
         /// <summary>
         /// Setup this tab for the specified preset.
         /// </summary>
@@ -201,11 +369,16 @@ namespace HandBrakeWPF.ViewModels
             this.Task = task;
             this.currentPreset = preset;
 
+            // Audio Behaviours
+            this.SetupLanguages(preset);
+
             if (preset != null && preset.Task != null)
             {
-                this.AddTracksFromPreset(preset);
                 this.Task.AllowedPassthruOptions = new AllowedPassthru(preset.Task.AllowedPassthruOptions);
+
+                this.SetupTracks();
             }
+
             this.NotifyOfPropertyChange(() => this.Task);
         }
 
@@ -239,7 +412,7 @@ namespace HandBrakeWPF.ViewModels
             this.SourceTracks = title.AudioTracks;
 
             // Only reset the audio tracks if we have none, or if the task is null.
-            if (this.Task == null || this.Task.AudioTracks.Count == 0)
+            if (this.Task == null)
             {
                 this.SetPreset(preset, task);
             }
@@ -251,7 +424,7 @@ namespace HandBrakeWPF.ViewModels
             }
             else
             {
-                this.AutomaticTrackSelection();
+                this.SetupTracks();
             }
 
             // Force UI Updates
@@ -260,8 +433,6 @@ namespace HandBrakeWPF.ViewModels
 
         #endregion
 
-        #endregion
-
         #region Methods
 
         /// <summary>
@@ -290,12 +461,6 @@ namespace HandBrakeWPF.ViewModels
             // For all the source audio tracks
             foreach (Audio sourceTrack in this.SourceTracks)
             {
-                // Step 1:  If "Add only One per language" is turned on, check to see if this language is already added.
-                if (this.CanSkipSourceTrack(sourceTrack))
-                {
-                    continue;
-                }
-
                 // Step 2: Check if the track list already contrains this track
                 bool found = this.Task.AudioTracks.Any(audioTrack => Equals(audioTrack.ScannedTrack, sourceTrack));
                 if (!found)
@@ -306,6 +471,75 @@ namespace HandBrakeWPF.ViewModels
             }
         }
 
+        /// <summary>
+        /// Attempt to automatically select the correct audio tracks based on the users settings.
+        /// </summary>
+        private void SetupTracks()
+        {
+            if (!this.SourceTracks.Any())
+            {
+                // Clear out the old tracks
+                this.Task.AudioTracks.Clear();
+
+                return;
+            }
+
+            // Step 1, Cleanup Previous Tracks
+            this.Task.AudioTracks.Clear();
+
+            // Step 2, Sanity Check
+            if (this.SourceTracks == null || !this.SourceTracks.Any())
+            {
+                return;
+            }
+
+            // Step 3, Setup the tracks from the preset
+            foreach (AudioTrack track in this.currentPreset.Task.AudioTracks)
+            {
+                this.Task.AudioTracks.Add(new AudioTrack(track) { ScannedTrack = this.GetPreferredAudioTrack() });
+            }
+           
+            // Step 4, Handle the default selection behaviour.
+            switch (this.AudioBehaviours.SelectedBehaviour)
+            {
+                case AudioBehaviourModes.FirstMatch: // Adding all remaining audio tracks
+                    this.AddFirstForSelectedLanguages();
+                    break;
+                case AudioBehaviourModes.AllMatching: // Add Langauges tracks for the additional languages selected, in-order.
+                    this.AddAllRemainingForSelectedLanguages();
+                    break;
+            }
+        }
+
+        /// <summary>
+        /// The add first for selected languages.
+        /// </summary>
+        private void AddFirstForSelectedLanguages()
+        {
+            foreach (Audio sourceTrack in this.GetSelectedLanguagesTracks())
+            {
+                // Step 2: Check if the track list already contrains this track
+                bool found = this.Task.AudioTracks.Any(audioTrack => Equals(audioTrack.ScannedTrack, sourceTrack));
+                if (!found)
+                {
+                    // Check if we are already using this language
+                    bool foundLanguage = false;
+                    foreach (var item in this.Task.AudioTracks.Where(item => item.ScannedTrack != null && sourceTrack.LanguageCode.Contains(item.ScannedTrack.LanguageCode)))
+                    {
+                        foundLanguage = true;
+                    }
+
+                    if (foundLanguage)
+                    {
+                        continue;
+                    }
+
+                    // If it doesn't, add it.
+                    this.Add(sourceTrack);
+                }
+            }
+        }
+
         /// <summary>
         /// Add all remaining for selected languages.
         /// </summary>
@@ -314,12 +548,6 @@ namespace HandBrakeWPF.ViewModels
             // Add them if they are not already added.
             foreach (Audio sourceTrack in this.GetSelectedLanguagesTracks())
             {
-                // Step 1:  If "Add only One per language" is turned on, check to see if this language is already added.
-                if (this.CanSkipSourceTrack(sourceTrack))
-                {
-                    continue;
-                }
-
                 // Step 2: Check if the track list already contrains this track
                 bool found = this.Task.AudioTracks.Any(audioTrack => Equals(audioTrack.ScannedTrack, sourceTrack));
                 if (!found)
@@ -330,64 +558,6 @@ namespace HandBrakeWPF.ViewModels
             }
         }
 
-        /// <summary>
-        /// Add the required tracks for the current preset
-        /// </summary>
-        /// <param name="preset">
-        /// The preset.
-        /// </param>
-        private void AddTracksFromPreset(Preset preset)
-        {
-            // Clear out the old tracks
-            this.Task.AudioTracks.Clear();
-
-            // Add the preset audio tracks with the preferred language
-            foreach (AudioTrack track in preset.Task.AudioTracks)
-            {
-                this.Task.AudioTracks.Add(new AudioTrack(track) { ScannedTrack = this.GetPreferredAudioTrack() });
-            }
-        }
-
-        /// <summary>
-        /// Attempt to automatically select the correct audio tracks based on the users settings.
-        /// </summary>
-        private void AutomaticTrackSelection()
-        {
-            if (!this.SourceTracks.Any())
-            {
-                // Clear out the old tracks
-                this.Task.AudioTracks.Clear();
-
-                return;
-            }
-
-            // We've changed source, so lets try reset the language, description and formats as close as possible to the previous track.
-            foreach (AudioTrack track in this.Task.AudioTracks)
-            {
-                track.ScannedTrack = this.GetPreferredAudioTrack();
-            }
-
-            // Handle the default selection behaviour.
-            int mode = this.UserSettingService.GetUserSetting<int>(UserSettingConstants.DubModeAudio);
-            if (mode == 1 || mode == 2)
-            {
-                // First, we'll clear out all current tracks and go back to what the current preset has.
-                // This will alteast provide a consistent behavior when switching tracks.
-                this.Task.AudioTracks.Clear();
-                this.AddTracksFromPreset(this.currentPreset);
-            }
-
-            switch (mode)
-            {
-                case 1: // Adding all remaining audio tracks
-                    this.AddAllRemaining();
-                    break;
-                case 2: // Add Langauges tracks for the additional languages selected, in-order.
-                    this.AddAllRemainingForSelectedLanguages();
-                    break;
-            }
-        }
-
         /// <summary>
         /// The get preferred audio track, or the first if none available.
         /// </summary>
@@ -396,12 +566,18 @@ namespace HandBrakeWPF.ViewModels
         /// </returns>
         private Audio GetPreferredAudioTrack()
         {
-            // Get the preferred Language
-            IEnumerable<Audio> preferredAudioTracks =
-                this.SourceTracks.Where(
-                    item =>
-                    item.Language.Contains(
-                        this.UserSettingService.GetUserSetting<string>(UserSettingConstants.NativeLanguage)));
+            // The first track in the selected languages list is considered the preferred language.
+            // So, try match tracks on this.
+            IEnumerable<Audio> preferredAudioTracks = new List<Audio>();
+            if (this.AudioBehaviours.SelectedLangauges.Count > 0)
+            {
+                string langName = this.AudioBehaviours.SelectedLangauges.FirstOrDefault(w => !w.Equals(Constants.Any));
+                if (!string.IsNullOrEmpty(langName))
+                {
+                    preferredAudioTracks = this.SourceTracks.Where(item => item.Language.Contains(langName));
+                }
+            }
+
             return preferredAudioTracks.FirstOrDefault() ?? this.SourceTracks.FirstOrDefault();
         }
 
@@ -414,34 +590,54 @@ namespace HandBrakeWPF.ViewModels
         private IEnumerable<Audio> GetSelectedLanguagesTracks()
         {
             List<Audio> trackList = new List<Audio>();
-            foreach (string language in this.UserSettingService.GetUserSetting<StringCollection>(UserSettingConstants.SelectedLanguages))
+
+            List<string> isoCodes = this.AudioBehaviours.SelectedLangauges.Contains(Constants.Any)
+                                            ? LanguageUtilities.GetIsoCodes()
+                                            : LanguageUtilities.GetLanguageCodes(
+                                                this.AudioBehaviours.SelectedLangauges.ToArray());
+
+            foreach (string code in isoCodes)
             {
-                trackList.AddRange(this.SourceTracks.Where(source => source.Language.Trim() == language));
+                trackList.AddRange(this.SourceTracks.Where(source => source.LanguageCode.Trim() == code));
             }
 
             return trackList;
         }
 
         /// <summary>
-        /// Checks to see if we can skip over the given source audio track.
-        /// True when the user has set "Add only one per language" feature AND the language is contained in the track list.
+        /// The setup languages.
         /// </summary>
-        /// <param name="sourceTrack">
-        /// The source track.
+        /// <param name="preset">
+        /// The preset.
         /// </param>
-        /// <returns>
-        /// True when the user has set "Add only one per language" feature AND the language is contained in the track list
-        /// </returns>
-        private bool CanSkipSourceTrack(Audio sourceTrack)
+        private void SetupLanguages(Preset preset)
         {
-            bool addOnlyOnePerLanguage = this.UserSettingService.GetUserSetting<bool>(UserSettingConstants.AddOnlyOneAudioPerLanguage);
-            bool sourceTrackLanguageFound = this.Task.AudioTracks.Any(audioTrack => audioTrack.ScannedTrack != null && sourceTrack.Language == audioTrack.ScannedTrack.Language);
-            if (addOnlyOnePerLanguage && sourceTrackLanguageFound)
+            // Step 1, Set the behaviour mode
+            this.AudioBehaviours.SelectedBehaviour = AudioBehaviourModes.None;
+            this.AudioBehaviours.SelectedLangauges.Clear();
+
+            // Step 2, Get all the languages
+            IDictionary<string, string> langList = LanguageUtilities.MapLanguages();
+            langList = (from entry in langList orderby entry.Key ascending select entry).ToDictionary(pair => pair.Key, pair => pair.Value);
+
+            // Step 3, Setup Available Languages
+            this.AvailableLanguages.Clear();
+            foreach (string item in langList.Keys)
             {
-                return true; // This track can be skipped.
+                this.AvailableLanguages.Add(item);
             }
 
-            return false;
+            // Step 4, Set the Selected Languages          
+            if (preset != null && preset.AudioTrackBehaviours != null)
+            {
+                this.AudioBehaviours.SelectedBehaviour = preset.AudioTrackBehaviours.SelectedBehaviour;
+
+                foreach (string selectedItem in preset.AudioTrackBehaviours.SelectedLangauges)
+                {
+                    this.AvailableLanguages.Remove(selectedItem);
+                    this.AudioBehaviours.SelectedLangauges.Add(selectedItem);
+                }
+            }
         }
 
         #endregion
