@@ -20,10 +20,12 @@ namespace HandBrakeWPF.ViewModels
     using HandBrake.ApplicationServices.Model;
     using HandBrake.ApplicationServices.Model.Encoding;
     using HandBrake.ApplicationServices.Parsing;
-    using HandBrake.ApplicationServices.Services.Interfaces;
 
+    using HandBrakeWPF.Services.Interfaces;
     using HandBrakeWPF.ViewModels.Interfaces;
 
+    using LumenWorks.Framework.IO.Csv;
+
     using Ookii.Dialogs.Wpf;
 
     /// <summary>
@@ -160,19 +162,17 @@ namespace HandBrakeWPF.ViewModels
             IDictionary<int, string> chapterMap = new Dictionary<int, string>();
             try
             {
-                var sr = new StreamReader(filename);
-                string csv = sr.ReadLine();
-                while (csv != null)
+                using (CsvReader csv = new CsvReader(new StreamReader(filename), false))
                 {
-                    if (csv.Trim() != string.Empty)
+                    while (csv.ReadNextRecord())
                     {
-                        csv = csv.Replace("\\,", "<!comma!>");
-                        string[] contents = csv.Split(',');
-                        int chapter;
-                        int.TryParse(contents[0], out chapter);
-                        chapterMap.Add(chapter, contents[1].Replace("<!comma!>", ","));
+                        if (csv.FieldCount == 2)
+                        {
+                            int chapter;
+                            int.TryParse(csv[0], out chapter);
+                            chapterMap[chapter] = csv[1];
+                        }
                     }
-                    csv = sr.ReadLine();
                 }
             }
             catch (Exception)
