@@ -10,15 +10,17 @@
 namespace HandBrakeWPF.ViewModels
 {
     using System.Collections.Generic;
+    using System.Globalization;
 
     using Caliburn.Micro;
 
     using HandBrake.ApplicationServices.Model;
+    using HandBrake.ApplicationServices.Model.Encoding;
     using HandBrake.ApplicationServices.Parsing;
-    using HandBrake.ApplicationServices.Services.Interfaces;
     using HandBrake.ApplicationServices.Utilities;
     using HandBrake.Interop.Model.Encoding;
 
+    using HandBrakeWPF.Services.Interfaces;
     using HandBrakeWPF.ViewModels.Interfaces;
 
     /// <summary>
@@ -26,6 +28,11 @@ namespace HandBrakeWPF.ViewModels
     /// </summary>
     public class FiltersViewModel : ViewModelBase, IFiltersViewModel
     {
+        /// <summary>
+        /// The is deinterlace mode.
+        /// </summary>
+        private bool isDeinterlaceMode;
+
         #region Constructors and Destructors
 
         /// <summary>
@@ -41,6 +48,7 @@ namespace HandBrakeWPF.ViewModels
         {
             this.CurrentTask = new EncodeTask();
             this.DeblockValue = 4; // OFF
+            this.IsDeinterlaceMode = true;
         }
 
         #endregion
@@ -138,7 +146,7 @@ namespace HandBrakeWPF.ViewModels
         {
             get
             {
-                return this.DeblockValue == 4 ? "Off" : this.DeblockValue.ToString();
+                return this.DeblockValue == 4 ? "Off" : this.DeblockValue.ToString(CultureInfo.InvariantCulture);
             }
         }
 
@@ -233,6 +241,11 @@ namespace HandBrakeWPF.ViewModels
                 // Show / Hide the Custom Control
                 this.ShowDeinterlaceCustom = this.CurrentTask.Deinterlace == Deinterlace.Custom;
                 this.NotifyOfPropertyChange(() => this.ShowDeinterlaceCustom);
+
+                if (value != Deinterlace.Off)
+                {
+                    this.IsDeinterlaceMode = true;
+                }
             }
         }
 
@@ -259,6 +272,11 @@ namespace HandBrakeWPF.ViewModels
                 // Show / Hide the Custom Control
                 this.ShowDecombCustom = this.CurrentTask.Decomb == Decomb.Custom;
                 this.NotifyOfPropertyChange(() => this.ShowDecombCustom);
+
+                if (value != Decomb.Off)
+                {
+                    this.IsDeinterlaceMode = false;
+                }
             }
         }
 
@@ -278,8 +296,13 @@ namespace HandBrakeWPF.ViewModels
                 this.NotifyOfPropertyChange(() => this.SelectedDenoise);
 
                 // Show / Hide the Custom Control
-                this.ShowDenoiseCustom = this.CurrentTask.Denoise == Denoise.Custom;
+                this.ShowDenoiseCustom = this.CurrentTask.Denoise == Denoise.hqdn3d && this.CurrentTask.DenoisePreset == DenoisePreset.Custom;
                 this.NotifyOfPropertyChange(() => this.ShowDenoiseCustom);
+
+                this.SelectedDenoisePreset = this.CurrentTask.Denoise == Denoise.hqdn3d ? DenoisePreset.Weak : DenoisePreset.Ultralight; // Default so we don't have an invalid preset.
+
+                this.NotifyOfPropertyChange(() => this.ShowDenoiseOptions);
+                this.NotifyOfPropertyChange(() => this.ShowDenoiseTune);
             }
         }
 
@@ -324,6 +347,127 @@ namespace HandBrakeWPF.ViewModels
         /// </summary>
         public bool ShowDetelecineCustom { get; set; }
 
+        /// <summary>
+        /// Gets or sets a value indicating whether is deinterlace mode.
+        /// </summary>
+        public bool IsDeinterlaceMode
+        {
+            get
+            {
+                return this.isDeinterlaceMode;
+            }
+            set
+            {
+                if (!object.Equals(this.isDeinterlaceMode, value))
+                {
+                    this.isDeinterlaceMode = value;
+                    this.NotifyOfPropertyChange(() => this.IsDeinterlaceMode);
+
+                    this.DeinterlaceControlText = value ? "Deinterlace:" : "Decomb:";
+
+                    if (value)
+                    {
+                        this.SelectedDecomb = Decomb.Off;
+                    }
+                    else
+                    {
+                        this.SelectedDeInterlace = Deinterlace.Off;
+                    }
+
+                    this.NotifyOfPropertyChange(() => this.DeinterlaceControlText);
+                }
+            }
+        }
+
+        /// <summary>
+        /// Gets or sets the deinterlace control text.
+        /// </summary>
+        public string DeinterlaceControlText { get; set; }
+
+        /// <summary>
+        /// Gets or sets the selected denoise tune.
+        /// </summary>
+        public DenoiseTune SelectedDenoiseTune
+        {
+            get
+            {
+                return this.CurrentTask.DenoiseTune;
+            }
+
+            set
+            {
+                this.CurrentTask.DenoiseTune = value;
+                this.NotifyOfPropertyChange(() => this.SelectedDenoiseTune);
+            }
+        }
+
+        /// <summary>
+        /// Gets or sets the selected denoise preset.
+        /// </summary>
+        public DenoisePreset SelectedDenoisePreset
+        {
+            get
+            {
+                return this.CurrentTask.DenoisePreset;
+            }
+
+            set
+            {
+                this.CurrentTask.DenoisePreset = value;
+                this.NotifyOfPropertyChange(() => this.SelectedDenoisePreset);
+
+                // Show / Hide the Custom Control
+                this.ShowDenoiseCustom = this.CurrentTask.Denoise == Denoise.hqdn3d && this.CurrentTask.DenoisePreset == DenoisePreset.Custom;
+                this.NotifyOfPropertyChange(() => this.ShowDenoiseCustom);
+                this.NotifyOfPropertyChange(() => this.ShowDenoiseOptions);
+                this.NotifyOfPropertyChange(() => this.ShowDenoiseTune);
+            }
+        }
+
+        /// <summary>
+        /// Gets the denoise presets.
+        /// </summary>
+        public IEnumerable<DenoisePreset> DenoisePresets
+        {
+            get
+            {
+                return EnumHelper<DenoisePreset>.GetEnumList();
+            }
+        }
+
+        /// <summary>
+        /// Gets the denoise tunes.
+        /// </summary>
+        public IEnumerable<DenoiseTune> DenoiseTunes
+        {
+            get
+            {
+                return EnumHelper<DenoiseTune>.GetEnumList();
+            }
+        }
+
+        /// <summary>
+        /// Gets a value indicating whether show denoise options.
+        /// </summary>
+        public bool ShowDenoiseOptions
+        {
+            get
+            {
+                return this.SelectedDenoise != Denoise.Off;
+            }
+        }
+
+        /// <summary>
+        /// Gets a value indicating whether show denoise tune.
+        /// </summary>
+        public bool ShowDenoiseTune
+        {
+            get
+            {
+                return this.SelectedDenoise == Denoise.NLMeans;
+            }
+        }
+
         #endregion
 
         #region Implemented Interfaces
@@ -352,6 +496,8 @@ namespace HandBrakeWPF.ViewModels
                 this.SelectedDetelecine = preset.Task.Detelecine;
                 this.Grayscale = preset.Task.Grayscale;
                 this.DeblockValue = preset.Task.Deblock == 0 ? 4 : preset.Task.Deblock;
+                this.SelectedDenoisePreset = preset.Task.DenoisePreset;
+                this.SelectedDenoiseTune = preset.Task.DenoiseTune;
 
                 // Custom Values
                 this.CustomDecomb = preset.Task.CustomDecomb;
