@@ -37,6 +37,17 @@ namespace HandBrakeWPF.ViewModels.Interfaces
         /// <param name="preset">
         /// The preset.
         /// </param>
-        void SetPreset(Preset preset);
+        /// <param name="task">
+        /// The task.
+        /// </param>
+        void SetPreset(Preset preset, EncodeTask task);
+
+        /// <summary>
+        /// Update all the UI controls based on the encode task passed in.
+        /// </summary>
+        /// <param name="task">
+        /// The task.
+        /// </param>
+        void UpdateTask(EncodeTask task);
     }
 }
