@@ -14,5 +14,14 @@ namespace HandBrakeWPF.ViewModels.Interfaces
     /// </summary>
     public interface IVideoViewModel : ITabInterface
     {
+        /// <summary>
+        /// Trigger a Notify Property Changed on the Task to force various UI elements to update.
+        /// </summary>
+        void RefreshTask();
+
+        /// <summary>
+        /// Clear the advanced x264 options.
+        /// </summary>
+        void ClearAdvancedSettings();
     }
 }
