@@ -14,21 +14,22 @@ namespace HandBrakeWPF.ViewModels
     using System.ComponentModel;
     using System.Globalization;
     using System.Linq;
+    using System.Windows;
 
     using Caliburn.Micro;
 
     using HandBrake.ApplicationServices.Model;
     using HandBrake.ApplicationServices.Model.Encoding;
     using HandBrake.ApplicationServices.Parsing;
-    using HandBrake.ApplicationServices.Services.Interfaces;
     using HandBrake.ApplicationServices.Utilities;
     using HandBrake.Interop;
     using HandBrake.Interop.Model.Encoding;
     using HandBrake.Interop.Model.Encoding.x264;
+    using HandBrake.Interop.Model.Encoding.x265;
 
     using HandBrakeWPF.Commands.Interfaces;
-    using HandBrakeWPF.Model;
     using HandBrakeWPF.Properties;
+    using HandBrakeWPF.Services.Interfaces;
     using HandBrakeWPF.ViewModels.Interfaces;
 
     /// <summary>
@@ -36,6 +37,12 @@ namespace HandBrakeWPF.ViewModels
     /// </summary>
     public class VideoViewModel : ViewModelBase, IVideoViewModel
     {
+        /*
+         * TODO
+         * 1. Refactor the Video Encoder Preset/Tune/Level options to be generic instead of encoder specific.
+         * 2. Model the UI Interactions in a better way.
+         */
+
         #region Constants and Fields
         /// <summary>
         /// Same as source constant.
@@ -45,7 +52,7 @@ namespace HandBrakeWPF.ViewModels
         /// <summary>
         /// The possible h264 levels.
         /// </summary>
-        private static readonly List<string> Levels = new List<string> { "Auto", "1.0", "1b", "1.1", "1.2", "1.3", "2.0", "2.1", "2.2", "3.0", "3.1", "3.2", "4.0", "4.1", "4.2", "5.0", "5.1", "5.2"};
+        private static readonly List<string> Levels = new List<string> { "Auto", "1.0", "1b", "1.1", "1.2", "1.3", "2.0", "2.1", "2.2", "3.0", "3.1", "3.2", "4.0", "4.1", "4.2", "5.0", "5.1", "5.2" };
 
         /// <summary>
         /// Backing field for the user setting service.
@@ -62,6 +69,21 @@ namespace HandBrakeWPF.ViewModels
         /// </summary>
         private bool displayX264Options;
 
+        /// <summary>
+        /// Backing field used to display / hide the x265 options
+        /// </summary>
+        private bool displayX265Options;
+
+        /// <summary>
+        /// The display qsv options.
+        /// </summary>
+        private bool displayQSVOptions;
+
+        /// <summary>
+        /// Backing field used to display / hide the h264 options
+        /// </summary>
+        private bool displayEncoderOptions;
+
         /// <summary>
         /// The quality max.
         /// </summary>
@@ -88,9 +110,9 @@ namespace HandBrakeWPF.ViewModels
         private int x264PresetValue;
 
         /// <summary>
-        /// The extra arguments.
+        /// The qsv preset value.
         /// </summary>
-        private string extraArguments;
+        private int qsvPresetValue;
 
         /// <summary>
         /// The can clear tracker.
@@ -102,6 +124,21 @@ namespace HandBrakeWPF.ViewModels
         /// </summary>
         private bool useAdvancedTab;
 
+        /// <summary>
+        /// The display framerate controls.
+        /// </summary>
+        private bool displayNonQSVControls;
+
+        /// <summary>
+        /// The x265 preset value.
+        /// </summary>
+        private int x265PresetValue;
+
+        /// <summary>
+        /// The display turbo first pass.
+        /// </summary>
+        private bool displayTurboFirstPass;
+
         #endregion
 
         #region Constructors and Destructors
@@ -126,10 +163,15 @@ namespace HandBrakeWPF.ViewModels
             this.VideoEncoders = EnumHelper<VideoEncoder>.GetEnumList();
 
             X264Presets = new BindingList<x264Preset>(EnumHelper<x264Preset>.GetEnumList().ToList());
+            QsvPresets = new BindingList<QsvPreset>(EnumHelper<QsvPreset>.GetEnumList().ToList());
             H264Profiles = EnumHelper<x264Profile>.GetEnumList();
             X264Tunes = EnumHelper<x264Tune>.GetEnumList().Where(t => t != x264Tune.Fastdecode);
             this.H264Levels = Levels;
 
+            X265Presets = new BindingList<x265Preset>(EnumHelper<x265Preset>.GetEnumList().ToList());
+            H265Profiles = EnumHelper<x265Profile>.GetEnumList();
+            X265Tunes = EnumHelper<x265Tune>.GetEnumList();
+
             this.userSettingService.SettingChanged += this.UserSettingServiceSettingChanged;
         }
 
@@ -149,13 +191,17 @@ namespace HandBrakeWPF.ViewModels
         {
             get
             {
-                bool showAdvTabSetting =
-                    this.userSettingService.GetUserSetting<bool>(UserSettingConstants.ShowAdvancedTab);
+                bool showAdvTabSetting = this.userSettingService.GetUserSetting<bool>(UserSettingConstants.ShowAdvancedTab);
                 if (!showAdvTabSetting)
                 {
                     this.UseAdvancedTab = false;
                 }
 
+                if (this.SelectedVideoEncoder == VideoEncoder.QuickSync)
+                {
+                    return false;
+                }
+
                 return showAdvTabSetting;
             }
         }
@@ -173,6 +219,17 @@ namespace HandBrakeWPF.ViewModels
             {
                 if (!object.Equals(value, this.useAdvancedTab))
                 {
+                    // Set the Advanced Tab up with the current settings, if we can.
+                    if (value)
+                    {
+                        this.Task.AdvancedEncoderOptions = this.GetActualx264Query();
+                    }
+
+                    if (!value)
+                    {
+                        this.Task.AdvancedEncoderOptions = string.Empty;
+                    }
+
                     this.useAdvancedTab = value;
                     this.Task.ShowAdvancedTab = value;
                     this.NotifyOfPropertyChange(() => this.UseAdvancedTab);
@@ -292,7 +349,7 @@ namespace HandBrakeWPF.ViewModels
         {
             get
             {
-                return 51.Equals(this.RF);
+                return 0.0.Equals(this.DisplayRF) && this.SelectedVideoEncoder == VideoEncoder.X264;
             }
         }
 
@@ -347,20 +404,29 @@ namespace HandBrakeWPF.ViewModels
             {
                 this.rf = value;
 
-                double cqStep = userSettingService.GetUserSetting<double>(UserSettingConstants.X264Step);
                 this.SetQualitySliderBounds();
                 switch (this.SelectedVideoEncoder)
                 {
                     case VideoEncoder.FFMpeg:
                     case VideoEncoder.FFMpeg2:
                         this.Task.Quality = (32 - value);
+                        break;    
+                    case VideoEncoder.VP8:
+                        this.Task.Quality = (63 - value);
                         break;
                     case VideoEncoder.X264:
+                    case VideoEncoder.X265:
+                        double cqStep = userSettingService.GetUserSetting<double>(UserSettingConstants.X264Step);
                         double rfValue = 51.0 - value * cqStep;
                         rfValue = Math.Round(rfValue, 2);
                         this.Task.Quality = rfValue;
+                        break;           
+                    case VideoEncoder.QuickSync:
+                        rfValue = 51.0 - value;
+                        rfValue = Math.Round(rfValue, 0);
+                        this.Task.Quality = rfValue;
                         break;
-                    case VideoEncoder.Theora:
+                    case VideoEncoder.Theora:                 
                         Task.Quality = value;
                         break;
                 }
@@ -382,6 +448,40 @@ namespace HandBrakeWPF.ViewModels
             }
         }
 
+        /// <summary>
+        /// Gets or sets a value indicating whether two pass.
+        /// </summary>
+        public bool TwoPass
+        {
+            get
+            {
+                return this.Task.TwoPass;
+            }
+
+            set
+            {
+                this.Task.TwoPass = value;
+                this.NotifyOfPropertyChange(() => this.TwoPass);
+            }
+        }
+
+        /// <summary>
+        /// Gets or sets a value indicating whether turbo first pass.
+        /// </summary>
+        public bool TurboFirstPass
+        {
+            get
+            {
+                return this.Task.TurboFirstPass;
+            }
+
+            set
+            {
+                this.Task.TurboFirstPass = value;
+                this.NotifyOfPropertyChange(() => this.TurboFirstPass);
+            }
+        }
+
         /// <summary>
         /// Gets the rfqp.
         /// </summary>
@@ -389,7 +489,18 @@ namespace HandBrakeWPF.ViewModels
         {
             get
             {
-                return this.SelectedVideoEncoder == VideoEncoder.X264 ? "RF" : "QP";
+                return this.SelectedVideoEncoder == VideoEncoder.X264 || this.SelectedVideoEncoder == VideoEncoder.X265 ? "RF" : "QP";
+            }
+        }
+
+        /// <summary>
+        /// Gets the high quality label.
+        /// </summary>
+        public string HighQualityLabel
+        {
+            get
+            {
+                return this.SelectedVideoEncoder == VideoEncoder.X264 ? Resources.Video_PlaceboQuality : Resources.Video_HigherQuality;
             }
         }
 
@@ -458,7 +569,27 @@ namespace HandBrakeWPF.ViewModels
                 // Hide the x264 controls when not needed.
                 this.DisplayX264Options = value == VideoEncoder.X264;
 
+                // Hide the x265 controls when not needed.
+                this.DisplayX265Options = value == VideoEncoder.X265;
+
+                this.DisplayQSVOptions = value == VideoEncoder.QuickSync;
+                this.DisplayH264Options = value == VideoEncoder.X264 || value == VideoEncoder.QuickSync;
+                this.UseAdvancedTab = value != VideoEncoder.QuickSync && this.UseAdvancedTab;
+                this.DisplayNonQSVControls = value != VideoEncoder.QuickSync;
+                this.DisplayTurboFirstPass = value == VideoEncoder.X264; 
+
                 this.NotifyOfPropertyChange(() => this.Rfqp);
+                this.NotifyOfPropertyChange(() => this.ShowAdvancedTab);
+
+                this.NotifyOfPropertyChange(() => this.HighQualityLabel);
+                if (value == VideoEncoder.QuickSync)
+                {
+                    this.IsConstantFramerate = true;
+                    this.TwoPass = false;
+                    this.TurboFirstPass = false;
+                    this.Task.Framerate = null;
+                    this.NotifyOfPropertyChange(() => SelectedFramerate);
+                }
             }
         }
 
@@ -494,7 +625,7 @@ namespace HandBrakeWPF.ViewModels
             }
             set
             {
-                if (!object.Equals(this.Task.AdvancedEncoderOptions, value))
+                if (!object.Equals(this.Task.ExtraAdvancedArguments, value))
                 {
                     this.Task.ExtraAdvancedArguments = value;
                     this.NotifyOfPropertyChange(() => this.ExtraArguments);
@@ -503,6 +634,22 @@ namespace HandBrakeWPF.ViewModels
             }
         }
 
+        /// <summary>
+        /// Gets or sets a value indicating whether to display H264
+        /// </summary>
+        public bool DisplayH264Options
+        {
+            get
+            {
+                return this.displayEncoderOptions;
+            }
+            set
+            {
+                this.displayEncoderOptions = value;
+                this.NotifyOfPropertyChange(() => this.DisplayH264Options);
+            }
+        }
+
         /// <summary>
         /// Gets or sets a value indicating whether display x 264 options.
         /// </summary>
@@ -520,6 +667,56 @@ namespace HandBrakeWPF.ViewModels
             }
         }
 
+        /// <summary>
+        /// Gets or sets a value indicating whether display x 264 options.
+        /// </summary>
+        public bool DisplayX265Options
+        {
+            get
+            {
+                return this.displayX265Options;
+            }
+            set
+            {
+                this.displayX265Options = value;
+                this.NotifyOfPropertyChange(() => this.DisplayX265Options);
+                this.NotifyOfPropertyChange(() => FullOptionsTooltip);
+            }
+        }
+
+        /// <summary>
+        /// Gets or sets a value indicating whether to display qsv options.
+        /// </summary>
+        public bool DisplayQSVOptions
+        {
+            get
+            {
+                return this.displayQSVOptions;
+            }
+            set
+            {
+                this.displayQSVOptions = value;
+                this.NotifyOfPropertyChange(() => this.DisplayQSVOptions);
+            }
+        }
+
+        /// <summary>
+        /// Gets or sets a value indicating whether display framerate controls.
+        /// </summary>
+        public bool DisplayNonQSVControls
+        {
+            get
+            {
+                return this.displayNonQSVControls;
+            }
+
+            set
+            {
+                this.displayNonQSVControls = value;
+                this.NotifyOfPropertyChange(() => this.DisplayNonQSVControls);
+            }
+        }
+
         /// <summary>
         /// Gets or sets the x 264 preset value.
         /// </summary>
@@ -562,6 +759,45 @@ namespace HandBrakeWPF.ViewModels
             }
         }
 
+        /// <summary>
+        /// Gets or sets X264Preset.
+        /// </summary>
+        public QsvPreset QsvPreset
+        {
+            get
+            {
+                return this.Task.QsvPreset;
+            }
+            set
+            {
+                if (!object.Equals(this.QsvPreset, value))
+                {
+                    this.Task.QsvPreset = value;
+                    this.NotifyOfPropertyChange(() => this.QsvPreset);
+                }
+            }
+        }
+
+        /// <summary>
+        /// Gets or sets the x 264 preset value.
+        /// </summary>
+        public int QsvPresetValue
+        {
+            get
+            {
+                return this.qsvPresetValue;
+            }
+            set
+            {
+                if (!object.Equals(this.QsvPresetValue, value))
+                {
+                    this.qsvPresetValue = value;
+                    this.QsvPreset = this.QsvPresets[value];
+                    this.NotifyOfPropertyChange(() => this.QsvPresetValue);
+                }
+            }
+        }
+
         /// <summary>
         /// Gets or sets H264Profile.
         /// </summary>
@@ -652,6 +888,11 @@ namespace HandBrakeWPF.ViewModels
         /// </summary>
         public BindingList<x264Preset> X264Presets { get; set; }
 
+        /// <summary>
+        /// Gets or sets QsvPreset.
+        /// </summary>
+        public BindingList<QsvPreset> QsvPresets { get; set; }
+
         /// <summary>
         /// Gets or sets X264Profiles.
         /// </summary>
@@ -674,7 +915,138 @@ namespace HandBrakeWPF.ViewModels
         {
             get
             {
-                return string.Format(Resources.Video_x264ExtraArgs, this.GetActualx264Query()); // "You can provide additional arguments using the standard x264 format"; 
+                return string.Format(Resources.Video_EncoderExtraArgs, this.GetActualx264Query()); // "You can provide additional arguments using the standard x264 format"; 
+            }
+        }
+
+        /// <summary>
+        /// Gets the qsv slider max.
+        /// </summary>
+        public int QsvSliderMax
+        {
+            get
+            {
+                return SystemInfo.IsHswOrNewer ? 2 : 1;
+            }
+        }
+
+        /// <summary>
+        /// Gets or sets the x 265 preset value.
+        /// </summary>
+        public int X265PresetValue
+        {
+            get
+            {
+                return this.x265PresetValue;
+            }
+            set
+            {
+                if (!object.Equals(this.X265PresetValue, value))
+                {
+                    this.x265PresetValue = value;
+                    this.X265Preset = this.X265Presets[value];
+                    this.NotifyOfPropertyChange(() => this.x265PresetValue);
+                    this.NotifyOfPropertyChange(() => FullOptionsTooltip);
+                }
+            }
+        }
+
+        /// <summary>
+        /// Gets or sets X265Preset.
+        /// </summary>
+        public x265Preset X265Preset
+        {
+            get
+            {
+                return this.Task.X265Preset;
+            }
+            set
+            {
+                if (!object.Equals(this.X265Preset, value))
+                {
+                    this.Task.X265Preset = value;
+                    this.NotifyOfPropertyChange(() => this.X265Preset);
+                    ResetAdvancedTab();
+                    this.NotifyOfPropertyChange(() => FullOptionsTooltip);
+                }
+            }
+        }
+
+        /// <summary>
+        /// Gets or sets H265Profile.
+        /// </summary>
+        public x265Profile H265Profile
+        {
+            get
+            {
+                return this.Task.H265Profile;
+            }
+
+            set
+            {
+                if (!object.Equals(this.H265Profile, value))
+                {
+                    this.Task.H265Profile = value;
+                    this.NotifyOfPropertyChange(() => this.H265Profile);
+                    ResetAdvancedTab();
+                    this.NotifyOfPropertyChange(() => FullOptionsTooltip);
+                }
+            }
+        }
+
+        /// <summary>
+        /// Gets or sets X265Tune.
+        /// </summary>
+        public x265Tune X265Tune
+        {
+            get
+            {
+                return this.Task.X265Tune;
+            }
+            set
+            {
+                if (!object.Equals(this.X265Tune, value))
+                {
+                    this.Task.X265Tune = value;
+                    this.NotifyOfPropertyChange(() => this.X265Tune);
+                    ResetAdvancedTab();
+                    this.NotifyOfPropertyChange(() => FullOptionsTooltip);
+                }
+            }
+        }
+
+        /// <summary>
+        /// Gets or sets X265Presets.
+        /// </summary>
+        public BindingList<x265Preset> X265Presets { get; set; }
+
+        /// <summary>
+        /// Gets or sets X265Profiles.
+        /// </summary>
+        public IEnumerable<x265Profile> H265Profiles { get; set; }
+
+        /// <summary>
+        /// Gets or sets X265Tunes.
+        /// </summary>
+        public IEnumerable<x265Tune> X265Tunes { get; set; }
+
+        /// <summary>
+        /// Gets or sets a value indicating whether display turbo first pass.
+        /// </summary>
+        public bool DisplayTurboFirstPass
+        {
+            get
+            {
+                return this.displayTurboFirstPass;
+            }
+            set
+            {
+                if (value.Equals(this.displayTurboFirstPass))
+                {
+                    return;
+                }
+                this.displayTurboFirstPass = value;
+                this.NotifyOfPropertyChange(() => this.DisplayTurboFirstPass);
             }
         }
 
@@ -736,42 +1108,10 @@ namespace HandBrakeWPF.ViewModels
                     break;
             }
 
-            double cqStep = userSettingService.GetUserSetting<double>(UserSettingConstants.X264Step);
-            double rfValue = 0;
-            this.SetQualitySliderBounds();
-            switch (this.SelectedVideoEncoder)
-            {
-                case VideoEncoder.FFMpeg:
-                case VideoEncoder.FFMpeg2:
-                    if (preset.Task.Quality.HasValue)
-                    {
-                        int cq;
-                        int.TryParse(preset.Task.Quality.Value.ToString(CultureInfo.InvariantCulture), out cq);
-                        this.RF = 32 - cq;
-                    }
-                    break;
-                case VideoEncoder.X264:
-
-                    double multiplier = 1.0 / cqStep;
-                    if (preset.Task.Quality.HasValue)
-                    {
-                        rfValue = preset.Task.Quality.Value * multiplier;
-                    }
-
-                    this.RF = this.QualityMax - (int)Math.Round(rfValue, 0);
-
-                    break;
-
-                case VideoEncoder.Theora:
-                    if (preset.Task.Quality.HasValue)
-                    {
-                        this.RF = (int)preset.Task.Quality.Value;
-                    }
-                    break;
-            }
-
-            this.Task.TwoPass = preset.Task.TwoPass;
-            this.Task.TurboFirstPass = preset.Task.TurboFirstPass;
+            this.SetRF(preset.Task.Quality);
+         
+            this.TwoPass = preset.Task.TwoPass;
+            this.TurboFirstPass = preset.Task.TurboFirstPass;
             this.Task.VideoBitrate = preset.Task.VideoBitrate;
 
             this.NotifyOfPropertyChange(() => this.Task);
@@ -779,15 +1119,61 @@ namespace HandBrakeWPF.ViewModels
             if (preset.Task != null)
             {
                 this.SetEncoder(preset.Task.VideoEncoder);
-                this.X264PresetValue = preset.Task.VideoEncoder == VideoEncoder.X264
-                                           ? (int)preset.Task.X264Preset
-                                           : (int)x264Preset.Medium;
-                this.H264Profile = preset.Task.VideoEncoder == VideoEncoder.X264 ? preset.Task.H264Profile : x264Profile.None;
-                this.X264Tune = preset.Task.VideoEncoder == VideoEncoder.X264 ? preset.Task.X264Tune : x264Tune.None;
-                this.H264Level = preset.Task.VideoEncoder == VideoEncoder.X264 ? preset.Task.H264Level : "Auto";
-                this.FastDecode = preset.Task.VideoEncoder == VideoEncoder.X264 && preset.Task.FastDecode;
-                this.ExtraArguments = preset.Task.ExtraAdvancedArguments;
 
+                // applies to both x264 and QSV
+                if (preset.Task.VideoEncoder == VideoEncoder.X264 ||
+                    preset.Task.VideoEncoder == VideoEncoder.QuickSync)
+                {
+                    this.H264Profile = preset.Task.H264Profile;
+                    this.H264Level = preset.Task.H264Level;
+                }
+                else
+                {
+                    this.H264Profile = x264Profile.None;
+                    this.H264Level = "Auto";
+                }
+
+                // x264 Only
+                if (preset.Task.VideoEncoder == VideoEncoder.X264)
+                {
+                    this.X264PresetValue = (int)preset.Task.X264Preset;
+                    this.X264Tune = preset.Task.X264Tune;
+                    this.FastDecode = preset.Task.FastDecode;
+                }
+                else
+                {
+                    this.X264PresetValue = (int)x264Preset.Medium;
+                    this.X264Tune = x264Tune.None;
+                    this.FastDecode = false;
+                }
+
+                // x265 Only
+                if (preset.Task.VideoEncoder == VideoEncoder.X265)
+                {
+                    this.X265PresetValue = (int)preset.Task.X265Preset;
+                    this.X265Tune = preset.Task.X265Tune;
+                    this.H265Profile = preset.Task.H265Profile;
+                }
+                else
+                {
+                    this.X265PresetValue = (int)x265Preset.VeryFast;
+                    this.X265Tune = x265Tune.None;
+                    this.H265Profile = x265Profile.None;
+                }
+
+                // QSV Only
+                if (preset.Task.VideoEncoder == VideoEncoder.QuickSync)
+                {
+                    this.QsvPresetValue = (int)preset.Task.QsvPreset;
+                }
+                else
+                {
+                    this.QsvPresetValue = SystemInfo.IsHswOrNewer
+                                              ? (int)QsvPreset.Quality
+                                              : (int)QsvPreset.Balanced;
+                }
+
+                this.ExtraArguments = preset.Task.ExtraAdvancedArguments;
                 this.UseAdvancedTab = !string.IsNullOrEmpty(preset.Task.AdvancedEncoderOptions) && this.ShowAdvancedTab;
             }
         }
@@ -801,15 +1187,21 @@ namespace HandBrakeWPF.ViewModels
         public void UpdateTask(EncodeTask task)
         {
             this.Task = task;
+            this.SetRF(task.Quality);
+
             this.NotifyOfPropertyChange(() => this.IsConstantFramerate);
             this.NotifyOfPropertyChange(() => this.IsConstantQuantity);
             this.NotifyOfPropertyChange(() => this.IsPeakFramerate);
             this.NotifyOfPropertyChange(() => this.IsVariableFramerate);
             this.NotifyOfPropertyChange(() => this.SelectedVideoEncoder);
             this.NotifyOfPropertyChange(() => this.SelectedFramerate);
+            this.NotifyOfPropertyChange(() => this.QualityMax);
+            this.NotifyOfPropertyChange(() => this.QualityMin);
             this.NotifyOfPropertyChange(() => this.RF);
             this.NotifyOfPropertyChange(() => this.DisplayRF);
+            this.NotifyOfPropertyChange(() => this.IsLossless);
             this.NotifyOfPropertyChange(() => this.Task.VideoBitrate);
+            this.NotifyOfPropertyChange(() => this.Task.Quality);
             this.NotifyOfPropertyChange(() => this.Task.TwoPass);
             this.NotifyOfPropertyChange(() => this.Task.TurboFirstPass);
 
@@ -819,6 +1211,11 @@ namespace HandBrakeWPF.ViewModels
             this.NotifyOfPropertyChange(() => this.H264Profile);
             this.NotifyOfPropertyChange(() => this.FastDecode);
             this.NotifyOfPropertyChange(() => this.ExtraArguments);
+            this.NotifyOfPropertyChange(() => this.QsvPreset);
+
+            this.NotifyOfPropertyChange(() => this.H265Profile);
+            this.NotifyOfPropertyChange(() => this.X265Preset);
+            this.NotifyOfPropertyChange(() => this.X265Tune);
         }
 
         /// <summary>
@@ -829,7 +1226,16 @@ namespace HandBrakeWPF.ViewModels
         /// </param>
         public void SetEncoder(VideoEncoder encoder)
         {
+            this.DisplayH264Options = encoder == VideoEncoder.X264 || encoder == VideoEncoder.QuickSync;
             this.DisplayX264Options = encoder == VideoEncoder.X264;
+            this.DisplayQSVOptions = encoder == VideoEncoder.QuickSync;
+            this.DisplayX265Options = encoder == VideoEncoder.X265;
+            this.DisplayTurboFirstPass = encoder == VideoEncoder.X264;
+
+            if (encoder == VideoEncoder.QuickSync)
+            {
+                this.UseAdvancedTab = false;
+            }
         }
 
         /// <summary>
@@ -839,7 +1245,7 @@ namespace HandBrakeWPF.ViewModels
         {
             this.NotifyOfPropertyChange(() => this.Task);
 
-            if (Task.OutputFormat == OutputFormat.Mp4 && this.SelectedVideoEncoder == VideoEncoder.Theora)
+            if ((Task.OutputFormat == OutputFormat.Mp4) && this.SelectedVideoEncoder == VideoEncoder.Theora)
             {
                 this.SelectedVideoEncoder = VideoEncoder.X264;
             }
@@ -858,6 +1264,18 @@ namespace HandBrakeWPF.ViewModels
             this.H264Level = "Auto";
             this.ExtraArguments = string.Empty;
             this.canClear = true;
+
+            this.H265Profile = x265Profile.None;
+            this.x265PresetValue = 1;
+            this.X265Tune = x265Tune.None;
+        }
+
+        /// <summary>
+        /// The copy query.
+        /// </summary>
+        public void CopyQuery()
+        {
+            Clipboard.SetDataObject(this.SelectedVideoEncoder == VideoEncoder.X264 ? this.GetActualx264Query() : this.ExtraArguments);
         }
 
         #endregion
@@ -876,11 +1294,17 @@ namespace HandBrakeWPF.ViewModels
                     this.QualityMin = 1;
                     this.QualityMax = 31;
                     break;
+                case VideoEncoder.QuickSync:
+                    this.QualityMin = 0;
+                    this.QualityMax = 51;
+                    break;
                 case VideoEncoder.X264:
+                case VideoEncoder.X265:
                     this.QualityMin = 0;
                     this.QualityMax = (int)(51 / userSettingService.GetUserSetting<double>(UserSettingConstants.X264Step));
                     break;
                 case VideoEncoder.Theora:
+                case VideoEncoder.VP8:
                     this.QualityMin = 0;
                     this.QualityMax = 63;
                     break;
@@ -906,7 +1330,7 @@ namespace HandBrakeWPF.ViewModels
         /// </returns>
         private string GetActualx264Query()
         {
-            if (this.userSettingService.GetUserSetting<bool>(UserSettingConstants.DisableLibHbFeatures))
+            if (!GeneralUtilities.IsLibHbPresent)
             {
                 return string.Empty; // Feature is disabled.
             }
@@ -938,8 +1362,21 @@ namespace HandBrakeWPF.ViewModels
                 width = 720;
             }
 
-            // TODO figure out what is wrong with this??
-            return HandBrakeUtils.CreateX264OptionsString(preset, tunes, this.ExtraArguments, profile, this.H264Level, width, height);
+            try
+            {
+                return HandBrakeUtils.CreateX264OptionsString(
+                    preset,
+                    tunes,
+                    this.ExtraArguments,
+                    profile,
+                    this.H264Level,
+                    width,
+                    height);
+            }
+            catch (Exception)
+            {
+                return "Error: Libhb not loaded.";
+            }
         }
 
         /// <summary>
@@ -958,5 +1395,63 @@ namespace HandBrakeWPF.ViewModels
                 this.NotifyOfPropertyChange(() => this.ShowAdvancedTab);
             }
         }
+
+        /// <summary>
+        /// The set rf.
+        /// </summary>
+        /// <param name="quality">
+        /// The quality.
+        /// </param>
+        private void SetRF(double? quality)
+        {
+            double cqStep = userSettingService.GetUserSetting<double>(UserSettingConstants.X264Step);
+            double rfValue = 0;
+            this.SetQualitySliderBounds();
+            switch (this.SelectedVideoEncoder)
+            {
+                case VideoEncoder.FFMpeg:
+                case VideoEncoder.FFMpeg2:
+                    if (quality.HasValue)
+                    {
+                        int cq;
+                        int.TryParse(quality.Value.ToString(CultureInfo.InvariantCulture), out cq);
+                        this.RF = 32 - cq;
+                    }
+                    break;
+                case VideoEncoder.VP8:
+                    if (quality.HasValue)
+                    {
+                        int cq;
+                        int.TryParse(quality.Value.ToString(CultureInfo.InvariantCulture), out cq);
+                        this.RF = 63 - cq;
+                    }
+                    break;
+                case VideoEncoder.X265:
+                case VideoEncoder.X264:
+                case VideoEncoder.QuickSync:
+
+                    if (this.SelectedVideoEncoder == VideoEncoder.QuickSync)
+                    {
+                        cqStep = 1;
+                    }
+                    double multiplier = 1.0 / cqStep;
+                    if (quality.HasValue)
+                    {
+                        rfValue = quality.Value * multiplier;
+                    }
+
+                    this.RF = this.QualityMax - (int)Math.Round(rfValue, 0);
+
+                    break;
+
+                case VideoEncoder.Theora:
+
+                    if (quality.HasValue)
+                    {
+                        this.RF = (int)quality.Value;
+                    }
+                    break;
+            }
+        }
     }
 }
\ No newline at end of file
