@@ -1,23 +1,21 @@
 ﻿// --------------------------------------------------------------------------------------------------------------------
-// <copyright file="AdvancedView.xaml.cs" company="HandBrake Project (http://handbrake.fr)">
+// <copyright file="X264View.xaml.cs" company="HandBrake Project (http://handbrake.fr)">
 //   This file is part of the HandBrake source code - It may be used under the terms of the GNU General Public License.
 // </copyright>
 // <summary>
-//   Interaction logic for AdvancedView.xaml
+//   Interaction logic for X264View.xaml
 // </summary>
 // --------------------------------------------------------------------------------------------------------------------
 
 namespace HandBrakeWPF.Views
 {
-    using System.Windows.Controls;
-
     /// <summary>
     /// Interaction logic for AdvancedView.xaml
     /// </summary>
-    public partial class X264View : UserControl
+    public partial class X264View
     {
         /// <summary>
-        /// Initializes a new instance of the <see cref="AdvancedView"/> class.
+        /// Initializes a new instance of the <see cref="X264View"/> class. 
         /// </summary>
         public X264View()
         {
